# Two whirls of golden spirals, winding in opposite the directions.
# This is an example of building 'pcurve_collage' from several parametric
# curves and drawing them simultaneously with the same settings.
# Note: A more efficient way to do this would be to compute one spiral only
# and then to use some suitable piece of code to make rotated copies of it.
# Here we let the plugin compute the spirals one by one.
# This is slow.

from functools import partial

def golden_spiral_general(t, rotate_angle, reflect=False): # rotate_angle in degrees
    from math import sin,cos,exp,pi
    angle_rad = rotate_angle*pi/180
    b = 0.30634896253003313 # (2/pi)*log((1+sqrt(5))/2)
    if reflect:
        r = exp(b*t)
    else:
        r = exp(-b*t)
    u = t - angle_rad
    return [r*cos(u), r*sin(u)]

N = 6

# We use Python's list comprehension to build a list of spirals,
# where the spirals are similar except for rotation.
spirals_ccw = [ParametricCurve(
    function = partial(golden_spiral_general, rotate_angle=i*360./N),
    pcurve_name = "rotated "+str(i*360./N)+" degrees",
    start  = -3*pi,
    end    = 3*pi,
    closed = False,
    custom = None
    ) for i in range(N)]

# Spirals winding in opposite direction
spirals_cw = [ParametricCurve(
    function = partial(golden_spiral_general, rotate_angle=i*360./N, reflect=True),
    pcurve_name = "rotated "+str(i*360./N)+" degrees",
    start  = -3*pi,
    end    = 3*pi,
    closed = False,
    custom = None
    ) for i in range(N)]

pcurvelist = spirals_ccw + spirals_cw

c2bs = Curve2BezierSettings()    # default

c2bo = Curve2BezierOptions(
    cusp_points           = False,
    high_curvature_points = False,
    low_curvature_points  = False,
    straight_end_points   = False,
    inflection_points     = False,
    angle_turn_points     = True # This is the only needed for the spiral
    )

coos = CoordinateSettings()      # default

drawo = DrawingOptions(
    bezier_curves  = True,
    bezier_collage = True, # Makes a merged-down path.
    )

pcurve_collage = ParametricCurveCollage(
    pcurve_list = pcurvelist,
    collage_name = 'Whirls of golden spirals'
    )

