# Spirograph: hypotrochoid and epitrochoid, and corresponding fixed circle.
# This is really slow!
#
# Formulas:
#
# Weisstein, Eric W. "Spirograph." From MathWorld--A Wolfram Web Resource.
# http://mathworld.wolfram.com/Spirograph.html
#
# Weisstein, Eric W. "Hypotrochoid." From MathWorld--A Wolfram Web Resource.
# http://mathworld.wolfram.com/Hypotrochoid.html
#
# Weisstein, Eric W. "Epitrochoid." From MathWorld--A Wolfram Web Resource.
# http://mathworld.wolfram.com/Epitrochoid.html 


############# Parameters: edit to get different curves ############################
fixed = 30  # Relative radius of fixed circle, integer.
roll  = 11  # Relative radius of rolling circle, integer.
dist  = .9  # Distance of pen from center of rolling circle relative to its radius.
sc = 1.     # Scaling factor, float.
rotate = 0. # Rotation in degrees, float.
###################################################################################

# Greatest common divisor.
def gcd(m,n):
    if m==0 and n==0:
        return None
    x,y = abs(m),abs(n)
    if x<y:
        x,y = y,x
    while y != 0:
        x,y = y, x%y
    return x

# a,b: Integers
# Find a/b = x/y where gcd(x,y)=1 and y>=0.
# Return None if a=b=0.
def reduce_fraction(a,b):
    if a==0 and b==0:
        return None
    if b == 0:
        return 1,0
    if a == 0:
        return 0,1
    if b<0:
        a,b = -a,-b
    d = gcd(a,b)
    return a//d, b//d

def make_hypotrochoid_function(
    fixed_circle,   # integer
    rolling_circle, # integer
    distance, # float, distance of pen from center of rolling circle relative to radius
    scale=1., # float
    rotate=0. # float
    ):
    from math import pi,cos,sin
    c,s = cos(rotate*pi/180), sin(rotate*pi/180)
    def hypotrochoid_function(t):
        from math import cos,sin
        R,r,H = fixed_circle, rolling_circle, distance*rolling_circle
        q = float(R)/float(r)
        x = (R-r)*cos(t) + H * cos((q-1)*t)
        y = (R-r)*sin(t) - H * sin((q-1)*t)
        x, y = c*x - s*y, s*x + c*y # rotation
        return [scale*x, scale*y]
    N,n = reduce_fraction(fixed_circle, rolling_circle)
    Q = float(n)/float(N)
    interval = [0., 2*n*pi]                 # Start, end for one closed period.
    maxima = [2*k*Q*pi for k in range(1,N)] # Excludes start, end.
    minima = [(2*k+1)*Q*pi for k in range(N)]
    return hypotrochoid_function, interval, maxima, minima

def make_epitrochoid_function(
    fixed_circle,   # integer
    rolling_circle, # integer
    distance, # float, distance from center of rolling circle relative to radius
    scale=1., # float
    rotate=0. # float
    ):
    from math import pi,cos,sin
    c,s = cos(rotate*pi/180), sin(rotate*pi/180)
    def epitrochoid_function(t):
        from math import cos,sin
        R,r,H = fixed_circle, rolling_circle, distance*rolling_circle
        q = float(R)/float(r)
        x = (R+r)*cos(t) - H * cos((q+1)*t)
        y = (R+r)*sin(t) - H * sin((q+1)*t)
        x, y = c*x - s*y, s*x + c*y # rotation
        return [scale*x, scale*y]
    N,n = reduce_fraction(fixed_circle, rolling_circle)
    Q = float(n)/float(N)
    interval = [0., 2*n*pi]                 # start, end
    minima = [2*k*Q*pi for k in range(1,N)] # excluding start, end
    maxima = [(2*k+1)*Q*pi for k in range(N)]
    return epitrochoid_function, interval, maxima, minima

def make_fixed_circle_function(fixed_circle, scale=1.):
    def fixed_circle_function(t):
        from math import cos,sin
        radius = fixed_circle * scale
        return [radius*cos(t), radius*sin(t)]
    interval = [0.,2*pi]
    return fixed_circle_function, interval

hypotrochoid, hinterval, hmx, hmn = make_hypotrochoid_function(fixed, roll, dist, sc, rotate)
epitrochoid,  einterval, emx, emn = make_epitrochoid_function(fixed, roll, dist, sc, rotate)
fixed_circle, finterval = make_fixed_circle_function(fixed, sc)

hpcurve = ParametricCurve(
    function = hypotrochoid,
    pcurve_name = "hypotrochoid "+str(fixed)+", "+str(roll)+", "+str(dist)+", "+str(sc)+", "+str(rotate),
    start  = hinterval[0],
    end    = hinterval[1],
    closed = True,
    custom = CustomPoints(params = hmx + hmn)
    )

epcurve = ParametricCurve(
    function = epitrochoid,
    pcurve_name = "epitrochoid "+str(fixed)+", "+str(roll)+", "+str(dist)+", "+str(sc)+", "+str(rotate),
    start  = einterval[0],
    end    = einterval[1],
    closed = True,
    custom = CustomPoints(params = emx + emn)
    )

fpcurve = ParametricCurve(
    function = fixed_circle,
    pcurve_name = "fixed circle",
    start  = finterval[0],
    end    = finterval[1],
    closed = True,
    )

pcurve_collage = ParametricCurveCollage(
    pcurve_list  = [hpcurve, epcurve, fpcurve], # Edit if you don't want all three curves.
    collage_name = 'Spirograph'
    )

c2bs = Curve2BezierSettings(
    #refinement = 1,               # Uncomment to force more handles.
    )

c2bo = Curve2BezierOptions(
    #high_curvature_points = False # Experiment: Comment out (more handles).
    )

coos = CoordinateSettings(
    fit_window = True,
    )

drawo = DrawingOptions()

