# This file is a valid file as advanced inputs, with helpful comments.
#
# Substitute your own inputs in the following. (Or better, save this file
# and make a copy for your own experiments.)
# Keep the indentations! This file is supposed to be valid Python code.
#
# Then pick the file in Gimp's GUI for Parametric curves.
#
# (For more detailed explanations of the meanings of the settings,
# please see the file doc.pdf.)

# First, the parametric functions, implemented as a Python functions:

def small_circle(t):
    return [cos(t)/2, sin(t)/2]

def big_circle(t):
    return [cos(t), sin(t)]

def astroid(t):
    return [cos(t)**3, sin(t)**3]

# From the parametric functions we build the parametric curves:

pcurve0 = ParametricCurve(
    function    = small_circle,
    pcurve_name = "small circle",
    start  = 0.,
    end    = 2*pi,
    closed = True,
    )

pcurve1 = ParametricCurve(
    function    = big_circle,
    pcurve_name = "big circle",
    start  = 0.,
    end    = 2*pi,
    closed = True,
    )

pcurve2 = ParametricCurve(
    function    = astroid,
    pcurve_name = "astroid",
    start  = 0.,
    end    = 2*pi,
    closed = True,
    custom = None # One might wish to give the cusp points as custom points.
    )

# And from the parametric curves we build a collage:

pcurve_collage = ParametricCurveCollage(
    pcurve_list  = [pcurve0, pcurve1, pcurve2],
    collage_name = 'circle, circle, astroid'
    )

# In the following settings, if a field is omitted, some default value
# is used; it may or may not work well.

# Curve2BezierSettings determine how the program computes the subdivision.
#
# The following are default values, and they should work well.

c2bs = Curve2BezierSettings(
    angle         = pi/2, # Maximum allowed turn in each subdivision arc.
    refinement    = 0,    # Higher refinement forces over-all refinement.
    max_error     = 0.01, # Maximum allowed relative error in each arc.
    max_recursion = 3,    # Maximum allowed recursion for troublesome arcs.
    )

# Curve2BezierOptions determine what kinds of points of the curve must be
# included as anchor points of the resulting Bezier curve.
#
# The following are default values, and they should work well.

c2bo = Curve2BezierOptions(
    cusp_points           = True, # Include cusp points as anchor points?
    high_curvature_points = True, # Include points of locally lowest curvature?
    low_curvature_points  = True, # Include points of locally highest curvature?
    straight_end_points   = True, # Include start and end points of straight segments?
    inflection_points     = False,# Include inflection points?
    angle_turn_points     = True  # Include points to restrict the amounts of turns in arcs?
    )

# CoordinateSettings determine how the curve will be set in the window.
#
# The following are default values.
# If 'fit_window' is set to True, then of the other settings only 'padding'
# has any effect.

coos = CoordinateSettings(
    fit_window   = True, # Is the plot drawn using the whole window minus paddings?
    padding      = 0.15, # The width of padding as a fraction of window size.
    origo = [600.,450.], # The origo. (Ignored if fit_window=True.)
    scale = 450.         # The scaling factor to Gimp's coordinates.  (Ignored if
    #                    # fit_window=True.)
    )

# DrawingOptions determine which objects will be drawn:
# The Bezier curve and the axes are drawn by default.
# The option 'bezier_collage' means that, from the various parametric curves
# in 'pcurve_collage', the program constructs one Gimp's vectors object and
# draws it.
# If one wishes to check visually the resulting Bezier curves,
# one can let the program draw (as separate Gimp's vectors objects)
# - the exact input curve as a sparse set of of points (set 'exact_pcurves'=True);
# - the anchors and handles of the calculated Bezier curve (set 'handles'=True).
#
# The following are default values.

drawo = DrawingOptions(
    bezier_curves  = True,  # Draw each Bezier curve individually?
    bezier_collage = False, # Draw the whole collage together?
    axes           = True,  # Mark the coordinate axes as guides?
    x_axis         = False, # Mark the x axis as a guide?
    y_axis         = False, # Mark the y axis as a guide?
    exact_pcurves  = False, # Sketch the exact input curve with sparse points?
    handles        = False, # Draw the anchors and handles too?
    )

s2 = sqrt(2) # Introducing extra names is ok
is2 = 1/s2

mark_polylines = [
                  # Mark origo (as an anchor of the path 'polylines'):
                  [[0,0]],
                  # Square around big circle, closed polygon
                  # ('True' makes it closed):
                  [[s2,0],[0,s2],[-s2,0],[0,-s2],True],
                  # Two wedges, just for demonstration (open polylines):
                  [[is2,is2], [0,-is2/2], [-is2,is2]],
                  [[is2,-is2], [0,is2/2], [-is2,-is2], False], # False is default
                 ]
