#!/usr/bin/env python
#
# Gimp plugin.
# Draw parametric curves as approximate Bezier curves.

# (c) Markku Koppinen 2020

# History:
# 
# v1.08 2020-03-18: Initial published version
# v1.09 2020-04-14: Small bug fixes


#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published
#   by the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This very file is the complete source code to the program.
#
#   If you make and redistribute changes to this code, please mark it
#   in reasonable ways as different from the original version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   The GPL v3 licence is available at: https://www.gnu.org/licenses/gpl-3..en.html'


from __future__ import division, print_function 
from gimpfu import *                            
from math import *                              
from copy import copy                           
from functools import partial                   


#==============================================================
#==============================================================
#                          BCurve                              
#==============================================================
#==============================================================
class BCurve(object):
    """Data structures to handle Gimp vectors objects by means of individual
    Bezier arcs.
    By a "Bezier arc" we mean here a Bezier curve determined by 4 control points
    p0,p1,p2,p3.
    This class contains data structures needed to do computations on
    a Bezier curve which are done arc by arc.
    To this end, Gimp data structures
    - gimp.Vectors
    - gimp.VectorsBezierStroke
    are here represented as data structures (classes)
    - GimpVectors
    - GimpStroke
    - BezierCurve
    - BezierArc
    - ControlPoint
    with the aim that
    - each control point [x,y] is represented as an instance of ControlPoint;
    - each Bezier arc (four control points, see above) is represented as an instance
      of BezierArc.
    """
    class ControlPoint(object):
        """Control point as 2-item list [x,y]. Handle just as a list.
        """
        def __init__(self,x,y): # x,y: float
            self._xy = [x,y]
        def __getitem__(self,i):
            try:
                return self._xy[i]
            except IndexError as e:
                raise Exception("ControlPoint get: "+str(e))
        def __setitem__(self,i,x):
            try:
                self._xy[i] = x
            except IndexError as e:
                raise Exception("ControlPoint set: "+str(e))
        def __str__(self):
            def str_float(x): return '{:1.3f}'.format(x)
            return str([str_float(self[0]),str_float(self[1])])
    
    class GimpVectors(object):
        """Essentially same data as in a gimp.Vectors object except that
        strokes are instances of GimpStroke rather than
        gimp.VectorsBezierStroke.
        Attributes:
        - stroke_list: [GimpStroke]
        - name:        string
        """
        def __init__(self,
                     stroke_list,            # [GimpStroke]
                     name='GimpVectors'      # string
                     ):
            self.stroke_list = stroke_list
            self.name = name
        def __str__(self):
            s = 'GimpVectors '+self.name
            count = 0
            for stroke in self.stroke_list:
                s += '\n  Stroke '+str(count)+': GimpStroke'
                s += '\n    '+str(stroke)
                count += 1
            return s
        def gv2vectors_object(self, image, name=None):
            """Conversion GimpVectors -> gimp.Vectors
            """
            def cp_list2xy_list(cp_list):
                xy = []
                for cp in cp_list:
                    xy += [cp[0], cp[1]]
                return xy
            if name is None:
                name = self.name
            vectors = pdb.gimp_vectors_new(image, name)
            for stroke in self.stroke_list:
                closed = stroke.closed
                xy_list = cp_list2xy_list(stroke.cp_list)
                stroke_id = pdb.gimp_vectors_stroke_new_from_points(
                             vectors, 0, len(xy_list), xy_list, closed)
            return vectors
    
    class GimpStroke(object):
        """Essentially same data as in a gimp.VectorsBezierStroke.points,
        except that control points list [x,y,x,y,...] is arranged as
        a list [ControlPoint] by joining successive x,y.
        Attributes:
        - cp_list:     [ControlPoint]
        - closed:      boolean
        - stroke_name: string
        """
        def __init__(self,
                     cp_list,                   # [ControlPoint]
                     closed,                    # boolean
                     stroke_name = 'GimpStroke' # string
                     ):
            self.stroke_name = stroke_name
            self.cp_list = cp_list
            self.closed = closed
        def __str__(self):
            s = self.stroke_name + '; Control points:'
            for cp in self.cp_list:
                s += '\n    '+str(cp)
            s += '\n    closed: '+str(self.closed)
            return s
        def gs2bc(self):
            """Conversion GimpStroke -> BezierCurve
            """
            head = self.cp_list[0]
            tail = self.cp_list[-1]
            inner = self.cp_list[1:-1]
            ba_list = []
            count = 0
            for i in range(0,len(inner)-1,3):
                ba_list.append(BCurve.BezierArc(inner[i:i+4], 'arc '+str(count)))
                count += 1
            return BCurve.BezierCurve(
                               bezier_arcs = ba_list,
                               head_handle = head,
                               tail_handle = tail,
                               closed = self.closed,
                               curve_name = self.stroke_name
                               )
    
    class BezierCurve(object):
        """BezierCurve is a list of butting Bezier arcs with some extra data.
        Attributes:
        - curve_name:  string
        - bezier_arcs: [BezierArc];
        - head_handle: ControlPoint;
        - tail_handle: ControlPoint;
        - closed:      boolean.
        Note: "Butting" means that for any pair of successive arcs, the last
              control point of the former equals the first control point of
              the latter.
              In initialization no checks are done about this condition.
        """
        def __init__(self,
                     bezier_arcs=[],           # [BezierArc]
                     head_handle=None,         # ControlPoint
                     tail_handle=None,         # ControlPoint
                     closed=False,             # boolean
                     curve_name='BezierCurve', # string
                     ):
            self.curve_name = curve_name
            self.bezier_arcs = bezier_arcs
            if head_handle is None:
                self.head_handle = bezier_arcs[0].cp4[0]
            else:
                self.head_handle = head_handle
            if tail_handle is None:
                self.tail_handle = bezier_arcs[-1].cp4[-1]
            else:
                self.tail_handle = tail_handle
            self.closed = closed
        def __str__(self):
            s = self.curve_name + ': Bezier curve' + '; Bezier arcs:'
            count = 0
            for arc in self.bezier_arcs:
                s += '\n  arc '+ str(count)+ ': ' +str(arc)
                count += 1
            s += '\n  Head handle:    '+str(self.head_handle)
            s += '\n  Tail handle:    '+str(self.tail_handle)
            s += '\n  Closed:'    +str(self.closed)
            return s
        def bc2gs(self):
            """Conversion BezierCurve -> GimpStroke
            """
            cp_list = [self.head_handle]
            for ba in self.bezier_arcs:
                cp_list += ba.cp4[:3]
            cp_list.append(self.bezier_arcs[-1].cp4[-1])
            cp_list.append(self.tail_handle)
            return BCurve.GimpStroke(cp_list, self.closed, self.curve_name)
        def bc2vectors_object(self, image, name=None):
            """Conversion BezierCurve -> gimp.Vectors
            Makes vectors_object with 1 stroke.
            """
            if name is None:
                name = self.curve_name
            gs = self.bc2gs()             # BCurve.GimpStroke
            gv = BCurve.GimpVectors([gs]) # BCurve.GimpVectors
            return gv.gv2vectors_object(image, name)
    
    class BezierArc(object):
        """Data structure for one Bezier arc: arc determined by 4 control
           points.
        Attributes:
        - cp4:      [p0,p1,p2,p3] where each pi:ControlPoint
        - arc_name: string
        """
        def __init__(self,
                     cp4,                 # cp4=[p0,p1,p2,p3] with pi:ControlPoint
                     arc_name='BezierArc' # string
                     ):
            self.cp4 = cp4
            self.arc_name = arc_name
        def reverse(self):
            self.cp4.reverse()
        def __str__(self):
            s = 'BezierArc, control points:'
            for cp in self.cp4:
                s += '\n  '+ str(cp)
            return s

# -----------------
# vectors_object2gv
# -----------------
# Conversion gimp.Vectors -> GimpVectors
# Args:
# - vectors_object: gimp.Vectors (Gimp vectors object)
# Returns:
# - GimpVectors
#
# Note: The inverse conversion is GimpVectors.gv2vectors_object(...).
#       See also BezierCurve.bc2vectors_object(...).
def vectors_object2gv(vectors_object):
    def xy_list2cp_list(xy_list): # xy_list = [x,y,x,y,...,]
        cp = []
        for i in range(0,len(xy_list),2):
            cp.append(BCurve.ControlPoint(xy_list[i], xy_list[i+1]))
        return cp
    stroke_list = []
    count = 0
    for stroke in vectors_object.strokes:
        xy_list = stroke.points[0]
        closed = stroke.points[1]
        cp_list = xy_list2cp_list(xy_list)
        stroke_name = vectors_object.name + ' stroke '+str(count)
        stroke_list.append(BCurve.GimpStroke(cp_list, closed, stroke_name))
        count += 1
    return BCurve.GimpVectors(stroke_list, vectors_object.name)


#==============================================================
#==============================================================
#                       Supported family                       
#==============================================================
#==============  This part should be imported  ================
class Support(object):
    """Data structure to store "support" for certain Bezier arcs.
    By a "support" we mean non-collinear points S0,S1,S2,S3 where it is
    assumed that S0 != S1 and S3 != S2.
    This "supports" a Bezier arc with control points p0,p1,p2,p3 if
        - p0 = S0
        - p1 is on the line through S0,S1
        - p2 is on the line through S3,S2
        - p3 = S3
    Attributes:
    - support_points = [S0,S1,S2,S3] where Si:[float,float]
    - case:string: one of
      'N':  S01,S32 not parallel; (where S01 = -S0+S1, S32 = -S3+S2)
      'P':  S01,S32     parallel.
    - bezier_area: float (signed area between Bezier arc and chord where
                          the support points S0,S1,S2,S3 are taken as
                          control points)
    Methods:
    - __init__
    - coefficients2bezier_arc
    - sum_dist2_to_targets   (measure: sum of squared distances from middle 
                              point of chord S0S3 to target points)
    """
    def __init__(self, support_points):
        def get_case(S0,S1,S2,S3):
            ZERO = 1e-10
            S01 = points2vector(S0,S1)
            S32 = points2vector(S3,S2)
            S03  = points2vector(S0,S3)
            if (vdot(S01,S01) < ZERO) or (vdot(S32,S32) < ZERO):
                m1 = "SupportedFamily: "
                m2 = "The points to determine tangents are coincident (or too close)"
                raise Exception(m1+m2)
            if collinear([S1,S0,S3,S2]):
                m1 = "SupportedFamily: "
                m2 = "The points to determine tangents are collinear (or too close)"
                raise Exception(m1+m2)
            S01xS32 = cross(S01,S32)
            S03xS01  = cross(S03,S01)
            S03xS32  = cross(S03,S32)
            if abs(S01xS32) > ZERO: # Handles not parallel
                case = 'N'
            else:                   # Handles parallel
                case = 'P'
            return case
        if len(support_points) == 4:
            S0,S1,S2,S3 = support_points
        else:
            m1 = "SupportedFamily: "
            m2 = "Bad number of support points:"+len(support_points)
            raise Exception(m1+m2)
        case = get_case(S0,S1,S2,S3)
        self.support_points = [S0,S1,S2,S3]
        self.case = case
        self.bezier_area = BezierFunction([S0,S1,S2,S3]).signed_area()
    def sum_dist2_to_targets(self, targets): # targets = [[x,y],[x,y],...]
        def distance2(x,y):
            xy = points2vector(x,y)
            return xy[0]**2 + xy[1]**2
        S0,S1,S2,S3 = self.support_points
        middle = [(S0[0]+S3[0])/2, (S0[1]+S3[1])/2] # Middle point of chord
        return sum([distance2(middle, target) for target in targets])
    # ---------------------------
    # coefficients2bezier_arc
    # ---------------------------
    # Given xi,eta, compute control points p0,p1,p2,p3 of a supported Bezier
    # arc such that |-p0+p1| = xi*|-S0+S1| and |-p3+p2| = eta*|-S3+S2|.
    # Return BezierFunction.
    def coefficients2bezier_arc(self,xi,eta):
        S0,S1,S2,S3 = self.support_points
        S01 = points2vector(S0,S1)
        S32 = points2vector(S3,S2)
        p01 = [xi*S01[0], xi*S01[1]]
        p32 = [eta*S32[0], eta*S32[1]]
        p0 = S0
        p1 = [p0[0] + p01[0], p0[1] + p01[1]]
        p3 = S3
        p2 = [p3[0] + p32[0], p3[1] + p32[1]]
        return BezierFunction([p0,p1,p2,p3])
    def __str__(self):
        s = 'Support, case '+self.case
        s += '\n  support_points: '+str(self.support_points)
        return s

class VectorValuedFunction(object):
    """Function R -> R2.
    Initialization arguments:
    - f:    R->R2:         the function;
    - fdot: None or R->R2: None or the derivative of f
    - use_ad_hoc_fdot: boolean: If fdot=None then
                       - if use_ad_hoc_fdot=True, insert in place of fdot
                         a simple ad-hoc substitute algorithm;
                       - if use_ad_hoc_fdot=False, leave fdot=None;
                         this is likely to cause an error somewhere?
    - interval:       [a,b]: a,b are float or None; None denotes +-infinity
                      (default is [None,None], meaning whole R);
    Attributes:
    - function:   R->R2
    - derivative: None or R->R2
    - interval:   ClosedInterval
    Methods:
    - __init__
    - minimize
    - closest_point
    """
    def __init__(self,
                 f,
                 fdot=None,
                 use_ad_hoc_fdot=True, # applies if fdot=None
                 interval=[None,None]
                 ):
        self.function   = f
        lo, hi = interval
        if not ((lo is None) or (hi is None)):
            if lo > hi:
                lo,hi = hi,lo
        self.interval = ClosedInterval(lo,hi)
        if fdot is not None:
            self.derivative = fdot
        elif use_ad_hoc_fdot:
            def der(x):
                delta = 0.001
                try:
                    x1 = self.interval.closest(x-delta)
                    x2 = self.interval.closest(x+delta)
                    a1,b1 = f(x1)
                    a2,b2 = f(x2)
                except Exception as e:
                    raise Exception("VectorValuedFunction: "+str(e))
                return [(a2-a1) / (x2-x1), (b2-b1) / (x2-x1)]
            self.derivative = der
        else:
            self.derivative = None
    # --------
    # minimize
    # --------
    # Minimize key(x) where x is the parameter for the curve.
    # Args:
    # - self:            VectorValuedFunction;
    # - key:             callable: float -> float;
    # - search_interval: [float,float];
    # - guess:           float;
    # - samples:         integer;
    # - rec_depth:       integer;
    # - sloppy: Boolean.
    # Returns:
    # - x_min:   float
    # - key_min: float
    def minimize(self,
                 key,
                 search_interval=None,
                 guess=None,
                 samples=10,
                 rec_depth=0,
                 sloppy=False
                 ):
        if search_interval is None:
            lo,hi = VectorValuedFunction.get_search_interval(self, key, guess, sloppy)
        else:
            lo, hi = min(search_interval), max(search_interval)
        lo, hi = self.intersect_interval(lo, hi)
        accuracy = 1e-3
        MIN_INTERVAL = max(1e-4, accuracy * (hi-lo)) # ?
        MAX_REC_DEPTH = 10
        # Create list of x's
        xs = sorted(ClosedInterval(lo,hi).samples(samples, include_endpoints=True))
        xlo, xhi = xs[0], xs[-1]
        if (rec_depth > MAX_REC_DEPTH) or (xhi-xlo < MIN_INTERVAL):
            x = (xlo+xhi)/2
            key_value = key(x)
            return x, key_value
        # Gather indices where local minima seem to occur:
        min_indices = []
        prev_key = key(xs[0])
        curr_key  = key(xs[1])
        for i in range(1,len(xs)-1): # i is index of current x
            next_key = key(xs[i+1]) # next x
            if prev_key >= curr_key <= next_key: # Catches minima not at ends of interval
                min_indices.append(i)
            elif i==1 and prev_key <= curr_key:         # Start of interval
                min_indices.append(0)
            elif i==len(xs)-2 and curr_key >= next_key: # End of interval
                min_indices.append(len(xs)-1)
            prev_key, curr_key = curr_key, next_key
        # Recursion: Call the routine for subintervals containing
        # the tentative minima:
        sub_x_keys = []
        for j in min_indices:
            e = 1
            low, high = max(j-e,0), min(j+e, len(xs)-1)
            subinterval = [xs[low], xs[high]] # Subinterval for a tentative minimum
            # Recursion:
            sub_x, key_value = self.minimize(key, subinterval,
                                             rec_depth=rec_depth+1)
            sub_x_keys.append([sub_x, key_value])
        # Choose from the list the one with smallest key_value:
        x_min, key_min = min(sub_x_keys, key=lambda d:d[1])
        return x_min, key_min
    # -------------
    # closest_point
    # -------------
    # Find locally closest point to v on the VectorValuedFunction arc F(t)
    # (self).
    #
    # Args:
    # - self:     VectorValuedFunction
    # - v:        point [x,y];
    # - search_interval: interval [start,end] for the parameter t;
    # - guess:    float in interval, an initial guess for the parameter value
    #             of the closest point (default=0.5, which is also used if
    #             the guess is not in the interval);
    # - samples: integer;
    # - sloppy: Boolean.
    # Returns:
    # - t0: float:         the parameter of a locally closest point;
    # - b0: [float,float]: the point on the curve with parameter value t0;
    # - d0: float:         the squared distance between v and b0.
    #
    # Notes:
    # 1. If the distance has several minima, the routine may not find the
    #    globally best but only a locally best in the vicinity of the guess.
    #    Also, the formula is derived by means of linear approximation
    #    and it assumes that the changes in the parameter value are small.
    #    Thus, it is crucial that the initial guess is good!
    # 2. Possibly an end point of the interval is returned.
    # 3. The search interval is intersected by the defining interval of self.
    def closest_point(self,
                      v,
                      search_interval=None,
                      guess=None,
                      samples=10,
                      sloppy=False
                      ):
        def distance2_to_v(t):
            w = self.function(t)
            return (v[0]-w[0])**2 + (v[1]-w[1])**2
        if search_interval is None:
            lo,hi = VectorValuedFunction.get_search_interval(self,
                                        distance2_to_v,
                                        guess,
                                        sloppy)
        else:
            lo, hi = search_interval
        lo, hi = self.intersect_interval(lo, hi)
        #accuracy=1e-8
        accuracy=1e-3
        #MAX_ROUNDS = 20     # Used to avoid possibility of infinite loop.
        MAX_ROUNDS = 10     # Used to avoid possibility of infinite loop.
        MAX_T_DIFF = accuracy * (hi-lo) # To stop work when nothing
        #                                 much seems to be happening.
        if guess is None:
            guess = (lo + hi)/2.
        if not (lo <= guess <= hi):
            guess = (lo + hi)/2.
        f, fdot = self.function, self.derivative
        if fdot is None:
            raise Exception("VectorValuedFunction.closest_point: fdot is None")
        guesses = ClosedInterval(lo,hi).samples(samples)
        if not (guess is None):
           guesses.append(self.interval.closest(guess))
        td = [[t, distance2_to_v(t)] for t in guesses]
        smallest = min(td, key=lambda x:x[1])
        t0, d2 = smallest # Use this t0 as guess.
        b0 = f(t0)
        d0 = distance2_to_v(t0)
        step = 1.
        for rnd in range(MAX_ROUNDS):
            # From t0 compute new tentative guess for closest point
            # by linear approximation.
            b0v = [b0[0]-v[0], b0[1]-v[1]]
            bd = fdot(t0)
            numer = vdot(b0v,bd)            # (B(t0)-v).B'(t0)
            denom1 = vdot(bd,bd)            # B'(t0).B'(t0)
            try:
                t1 = t0 - step * numer/denom1
            except ZeroDivisionError:
                # t0 is a cusp, therefore an end point: try to get some reasonable t1
                t1 = t0 + step * (hi-lo)
            t1 = max(lo, min(hi, t1))
            t12 = [t1]
            t_diff = max([t0]+t12) - min([t0]+t12)
            tbd = []
            for tx in t12:
                bx = f(tx)
                dx = distance2_to_v(tx)
                tbd.append([tx,bx,dx])
            tbd_smallest = min(tbd, key=lambda x:x[2]) # smallest wrt d
            if d0 < tbd_smallest[2]: # d0 smallest
                step /= 2
            else:
                t0,b0,d0 = tbd_smallest        
            if t_diff < MAX_T_DIFF:
                break
        return t0,b0,d0
    # -------------------
    # get_search_interval
    # -------------------
    # For a vector valued function self, find a suitable interval for searching for
    # minimal value of key.
    # Args:
    # - pf:    VectorValuedFunction;
    # - key:   callable: float -> float;
    # - guess: float;
    # - sloppy: Boolean.
    # Returns:
    # - [lo value, hi value]: [float,float]
    def get_search_interval(self, key, guess=None, sloppy=False):
        pf = self
        if sloppy:
            pf_lo, pf_hi = [self.interval.lo, self.interval.hi]
            if not (None in (pf_lo, pf_hi)):
                return [pf_lo, pf_hi]
        MAX_ROUNDS = 100
        point = self.interval.typical_point
        step  = self.interval.typical_step
        #lo, hi = self.interval.lo, self.interval.hi
        if guess is None:
            cur_lo = point
        else:
            cur_lo = min(point, guess)
        cur_key = key(cur_lo)
        for r in range(MAX_ROUNDS):
            next_lo = self.interval.closest(cur_lo - step)
            next_key = key(next_lo)
            found_lo = next_lo
            found_key = next_key
            if next_key > cur_key:
                break
            else:
                cur_lo = next_lo
                cur_key = next_key
        else: # No break occured
            if sloppy:
                pass
                #found_lo = next_lo
            else:
                raise Exception("get_search_interval: No lower end found")
        if guess is None:
            cur_hi = point
        else:
            cur_hi = max(point, guess)
        cur_key = key(cur_hi)
        for r in range(MAX_ROUNDS):
            next_hi = self.interval.closest(cur_hi + step)
            next_key = key(next_hi)
            found_hi = next_hi
            found_key = next_key
            if next_key > cur_key:
                break
            else:
                cur_hi = next_hi
                cur_key = next_key
        else: # No break occured
            if sloppy:
                pass
            else:
                raise Exception("get_search_interval: No upper end found")
        return [found_lo, found_hi]
    # ------------------
    # intersect_interval
    # ------------------
    # Restrict interval [lo,hi] by intersecting with defining interval of self.
    # Args:
    # - lo: float
    # - hi: float
    # Returns:
    # - float
    # - float
    def intersect_interval(self, lo, hi):
        new_closed_interval = ClosedInterval(lo,hi).intersect(self.interval)
        return new_closed_interval.lo, new_closed_interval.hi


class SupportedFamily(Support, VectorValuedFunction):
    """One-parameter family of Bezier arcs in a given support.
    Attributes:
    - all attributes of Support
    - all attributes of VectorValuedFunction
    - type: string: one of
            - 'fixed point N';
            - 'fixed point P';
            (- 'fixed area'  does not appear in current version)
    - data: float
            - the fixed point ([float,float]) for type 'fixed point N';
            - the parameter for the fixed point (float) for type
              'fixed point P';
            (- the fixed area_ratio (float) for type 'fixed area')
    Methods:
    - __init__
    - parameter2bezier_arc
    """
    # Internally, the family is a vector valued function (plus some data).
    # From any parameter value x, a Bezier arc is produced by
    # parameter2bezier_arc.
    def __init__(self,
                 support_points,
                 f,
                 fdot=None,
                 use_ad_hoc_fdot=True,
                 interval=[None,None],
                 type=None,
                 data=None):
        Support.__init__(self, support_points)
        try:
            VectorValuedFunction.__init__(self,
                                        f=f,
                                        fdot=fdot,
                                        use_ad_hoc_fdot=True,
                                        interval=interval
                                        )
        except Exception as e:
            raise Exception("SupportedFamily: "+str(e))
        self.type = type
        self.data = data
    # --------------------
    # parameter2bezier_arc
    # --------------------
    # For a SupportedFamily (self) and a parameter value x, construct a BezierFunction
    # (actually a Bezier arc, four control points):
    #     x -> xi,eta -> BezierFunction
    def parameter2bezier_arc(self,x):
        xi,eta = self.function(x)
        #support = Support(self.support_points)
        #bezier_arc = support.coefficients2bezier_arc(xi,eta)
        bezier_arc = self.coefficients2bezier_arc(xi,eta)
        return bezier_arc


class ClosedInterval(object):
    """Closed real number interval. Four cases: finite, infinite at one end,
    infinite at both ends.
    Initialization arguments:
    - a: float or None (None denotes -infinity);
    - b: float or None (None denotes +infinity).
    Attributes:
    - type: string: one of  'lo,hi',  '-inf,hi',  'lo,inf',  '-inf,inf'
    - lo:   float or None (None denotes -infinity);
    - hi:   float or None (None denotes +infinity);
    - typical_point: float: a good point to start coarse searching;
    - typical_step:  float: a good starting step in a coarse searching.
    Methods:
    - __init__
    - closest
    - samples
    - intersect
    """
    def __init__(self, a=None, b=None, typical_point=None, typical_step=None):
        STEP = 1.
        NSTEPS = 10
        self.lo, self.hi = a, b
        if not (a is None):
            if not (b is None):
                if a > b:
                    self.lo, self.hi = b, a
                self.type = 'lo,hi'
            else: # b = None meaning infinity
                self.type = 'lo,inf'
        else: # a = None meaning -infinity
            if not (b is None):
                self.type = '-inf,hi'
            else: # b = None meaning infinity
                self.type = '-inf,inf'
        if not(typical_point is None):
            self.typical_point = typical_point
        else:
            if self.type == 'lo,hi':
                self.typical_point = (self.lo + self.hi)/2
            elif self.type == 'lo,inf':
                self.typical_point = self.lo + NSTEPS*STEP
            elif self.type == '-inf,hi':
                self.typical_point = self.hi - NSTEPS*STEP
            elif self.type == '-inf,inf':
                self.typical_point = 0.
        if not(typical_step is None):
            self.typical_step = max(typical_step,0.)
        else:
            if self.type == 'lo,hi':
                self.typical_step = (self.hi - self.lo)/NSTEPS
            else:
                self.typical_step = STEP
    def closest(self, x): # The closest point to x in interval
        if self.type == 'lo,hi':
            return max(self.lo, min(self.hi, x))
        elif self.type == '-inf,hi':
            return min(self.hi, x)
        elif self.type == 'lo,inf':
            return max(self.lo, x)
        else:
            return x
    def samples(self, N=8, include_endpoints=False): # Some points in the interval
        N = max(0,N)
        if self.type == 'lo,hi':
            a,b = self.lo,self.hi
        elif self.type == '-inf,hi':
            a,b = self.hi - N, self.hi
        elif self.type == 'lo,inf':
            a,b = self.lo, self.lo + N
        else:
            a,b = -N/2, N/2
        if N <= 0:
            s = []
        elif N == 1:
            s = [(a+b)/2]
        else:
            n = N+1
            s = [((n-i)*a + i*b)/n for i in range(1,n)] # (a,b) open
        if include_endpoints:
            if self.type == 'lo,hi':
                s = [a]+s+[b]
            elif self.type == '-inf,hi':
                s = s+[b]
            elif self.type == 'lo,inf':
                s = [a]+s
        return s
    def intersect(self,other):
        lo1,hi1 = self.lo, self.hi
        lo2,hi2 = other.lo,other.hi
        if   lo1 is None: lo = lo2
        elif lo2 is None: lo = lo1
        else: lo = max(lo1,lo2)
        if   hi1 is None: hi = hi2
        elif hi2 is None: hi = hi1
        else: hi = min(hi1,hi2)
        typical_step = min(self.typical_step, other.typical_step)
        return ClosedInterval(lo,hi,
                              typical_point=None,
                              typical_step=typical_step)

class BezierFunction(VectorValuedFunction):
    """Bezier arc as VectorValuedFunction,
    Attributes:
    - all attributes of VectorValuedFunction
    - control_points
    Methods:
    - __init__
    - signed_area (between arc and chord)
    """
    def __init__(self, control_points):
        VectorValuedFunction.__init__(self,
                f       =partial(bezier, control_points=control_points),
                fdot    =partial(bezier_dot, control_points=control_points),
                use_ad_hoc_fdot=False,
                interval=[0.,1.])
        self.control_points = control_points
    # -----------
    # signed_area
    # -----------
    # Signed area between arc and chord.
    # Sign of area is + if the loop is run counterclockwise *in Gimp's window*
    # when
    # - first the arc is run from start to end,
    # - followed by running the chord from end to start.
    # Notes:
    # 1. Since the y coordinate in Gimp's window runs downwards, the sign
    #    in the formula is opposite to what it should be in general!
    # 2. Currently this feature is used nowhere (but it is nice to keep
    #    the formula around just in case).
    def signed_area(self):
        p0,p1,p2,p3 = self.control_points
        p31 = points2vector(p3,p1)
        p02 = points2vector(p0,p2)
        p03 = points2vector(p0,p3)
        return (3./20) * (cross(p31,p02) + cross(p31,p03) + cross(p02,p03))

###### The usual Bezier curve and derivatives from control points  #######

def bezier(t, control_points): # control_points = [P0,P1,P2,P3]
    P0,P1,P2,P3 = control_points
    if t > 0.5:
        u = (1-t)/t
        t3 = t**3
        return [t3*(u*(u*(u*P0[0]+3*P1[0])+3*P2[0])+P3[0]),
                t3*(u*(u*(u*P0[1]+3*P1[1])+3*P2[1])+P3[1])]
    else:
        u = t/(1-t)
        t3 = (1-t)**3
        return [t3*(u*(u*(u*P3[0]+3*P2[0])+3*P1[0])+P0[0]),
                t3*(u*(u*(u*P3[1]+3*P2[1])+3*P1[1])+P0[1])]


def bezier_dot(t, control_points):
    P0,P1,P2,P3 = control_points
    p0, q0 = -P0[0]+P1[0], -P0[1]+P1[1]
    p1, q1 = -P1[0]+P2[0], -P1[1]+P2[1]
    p2, q2 = -P2[0]+P3[0], -P2[1]+P3[1]
    if t > 0.5:
        u = (1-t)/t
        t2 = t**2
        return [3*t2*(u*(u*p0+2*p1)+p2),
                3*t2*(u*(u*q0+2*q1)+q2)]
    else:
        u = t/(1-t)
        t2 = (1-t)**2
        return [3*t2*(u*(u*p2+2*p1)+p0),
                3*t2*(u*(u*q2+2*q1)+q0)]

# Used nowhere!
def bezier_dot_dot(t, control_points): # Could be made faster as above.
    p0,p1,p2,p3 = control_points
    p01 = [-p0[0]+p1[0], -p0[1]+p1[1]]
    p12 = [-p1[0]+p2[0], -p1[1]+p2[1]]
    p23 = [-p2[0]+p3[0], -p2[1]+p3[1]]
    return [-6*(1-t)*p01[0] + 6*(1-2*t)*p12[0] + 6*t*p23[0],
            -6*(1-t)*p01[1] + 6*(1-2*t)*p12[1] + 6*t*p23[1]]


################################################################
#             Central functions f,g,h in the algorithm
################################################################

def f_function(x): return 3.*x*x - 2.*x*x*x # bernstein2 + bernstein3

def g_function(x): return 3.*(1-x)*(1-x)*x  # bernstein1

def h_function(x): return 3.*(1-x)*x*x      #  bernstein2

# -------
# solve_f
# -------
# Let 0<a<1. Solve f_function(t)=a.
# Return roots in ascending order.
# Return None if not 0<a<1.
# Note: Of the three roots the important is t2 one since it belongs to (0,1).
def solve_f(a):
    from math import cos, acos, pi
    try:
        theta = acos(-2*a+1)
    except ValueError:
        raise Exception("solve_f: Got invalid argument a =",a)
    t0 = .5 + cos(theta/3)
    t1 = .5 + cos(theta/3 + 2*pi/3)
    t2 = .5 + cos(theta/3 + 4*pi/3)
    return t1,t2,t0

#def str_float(x): return '{:1.3f}'.format(x)

################################################################
#                       Plane vectors
################################################################

def cross(v,w):
    return -v[0]*w[1] + v[1]*w[0]

def points2vector(x,y): # xy = -x+y
    return [-x[0]+y[0], -x[1]+y[1]]

def vdot(v,w):
    return v[0]*w[0] + v[1]*w[1]

def collinear(points):
    ZERO = 1e-6
    n = len(points)
    if n < 3:
        return True
    for i in range(n):
        for j in range(i+1,n):
            for k in range(j+1,n):
                a,b,c = points[i], points[j], points[k]
                ab = points2vector(a,b)
                ac = points2vector(a,c)
                bc = points2vector(b,c)
                if abs(cross(ab,ac)) > ZERO:
                    return False
                if abs(cross(ab,bc)) > ZERO:
                    return False
                if abs(cross(ac,bc)) > ZERO:
                    return False
    return True


################################################################
#                    Squared distances
################################################################

# --------------------
# sum_dist2_arc_points
# --------------------
# Given a parametric arc and a list of points,
# compute the sum of squared distances of the points from the arc.
# Args:
# - pf: VectorValuedFunction
# - points:         [X1,X2,...,Xn] with all Xi [float,float].
# Returns
# - float
def sum_dist2_arc_points(pf, points, sloppy=False):
    sum_d2 = 0.
    for point in points:
        t, b, d2 = pf.closest_point(point, sloppy=sloppy)
        sum_d2 += d2
    return sum_d2

# -----------------------
# sum_dist2_family_points
# -----------------------
# Given a family of Bezier arcs and one parameter value x for the family
# (giving rise to one arc in the family),
# and a list of points,
# compute the sum of squared distances of the points from the arc.
# Args:
# - family:  SupportedFamily
# - x:       float
# - points:  [X1,X2,...,Xn] with all Xi [float,float].
# Returns
# - float
def sum_dist2_family_points(family, x, points, sloppy=False):
    bezier_arc = family.parameter2bezier_arc(x)
    return sum_dist2_arc_points(bezier_arc, points, sloppy=sloppy)


################################################################
#                       Make families
################################################################
# Make one-parameter families of Bezier arcs with common support S
# and some chosen other condition.
# (In this version the only appearing condition is that the arcs pass
# through a fixed point.)

# ------------------
# family_fixed_point
# ------------------
# Given support S:Support and a point X:[float,float],
# create the family of Bezier arcs having the support S and passing
# through X.
# If respect_directions is True, include in the family only those Bezier
# arcs for which the handles point in the same directins as the handles
# of the support.
# Args:
# - support:            Support
# - point:              [float,float]
# - respect_directions: boolean
# Returns: SupportedFamily
def family_fixed_point(support, point, respect_directions=False):
    def check_condition(xi, eta, t, X): # Condition B(t) = X
        ba = support.coefficients2bezier_arc(xi,eta)
        cp = ba.control_points
        Bt = bezier(t, cp)
        diff = points2vector(Bt,X)
    PADDING = 1e-8 # To avoid interval ends 0 and 1
    ZERO = 1e-10
    S0,S1,S2,S3 = support.support_points
    X = point
    case = support.case
    S0X = points2vector(S0,X)
    S32 = points2vector(S3,S2)
    S01 = points2vector(S0,S1)
    S03 = points2vector(S0,S3)
    S0XxS01 = cross(S0X,S01)
    S0XxS03 = cross(S0X,S03)
    S03xS32 = cross(S03,S32)
    S03xS01 = cross(S03,S01)
    S01xS32 = cross(S01,S32)
    S01xS03 = -S03xS01
    S32xS01 = -S01xS32
    S32xS03 = -S03xS32
    if case == 'N':
        def F(t):
            ft, gt, ht = f_function(t), g_function(t), h_function(t)
            diff = [S0X[0] - ft*S03[0], S0X[1] - ft*S03[1]]
            try:
                xi  = cross(diff, S32) / (gt * S01xS32)
                eta = cross(diff, S01) / (ht * S32xS01)
            except ZeroDivisionError:
                raise Exception("parametric function: t = 0 or 1.")
            #check_condition(xi, eta, t, X)
            return [xi,eta]
        def Fdot(t):
            xi, eta = F(t)
            xidot  = -((1-3*t)*xi/t      + 2*S03xS32/S01xS32) / (1-t)
            etadot = -((2-3*t)*eta/(1-t) + 2*S03xS01/S32xS01) / t
            return [xidot, etadot]
        if respect_directions:
            interval, None_case = first_quadrant_subinterval(support, point)
            if interval is None:
                raise Exception("family_fixed_point: Got subinterval=None:"+None_case)
            if interval[0] < ZERO:
                interval[0] = PADDING
            if interval[1] > 1-ZERO:
                interval[1] = 1-PADDING
        else:
            interval=[PADDING,1-PADDING]
        type = 'fixed point N'
        data = point
    else: # Case P
        if not (0 + ZERO <= S0XxS01 / S03xS01 <= 1 - ZERO):
            raise Exception("family_fixed_point: The target point in illegal area: not between the tangents lines")
        try:
            t0 = solve_f(S0XxS01 / S03xS01)[1]
        except ZeroDivisionError:
            raise Exception("parametric function: Support points collinear.")
        ft, gt, ht = f_function(t0), g_function(t0), h_function(t0)
        a = gt * S01xS03
        b = ht * S32xS03
        c = S0XxS03
        try:
            invb = 1./b
        except ZeroDivisionError:
            raise Exception("parametric function: Support points collinear.")
        def F(xi):
            eta = (c-a*xi) * invb
            #check_condition(xi, eta, t0, X)
            return [xi,eta]
        def Fdot(xi):
            return [1., -a*invb]
        if respect_directions:
            interval, None_case = first_quadrant_subinterval(support, point)
            if interval is None:
                raise Exception("family_fixed_point: Got subinterval=None:"+None_case)
        else:
            interval=[None,None]
        type = 'fixed point P'
        data = t0
    try:
        supported_family = SupportedFamily(support_points=[S0,S1,S2,S3],
                               f=F,
                               fdot=Fdot,
                               use_ad_hoc_fdot=False,
                               interval=interval,
                               type=type,
                               data=data)
    except Exception as e:
        raise Exception('family_fixed_point: ' +str(e))
    return supported_family

# --------------------------
# first_quadrant_subinterval
# --------------------------
# In the algorithm it may be required to find only those solutions
# (Bezier arcs) whose tangents obey the directions S01 and S32 in
# the support.
# This routine finds the interval [a,b] where the parameter x needs
# to be restricted to this end. The family in question is the one produced
# by family_fixed_point, and x is its parameter. Let F be the vector-valued
# function corresponding to the family. The condition to be enforced is that
# if x is in [a,b] and F(x) = (xi,eta), then xi >= 0 and eta >= 0.
#
# Returns:
# - [a,b] or None (may be b=None, meaning infinity)
# - None_case: string (only for debugging)
# Note: points 0,1 included: caller must handle!
def first_quadrant_subinterval(support, point):
    ZERO = 1e-10
    f = f_function
    S0,S1,S2,S3 = support.support_points
    X = point
    case = support.case
    S0X = points2vector(S0,X)
    S32 = points2vector(S3,S2)
    S01 = points2vector(S0,S1)
    S03 = points2vector(S0,S3)
    S0XxS01 = cross(S0X,S01)
    S0XxS03 = cross(S0X,S03)
    S03xS32 = cross(S03,S32)
    S03xS01 = cross(S03,S01)
    S01xS32 = cross(S01,S32)
    S01xS03 = -S03xS01
    S32xS01 = -S01xS32
    S32xS03 = -S03xS32
    S0XxS32 = cross(S0X,S32)
    None_case = None
    if case == 'N':
        a = S0XxS32 / S01xS32
        b = S03xS32 / S01xS32
        c = S0XxS01 / S32xS01
        d = S03xS01 / S32xS01
        cd_ab = []
        try:
            if 0 < c/d < 1:
                cd_ab.append(c/d)
        except ZeroDivisionError:
            pass
        try:
            if 0 < a/b < 1:
                cd_ab.append(a/b)
        except ZeroDivisionError:
            pass
        tx_ty = [solve_f(q)[1] for q in cd_ab]
        t1_t2 = sorted(tx_ty)
        if len(t1_t2) == 2:
            if -ZERO < t1_t2[0] - t1_t2[1] < ZERO:
                t1_t2 = [t1_t2[0]]
        if len(t1_t2) == 2:
            t1,t2 = t1_t2
            if a - b*f((t1+t2)/2) > 0 and c - d*f((t1+t2)/2) > 0:
                subinterval = [t1,t2]
            elif a - b*f(t1/2) > 0 and c - d*f(t1/2) > 0:
                subinterval = [0.,t1]
            elif a - b*f((t2+1)/2) > 0 and c - d*f((t2+1)/2) > 0:
                subinterval = [t2,1.]
            else:
                subinterval = None
                None_case = '2'
        elif len(t1_t2) == 1:
            t1 = t1_t2[0]
            if a - b*f(t1/2) > 0 and c - d*f(t1/2) > 0:
                subinterval = [0.,t1]
            elif a - b*f((t1+1)/2) > 0 and c - d*f((t1+1)/2) > 0:
                subinterval = [t1,1.]
            else:
                subinterval = None
                None_case = '1'
        elif len(t1_t2) == 0:
            if a - b*f(1/2) > 0 and c - d*f(1/2) > 0:
                subinterval = [0.,1.]
            else:
                subinterval = None
                None_case = '0'
    else: # Case 'P'
        try:
            t0 = solve_f(S0XxS01 / S03xS01)[1]
        except ZeroDivisionError:
            raise Exception("first_quadrant_subinterval: Support points collinear.")
        ft, gt, ht = f_function(t0), g_function(t0), h_function(t0)
        a = gt * S01xS03
        b = ht * S32xS03
        c = S0XxS03
        if a*b > 0 and c*a > 0:
            subinterval = [0, c/a]
        elif a*b < 0:
            subinterval = [max(0, c/a), None]
        else:
            subinterval = None
            None_case = 'P'
    return subinterval, None_case


################################################################
#                   Create special Bezier arcs
################################################################

# ------------
# check_target
# ------------
# Check that target is far enough from S0 and S3
def check_target(support,trg):
    MIN_DIST2 = 1e-8
    S0 = support.support_points[0]
    S3 = support.support_points[3]
    dist0 = (trg[0] - S0[0])**2 + (trg[1] - S0[1])**2
    dist3 = (trg[0] - S3[0])**2 + (trg[1] - S3[1])**2
    return (dist0 > MIN_DIST2) and (dist3 > MIN_DIST2)

# -----------------------------
# arc_fixed_point_approx_points
# -----------------------------
# Given a support S and a list of points X0,...,Xn
# find a Bezier arc with support S, passing exactly through X0,
# and as closely to X1,...,Xn as possible.
# Args:
# - support: Support;
# - target0: [float,float];
# - other_targets: [[float,float]];
# - respect_directions: boolean: if True, search among Bezier arcs whose
#                       handle directions obey those of support;
# - sloppy: boolean: if True, include not-so-exact results in calculations.
# Returns:
# - ba_approx_cp (list of 4 control points: [p0,p1,p2,p3]);
# - abs_error: float (an error measure);
# - rel_error: float (another error measure).
def arc_fixed_point_approx_points(support,
                                  target0,
                                  other_targets,
                                  respect_directions=False,
                                  sloppy=False
                                  ):
    from math import sqrt
    # Discard targets too close to the corner(s) of the support
    # and that sufficiently many remain:
    if not check_target(support,target0):
        message = 'arc_fixed_point_approx_points: first target too close to corners'
        raise Exception(message)
    other_targets = [trg for trg in other_targets if check_target(support,trg)]
    if len(other_targets) < 1:
        message = 'arc_fixed_point_approx_points: not enough targets not too close to corners'
        raise Exception(message)
    #
    X0 = target0
    Xpoints = other_targets
    try:
        F0 = family_fixed_point(support, X0, respect_directions)
    except Exception as e:
        raise Exception("arc_fixed_point_approx_points: "+str(e))
    def pf_key(x):
        return sum_dist2_family_points(F0, x, Xpoints, sloppy)
    xmin, keymin = F0.minimize(key=pf_key, search_interval=None,
                                sloppy=sloppy)
    xi,eta = F0.function(xmin)
    ba_approx = support.coefficients2bezier_arc(xi,eta)
    abs_error = sum_dist2_arc_points(ba_approx, other_targets,sloppy)
    yard_stick = support.sum_dist2_to_targets(other_targets)
    rel_error = sqrt(abs_error / yard_stick)
    return ba_approx.control_points, abs_error, rel_error

# -----------------
# arc_approx_points
# -----------------
# Given a support S and a list of target points X0,X1,..., find a Bezier arc
# with support S and passing close(?) to the points.
# Args:
# - support: Support;
# - targets:  [[float,float]];
# - respect_directions: boolean: if True, search among Bezier arcs whose
#                       handle directions obey those of support;
# - sloppy: boolean: if True, include not-so-exact results in calculations.
# - error_targets: None or [[float,float]]: if not None, compute error
#                  of the resulting arc from the targets in this list,
#                  otherwise use the list targets
# Returns:
# - list of 4 control points: [p0,p1,p2,p3];
# - abs_error: float (an error measure);
# - rel_error: float (another error measure).
# Note: This is rather ad-hoc, especially the usage of weighted average.
def arc_approx_points(support,
                      targets,
                      respect_directions=False,
                      sloppy=False,
                      error_targets=None
                      ):
    # Given list [[cp,error],...], where cp = [p0,p1,p2,p3] (control points)
    # compute weighted average of the control points,
    # using as weights the inverses of the errors mapped with some function
    # tweak: float -> float
    def weighted_average_cp(cp_err_list, tweak):
        ave_p0 = [sum([cp[0][0]/tweak(err) for cp,err in cp_err_list]),
                  sum([cp[0][1]/tweak(err) for cp,err in cp_err_list])]
        ave_p1 = [sum([cp[1][0]/tweak(err) for cp,err in cp_err_list]),
                  sum([cp[1][1]/tweak(err) for cp,err in cp_err_list])]
        ave_p2 = [sum([cp[2][0]/tweak(err) for cp,err in cp_err_list]),
                  sum([cp[2][1]/tweak(err) for cp,err in cp_err_list])]
        ave_p3 = [sum([cp[3][0]/tweak(err) for cp,err in cp_err_list]),
                  sum([cp[3][1]/tweak(err) for cp,err in cp_err_list])]
        m = sum([1./tweak(err) for cp,err in cp_err_list])
        ave_p0 = [ave_p0[0] / m, ave_p0[1] / m]
        ave_p1 = [ave_p1[0] / m, ave_p1[1] / m]
        ave_p2 = [ave_p2[0] / m, ave_p2[1] / m]
        ave_p3 = [ave_p3[0] / m, ave_p3[1] / m]
        return [ave_p0, ave_p1, ave_p2, ave_p3]
    # Discard targets too close to the corner(s) of the support
    # and that sufficiently many remain:
    targets = [trg for trg in targets if check_target(support,trg)]
    if len(targets) < 1:
        message = 'arc_fixed_point_approx_points: not enough targets not too close to corners'
        raise Exception(message)
    # Compute a list of tentative approximate Bezier arcs (control points):
    cp_error_list = []
    for i in range(len(targets)):
        X0 = targets[i]
        rest = [targets[j] for j in range(len(targets)) if j != i]
        try:
            # Bezier arc exactly through X0 and trying
            # to run close to the rest of the targets:
            approx_result = arc_fixed_point_approx_points(support,
                                                      X0,
                                                      rest,
                                                      respect_directions,
                                                      sloppy=sloppy)
        except Exception as e:
            raise Exception("arc_approx_points: "+str(e))
        approx_cp, abs_error, rel_error = approx_result
        # Save control points and abs_error in list:
        cp_error_list.append([approx_cp, abs_error])
    # Sort with respect to abs_error:
    cp_error_list = sorted(cp_error_list, key=lambda x:x[1])
    # Compute weighted averages from the starting sections of the the list:
    ave_cp_error_list = []
    for i in range(1, 1+len(cp_error_list)):
        ave_cp = weighted_average_cp(cp_error_list[:i], tweak=sqrt)
        #ave_cp = weighted_average_cp(cp_error_list[:i], tweak=lambda x:x)
        ave_error = sum_dist2_arc_points(BezierFunction(ave_cp), targets, sloppy=True)
        ave_cp_error_list.append([ave_cp, ave_error])
    # Find the best; this one will be returned:
    best_cp, best_error = min(ave_cp_error_list, key=lambda x:x[1])
    # Compute anew the absolute and relative errors of the best:
    best_bf = BezierFunction(best_cp)
    if error_targets is None:
        error_targets = targets
    abs_error = sum_dist2_arc_points(best_bf, error_targets, sloppy=True)
    yard_stick = support.sum_dist2_to_targets(targets) # For relative error
    rel_error = sqrt(abs_error / yard_stick)
    return best_cp, abs_error, rel_error


#==============================================================
#==============================================================
#           Parametric curve -> Bezier curve (algorithm)       
#==============================================================
#==============================================================
### Changes from 1.08 marked: CHANGE FROM 1.08

# ----------------------
# paramcurve2beziercurve
# ----------------------
# From a given parametric curve construct an approximate Bezier curve.
# Args:
# - f:             callable: float -> [float,float];
# - fdot:          None or callable: float -> [float,float];
# - interval:      [float,float]
# - custom_params: [float,...];
# - closed:        boolean;
# - work_level:    integer 1..10 (has effect on recursion depth and accuracy);
# - messages:      boolean (messages go to message_handler);
# - message_handler: callable.
# Returns:
# - BCurve.BezierCurve
# - float (an error measure)
# Notes:
# 1. The parametric curve is given as the function f:R->R2 and the interval.
# 2. fdot is None or the derivative f':R->R2. If it is None, a very simple
#    substitute algorithm is used. Results are probably better if a correct
#    expression for fdot is given instead.
# 3. custom_params is a list of floats, such as [0, pi/2, 3*pi/2].
#    The items in the list will appear as anchors of the resulting Bezier
#    curve except that values too close to each other or to the ends of the
#    interval are not accepted).
def paramcurve2beziercurve(function_name,
                           f,
                           fdot=None,
                           interval=[0,1],
                           custom_params=[],
                           closed=False,
                           work_level=1,
                           messages=False,
                           message_handler=None
                           ):
    # Subintervals [lo,hi] from interval + custom points:
    ACCURACY = 1e-2      # Effects: accuracy and running time
    NSAMPLES = 2         # Effects: accuracy and running time
    ERROR_COMP_LEVEL = 2 # Effects: error computation and recursion depth
    work_level = min(100,max(1,work_level))
    def make_subintervals(interval, custom):
        if len(custom) == 0:
            return [interval]
        PADDING = 1e-5
        #PADDING = 1.5
        lo,hi = interval
        if lo < hi:
            reversed = False
        else:
            reversed = True
            lo,hi = hi,lo
        custom = sorted(custom)
        custom = [c for c in custom if lo+PADDING < c < hi-PADDING]
        # Strip from custom close duplicates by always
        # replacing close duplicates with their averages:
        if len(custom) <= 1:
            stripped = custom
        else:
            stripped = []
            i = 0
            at_end = False
            while not at_end:
                duplicates = [custom[i]]
                for j in range(i,len(custom)-1):
                    if custom[j+1]-custom[j] < PADDING:
                        duplicates.append(custom[j+1])
                    else:
                        break
                else: # No break: reached end
                    at_end = True
                stripped.append(sum(duplicates) / len(duplicates))
                i = j+1
        stripped = [float(lo)] + stripped + [float(hi)]
        if reversed:
            stripped = stripped[::-1]
        pairs = [] # arrange in pairs
        for i in range(1,len(stripped)):
            pairs.append([stripped[i-1], stripped[i]])
        return pairs
    total_cp4_list = []
    max_rel_error = 0.
    for subinterval in make_subintervals(interval, custom_params):
        cp4_list, rel_error, _ = paramcurve2cp4list(f,
                            fdot,
                            interval=subinterval,
                            accuracy=ACCURACY/work_level,
                            nsamples=NSAMPLES,
                            error_comp_level=ERROR_COMP_LEVEL,
                            rec_depth=0,
                            messages=messages,
                            message_handler=message_handler
                            )
        total_cp4_list += cp4_list
        max_rel_error = max(max_rel_error, rel_error)
    if interval[0] > interval[1]: # Reversed
        total_cp4_list = total_cp4_list[::-1]
    ba_list = [BCurve.BezierArc(cp4) for cp4 in total_cp4_list]
    bc = BCurve.BezierCurve(
                     curve_name =function_name,
                     bezier_arcs=ba_list, # [BCurve.BezierArc]
                     head_handle=None,    # BCurve.ControlPoint
                     tail_handle=None,    # BCurve.ControlPoint
                     closed=closed)
    return bc, max_rel_error

# ------------------
# paramcurve2cp4list
# ------------------
# From a parametric curve construct an approximation as a list of
# butting Bezier arcs. Each Bezier arc will be represented as its
# control point list cp4=[p0,p1,p2,p3].
# Args:
# - f:                  callable: R -> R2
# - fdot:               None or callable: R -> R2
# - interval:           [float,float]
# - accuracy:           float
# - nsamples: integer:  number of sample points used in the algorithm;
# - error_comp_level:   integer >= 1: how laboriously to compute error of
#                       resulting arc - has heavy effect on the depth of
#                       recursion in the algorithm;
# - rec_depth: integer: only for internal use to control recursion depth;
# - messages:           boolean (messages go to message_handler);
# - message_handler:    callable.
# Returns:
# - list [cp4,cp4,...] where cp4=[p0,p1,p2,p3] (control points of butting
#   Bezier arcs);
# - rel_error: float;
# - n_max_rd:  integer: how many times the maximum recursion depth was reached.
def paramcurve2cp4list(f,
                       fdot,
                       interval=None,
                       accuracy=1e-3,
                       max_rec_depth=4,
                       nsamples=4,
                       error_comp_level=2,
                       rec_depth=0,
                       very_short_arc=False, # CHANGE FROM 1.08
                       messages=False,
                       message_handler=None
                       ):
    MAX_REC_DEPTH = max_rec_depth
    #print("Using MAX_REC_DEPTH ="+str(MAX_REC_DEPTH))
    MAX_REL_ERROR = accuracy # maximum allowed relative error
    MIN_SUPPORT_SIZE = 1e-5  # When going under this, use simple solution
    reached_MAX_REC_DEPTH = 0
    nsamples = max(2, nsamples)
    error_comp_level = max(1, error_comp_level)
    if messages and (message_handler is None):
        m = ("Cannot display messages since"
             "\nmessage_handler is None!")
        raise Exception(m)
    def parallel_vectors(a,b): # May raise ZeroDivisionError
        ZERO = 1e-8
        alen = sqrt(a[0]**2 + a[1]**2)
        blen = sqrt(b[0]**2 + b[1]**2)
        x0,x1 = a[0]/alen, a[1]/alen
        y0,y1 = b[0]/blen, b[1]/blen
        return abs(-x0*y1 + x1*y0) < ZERO
    def check_directions(support, cp):
        S0,S1,S2,S3 = support.support_points
        p0,p1,p2,p3 = cp
        S01 = [-S0[0]+S1[0], -S0[1]+S1[1]]  # handles from support
        S32 = [-S3[0]+S2[0], -S3[1]+S2[1]]
        p01 = [-p0[0]+p1[0], -p0[1]+p1[1]]  # handles from cp
        p32 = [-p3[0]+p2[0], -p3[1]+p2[1]]
        Sp0 = S01[0]*p01[0] + S01[1]*p01[1] # dot products
        Sp3 = S32[0]*p32[0] + S32[1]*p32[1]
        return (Sp0 >= 0) and (Sp3 >= 0)    # same directions?
    class MakeSupportFailure(Exception):
        def __init__(self,m):
            self.message = m
        def __str__(self):
            #return 'make_support: '+self.message
            return self.message # CHANGE FROM 1.08
    def make_support(pf):
        try:
            ZERO = 1e-10
            lo, hi = pf.interval.lo, pf.interval.hi
            reversed = lo>hi
            if reversed:
                lo,hi = hi,lo
            S0 = pf.function(lo)
            S3 = pf.function(hi)
            S03 = [-S0[0]+S3[0], -S0[1]+S3[1]]
            #
            delta = (hi-lo)/100
            # 1. Compute S01 first with algorithm which works even
            #    if pf.derivative=None or if it is 0 at lo (cusp):
            tang = tangent_direction(pf.function, lo, delta, algorithm=3)
            tanglen = sqrt(vdot(tang,tang))
            S03len = sqrt(vdot(S03,S03))
            try:
                q = S03len / (4*tanglen) # ad hoc
            except ZeroDivisionError:
                raise MakeSupportFailure("Got zero tangent at lo")
            S01 = [q*tang[0], q*tang[1]]             # S01 from tangent_direction
            # 2. Compute S01 then from pf.derivative if possible:
            if pf.derivative is not None:
                pfdot0 = pf.derivative(lo)
                # 1.9 is based on experiments:
                S01_der = [pfdot0[0]/1.9, pfdot0[1]/1.9] # S01 from pf.derivative
                try:
                    if parallel_vectors(S01,S01_der):
                        S01 = S01_der # More accurate than tangent_direction?
                except ZeroDivisionError: # pfdot0 = 0
                    pass # use the one got from tangent_direction
            fplus = pf.function(lo + delta)
            S01dir = [fplus[0]-S0[0], fplus[1]-S0[1]]
            if vdot(S01, S01dir) < 0: # Check direction
                S01 = [-S01[0], -S01[1]]
            #
            # 1. Compute S32 first with algorithm which works even
            #    if pf.derivative=None or if it is 0 at lo (cusp):
            tang = tangent_direction(pf.function, hi, -delta, algorithm=3)
            tanglen = sqrt(vdot(tang,tang))
            #S03len = sqrt(vdot(S03,S03))
            try:
                q = S03len / (4*tanglen) # ad hoc
            except ZeroDivisionError:
                raise MakeSupportFailure("Got zero tangent at hi")
            S32 = [q*tang[0], q*tang[1]]             # S32 from tangent_direction
            # 2. Compute S32 then from pf.derivative if possible:
            if pf.derivative is not None:
                pfdot3 = pf.derivative(hi)
                # 1.9 is based on experiments:
                S32_der = [pfdot3[0]/1.9, pfdot3[1]/1.9] # S32 from pf.derivative
                try:
                    if parallel_vectors(S32,S32_der):
                        S32 = S32_der # More accurate than tangent_direction?
                except ZeroDivisionError: # pfdot3 = 0
                    pass # use the one got from tangent_direction
            fminus = pf.function(hi - delta)
            S32dir = [fminus[0]-S3[0], fminus[1]-S3[1]]
            if vdot(S32, S32dir) < 0: # Check direction
                S32 = [-S32[0], -S32[1]]
            #
            S1 = [S0[0] + S01[0], S0[1] + S01[1]]
            S2 = [S3[0] + S32[0], S3[1] + S32[1]]
            support_size = vdot(S03,S03) + vdot(S01,S01) + vdot(S32,S32)
            try:
                if not reversed:
                    support = Support([S0,S1,S2,S3])
                else:
                    support = Support([S3,S2,S1,S0])
                return support, support_size
            except Exception as e:
                raise MakeSupportFailure(str(e))
        except MakeSupportFailure as e:
            raise MakeSupportFailure(str(e))
        except:
            raise MakeSupportFailure('Unknown exception!')
    # Simple solution: arc through X
    def simple_cp4(support, X, messages, message_handler):    # CHANGE FROM 1.08
        S0,S1,S2,S3 = support.support_points
        try:
            try: # Try first non-parallel handles
                S01 = points2vector(S0,S1)
                S32 = points2vector(S3,S2)
                S01xS32 = cross(S01,S32)
                invS01xS32 = 1 / S01xS32 # If ZeroDivisionError go to parallel handles
                M = [(S0[0]+S3[0])/2, (S0[1]+S3[1])/2]
                MX = points2vector(M,X)
                MXxS32 = cross(MX,S32)
                MXxS01 = cross(MX,S01)
                xi  =  (8./3) * MXxS32 * invS01xS32
                eta = -(8./3) * MXxS01 * invS01xS32
                #S03xS32  = cross(S03,S32)
            except ZeroDivisionError: # Parallel handles
                S03  = points2vector(S0,S3)
                S0X  = points2vector(S0,X)
                S03xS01  = cross(S03,S01)
                S03xS32  = cross(S03,S32)
                S0XxS01  = cross(S0X,S01)
                S0XxS03  = cross(S0X,S03)
                t0 = solve_f(S0XxS01 / S03xS01)[1]
                a = -g_function(t0) * S03xS01
                b = -h_function(t0) * S03xS32
                c = S0XxS03
                xi = eta = c / (a+b)
        except: # Some unknown error
            if messages:
                message_handler("\nResorting to a very simple solution")
            return very_simple_cp4(support)
        # control points
        p0 = S0
        p1 = [S0[0] + xi*S01[0], S0[1] + xi*S01[1]]
        p2 = [S3[0] + eta*S32[0], S3[1] + eta*S32[1]]
        p3 = S3
        return [p0,p1,p2,p3]
    # Very simple solution: cp = [S0,S1,S2,S3]
    def very_simple_cp4(support):             # CHANGE FROM 1.08
        return support.support_points
    def linear_cp4(pf): # Control points cp4 for linear edge
        lo, hi = pf.interval.lo, pf.interval.hi
        x0,y0 = pf.function(lo)
        x3,y3 = pf.function(hi)
        return [BCurve.ControlPoint(x0,y0)]*2 + [BCurve.ControlPoint(x3,y3)]*2
    def dist2(v,w):
        return (v[0]-w[0])**2 + (v[1]-w[1])**2
    def get_sample(pf, N): # [X1,...,XN]
        # ----------------
        # divide_uniformly
        # ----------------
        # Given function f(t):R->R2, arc lo<=t<=hi, find 2**N-1
        # parameter values ti, lo<ti<hi, such that the f(ti) are roughly
        # equally spaced along the arc.
        # Check result and return sorted list of parameter values.
        # If include_ends=True, include lo and hi.
        # Returns:
        # - boolean
        # - [float]
        def divide_uniformly(f,lo,hi,N, include_ends=False):
            flo,fhi = f(lo),f(hi)
            sections = [[lo,hi,flo,fhi, dist2(flo,fhi)]]
            for split in range(2**N-1):
                # Find longest section, first its index:
                index_longest = max(range(len(sections)),
                                    key=lambda i:sections[i][-1])
                longest = sections[index_longest]
                # Split longest:
                a,b,fa,fb,d2 = longest
                c = (a+b)/2
                fc = f(c)
                left  = [a,c,fa,fc,dist2(fa,fc)]
                right = [c,b,fc,fb,dist2(fc,fb)]
                # Replace longest with the two new:
                head = sections[:index_longest]
                tail = sections[1+index_longest:]
                sections = head+[left,right]+tail
            inner = [s[0] for s in sections[1:]] #lo,hi excluded
            check = check_uniformity(f,[lo]+inner+[hi])
            if include_ends:
                return check, [lo]+inner+[hi]
            else:
                return check, inner
        def check_uniformity(f,params):
            ACCEPT = 7 # Experimental limit
            ft = [f(t) for t in params]
            lengths2 = [dist2(ft[i],ft[i-1]) for i in range(1,len(ft))]
            m,M = min(lengths2),max(lengths2)
            try:
                return M/m < ACCEPT
            except ZeroDivisionError:
                return False
        # First try simple: parameters equally spaced
        lo, hi = pf.interval.lo, pf.interval.hi
        f = pf.function
        sample = [(N+1-i)*lo/(N+1) + i*hi/(N+1) for i in range(1,N+1)]
        check = check_uniformity(f, [lo]+sample+[hi])
        # If check not good, make better(?) sample
        if not check:
            M = 1
            while 2**M < N+1:
                M *= 2
            check_new_sample, new_sample = divide_uniformly(f,lo,hi,M)
            if messages and not check_new_sample:
                message_handler("Using not-very-uniform sample.")
            return [pf.function(t) for t in new_sample]
        return [pf.function(t) for t in sample]
    pf_input = VectorValuedFunction(
                    f=f,
                    fdot=fdot,
                    # If fdot=None and use_ad_hoc_fdot=False,
                    # tangent directions are computed by
                    # tangent_direction (algorithm 3)
                    # If fdot=None and use_ad_hoc_fdot=True,
                    # tangent directions are from simple ad-hoc
                    # derivation (VectorValuedFunction init)
                    use_ad_hoc_fdot=False,
                    interval=interval)
    over_MAX_REC_DEPTH = False
    try:
        support, support_size = make_support(pf_input)
        #if support_size < MIN_SUPPORT_SIZE:
        if very_short_arc or (support_size < MIN_SUPPORT_SIZE): # CHANGE FROM 1.08
            #cp4 = simple_cp4(support)
            # CHANGE FROM 1.08
            middle = f((interval[0] + interval[1])/2) # CHANGE FROM 1.08
            cp4 = simple_cp4(support, middle, messages, message_handler)
            ##############
            rel_error = 0 # Not true!
            recursion = True
            if messages:
                if very_short_arc:
                    m = "Very short arc: no recursion"
                else:
                    m = "support_size < MIN_SUPPORT_SIZE: "
                #message_handler(m)
            raise Exception(m)
        else:
            try:
                Xpoints = get_sample(pf_input, nsamples)
                Epoints = get_sample(pf_input, nsamples*error_comp_level)
            except Exception as e:
                raise Exception("paramcurve2cp4list: get_sample: "+str(e))
            try:
                cp4, abs_error, rel_error = arc_approx_points(support,
                                         Xpoints,
                                         respect_directions=True,
                                         sloppy=True,
                                         error_targets = Epoints)
                check = check_directions(support, cp4)
               #
                if rec_depth >= MAX_REC_DEPTH:
                    recursion = False
                elif rel_error > MAX_REL_ERROR:
                    recursion = True
                else:
                    recursion = not check
            except Exception as e:
                raise Exception("paramcurve2cp4list: "+str(e))
    except MakeSupportFailure as e: # No support!
        if very_short_arc:                # CHANGE FROM 1.08
            recursion = False
            cp4 = linear_cp4(pf_input) # No support; use linear
            rel_error = 0 # Not true!
            if messages:
                m = ("MakeSupportFailure:"
                     +"\n"+str(e)
                     +"\nVery short arc (case 1): no recursion"
                     +"\nResorting to a linear solution."
                     #+"\nThe result may be quite wrong!"
                    )
                message_handler(m)
        elif rec_depth < MAX_REC_DEPTH:
            recursion = True
            if messages:
                m = ("MakeSupportFailure:"
                     "\n"+str(e)+"."
                        #"\nwhen rec_depth < MAX_REC_DEPTH."
                        "\nTrying recursion: fingers crossed!"
                    )
                message_handler(m)
        else:
            over_MAX_REC_DEPTH = True
            recursion = False
            cp4 = linear_cp4(pf_input) # No support; use linear
            rel_error = 0 # Not true!
            if messages:
                m = ("Met with Exception (raised by make_support?):"
                     "\n"+str(e)+" "
                     "\nwhen rec_depth >= MAX_REC_DEPTH."
                     "\nResorting to linear solution."
                     "\nThe result may be quite wrong!"
                    )
                message_handler(m)
    except Exception as e: # Some Exception, but support is available
        if very_short_arc:                # CHANGE FROM 1.08
            recursion = False
            #cp4 = simple_cp4(support) # support available; use simple
            # CHANGE FROM 1.08
            middle = f((interval[0] + interval[1])/2) # CHANGE FROM 1.08
            cp4 = simple_cp4(support, middle, messages, message_handler)
            ##############
            rel_error = 0 # Not true!
            if messages:
                m = ("Very short arc (case 2): no recursion"
                     "\nResorting to a simple solution"
                     #"\nThe result may be quite wrong!"
                    )
                message_handler(m)
        elif rec_depth < MAX_REC_DEPTH:   # CHANGE FROM 1.08
            recursion = True
            if messages:
                m = ("paramcurve2cp4list:"
                     "\n"+str(e)+"."
                     #"\nwhen rec_depth < MAX_REC_DEPTH."
                     "\nTrying recursion: fingers crossed!"
                    )
                message_handler(m)
        else:
            over_MAX_REC_DEPTH = True
            recursion = False
            #cp4 = simple_cp4(support) # support available; use simple
            # CHANGE FROM 1.08
            middle = f((interval[0] + interval[1])/2) # CHANGE FROM 1.08
            cp4 = simple_cp4(support, middle, messages, message_handler)
            ##############
            rel_error = 0 # Not true!
            if messages:
                m = ("arc_approx_points raised Exception:"
                     "\n"+str(e)+" "
                     "\nwhen rec_depth >= MAX_REC_DEPTH 2."
                     "\nResorting (partially) to a simple solution;"
                     "\nThe result may be quite wrong!"
                    )
                message_handler(m)
    if not recursion:
        cp4_list = [cp4]
        if rec_depth >= MAX_REC_DEPTH:
            n_max_rd = 1
        else:
            n_max_rd = 0
    else: # Recursion
        if interval[0] < interval[1]:
            left_half  = [interval[0], (interval[0]+interval[1])/2]
            right_half = [(interval[0]+interval[1])/2, interval[1]]
        else: # reverse
            left_half  = [interval[1], (interval[0]+interval[1])/2]
            right_half = [(interval[0]+interval[1])/2, interval[0]]
        left_cp4_list, left_rel_error, left_n_max_rd = paramcurve2cp4list(
                            f,
                            fdot,
                            interval=left_half,
                            accuracy=accuracy,
                            nsamples=nsamples,
                            rec_depth=rec_depth+1,
                            messages=messages,
                            message_handler=message_handler
                            )
        right_cp4_list, right_rel_error, right_n_max_rd = paramcurve2cp4list(
                            f,
                            fdot,
                            interval=right_half,
                            accuracy=accuracy,
                            nsamples=nsamples,
                            rec_depth=rec_depth+1,
                            messages=messages,
                            message_handler=message_handler
                            )
        #cp4_list = left_cp4_list + right_cp4_list
        #
        if interval[0] < interval[1]:
            cp4_list = left_cp4_list + right_cp4_list
        else: # reversed
            left_cp4_list.reverse()
            right_cp4_list.reverse()
            cp4_list = right_cp4_list + left_cp4_list
        #
        #rel_error = (left_rel_error + right_rel_error) / 2   # ?
        rel_error = max(left_rel_error, right_rel_error)   # ?
        n_max_rd = left_n_max_rd + right_n_max_rd
    #if messages and (rec_depth == 0) and (n_max_rd > 0):
    if messages and over_MAX_REC_DEPTH and (rec_depth == 0) and (n_max_rd > 0):
        m = ("The algorithm reached maximum allowed"
             "\nrecursion depth (="+str(MAX_REC_DEPTH)+") "+str(n_max_rd)+" times"
             "\nThe result may be wrong!"
            )
        message_handler(m)
    return cp4_list, rel_error, n_max_rd

# -----------------
# tangent_direction
# -----------------
# Compute, for the input arc, a direction vector of the tangent at
# the input parameter value.
#
# Args:
# - f:     a vector valued function;
# - t:     a parameter value for f;
# - delta: a small increment for t, to be used in the calculations,
#          positive or negative:
#          the tangent is calculated from points
#          f(t), f(t+delta), f(t+2*delta), ..., so the sign of delta
#          determines which of the two directions along the curve is
#          taken into account;
# - algorithm: one of 0,1,2,3 (default=3): the choice of the algorithm.
#              If a value greater than 3 is inserted, the value 3 is used.
#              If a value less than 0 is inserted, the value 0 is used.
#
# Returns: A direction vector [dx,dy] of the tangent; the length and
#          orientation of the vector are arbitrary. 
#
# Note: The tangent is calculated from points
#       f(t), f(t+delta), f(t+2*delta), ...
# Algorithms:
# 0: uses two points:   f(t) and f(t+delta)
# 1: uses three points: f(t), f(t+delta), f(t+2*delta) (fits a circle)
# 2: uses four points:  f(t), f(t+delta), ... (fits a curve of degree 2)
# 3: uses eight points: f(t), f(t+delta), ... (fits a curve of degree 3)
def tangent_direction(f, t, delta, algorithm=3):
    n = algorithm
    n = max(n,0) # If n < 0, value 0 is used.
    n = min(n,3) # If n > 3, value 3 is used.
    if n == 0: # line through f(t) and f(t+delta)
        x0,y0 = f(t)
        x1,y1 = f(t+delta)
        dx = x1-x0
        dy = y1-y0
        return [dx,dy]
    if n == 1: # fit a circle to the curve
        x0,y0 = f(t)
        x1,y1 = f(t+delta)
        x2,y2 = f(t+2*delta)
        c1 = (-y2+y1)*(x2-x0) + (x2-x1)*(y2-y0)
        c2 = (x2-x1)*(x2-x0) + (y2-y1)*(y2-y0)
        dx = c1*(-y0+y1) + c2*(x0-x1)
        dy = c1*(x0-x1) + c2*(y0-y1)
        return [dx,dy]
    if 2 <= n:
        # fit a curve of degree 2: ax^2+bxy+cy^2+z0x+z1y=z2
        # or of degree 3: ax^3+bx^2y+cxy^2+dy^3+ex^2+fxy+gy^2+z0x+z1y=z2
        x,y = [],[]
        x0,y0 = f(t)
        # Make (x0,y0) origo. Then z2=0 and (dx,dy)=(z0,z1).
        for i in range(((n+1)*(n+2))//2 - 2):
            x.append(f(t+(i+1)*delta)[0] - x0)
            y.append(f(t+(i+1)*delta)[1] - y0)
        scaling = 0.
        for i in range(len(x)):
            scaling += abs(x[i])
            scaling += abs(y[i])
        scaling /= (2*len(x))
        try:
            for i in range(len(x)):
                x[i] = x[i]/scaling
                y[i] = y[i]/scaling
        except ZeroDivisionError:
            mess = "tangent_direction"
            mess += "\n    Error: perhaps the parametric curve is locally constant?"
            raise ParametricCurveError(mess)
        matrix = []
        for i in range(len(x)):
            if n == 2:
                matrix.append( [x[i]**2, x[i]*y[i], y[i]**2, x[i], y[i]] )
            if n == 3:
                matrix.append( [ x[i]**3, (x[i]**2)*y[i], x[i]*(y[i]**2), y[i]**3,
                                 x[i]**2, x[i]*y[i], y[i]**2, 
                                 x[i], y[i] ] )
        reduced = reducematrix(matrix)
        dx,dy = reduced[0][0],reduced[0][1]
        return [dx,dy]

# ------------
# reducematrix
# ------------
# Currently needed only to handle cusps!
#
# Given an n x (n+1) matrix (n>=2), reduce it to a 1x2 matrix by means
# of row operations, always discarding one row and one column,
# and keeping the two right-most columns.
def reducematrix(a):
    if len(a) <= 1:
        return a
    imax = jmax = 0
    amax = a[0][0]
    for i in range(len(a)): # search for largest element
        for j in range(len(a[i])-2):
            if abs(a[i][j]) > amax:
                imax,jmax,amax = i,j, float(a[i][j])
    rowmax = []
    try:
        for j in range(len(a[imax])): # Divide row a[imax] by amax
            rowmax.append( a[imax][j] / amax )
    except ZeroDivisionError:
        for j in range(len(a[imax])): # Divide by 1
            rowmax.append( a[imax][j])
    newmatrix = []
    for i in range(len(a)): # 
        if i == imax:
            continue
        newrow = []
        for j in range(len(a[i])):
            if j == jmax:
                continue
            c = a[i][jmax]
            newrow.append( a[i][j] - c*rowmax[j] )
        newmatrix.append( newrow )
    if len(newmatrix)==1:
        return newmatrix
    else:
        return reducematrix(newmatrix)


#==============================================================
#==============================================================
#=============  The plugin proper starts here  ================
#==============================================================
#==============================================================
### Changes from 1.08 marked: CHANGE FROM 1.08
### NOT TESTED!!!

#==============================================================
#==============================================================
#                Classes for user interface
#==============================================================
#==============================================================

class ParametricCurveCollage:
    """A collection of objects of class ParametricCurve together with
    a name and possibly the bounding box of the collection.
    
    The purpose of the class 'ParametricCurveCollage' is to enable
    drawing simultaneously several parametric curves in the same
    picture using the same settings and options.
    
    Initialization arguments:
    - pcurve_list:  [ParametricCurve]
    - collage_name: string or None
    
    Attributes:
        pcurve_list:  [ParametricCurve];
        collage_name: the name of the collage;
        bounding_box: BoundingBox or None
                      (must be initialized by the method
                      'compute_bounding_box');
    
    Methods:
        __init__
        compute_bounding_box
    """
    def __init__(self,
            pcurve_list  = None,
            collage_name = None
            ):
        if pcurve_list is None:
            mess = "No parametric function given."
            raise ParametricCurveError(mess)
        elif len(pcurve_list) == 0:
            mess = "No parametric function given."
            raise ParametricCurveError(mess)
        else:
            self.pcurve_list = pcurve_list
        if collage_name is None:
            name_list = [pc.pcurve_name for pc in pcurve_list]
            self.collage_name = ', '.join(name_list)
        else:
            self.collage_name = collage_name
        self.bounding_box = None
    def compute_bounding_box(self):
        if len(self.pcurve_list) > 0:
            # Initialize bounding box of each pcurve in the collage:
            for pc in self.pcurve_list:
                if pc.bounding_box is None:
                    pc.compute_bounding_box()
            xmin_list = [pc.bounding_box.xmin for pc in self.pcurve_list]
            xmax_list = [pc.bounding_box.xmax for pc in self.pcurve_list]
            ymin_list = [pc.bounding_box.ymin for pc in self.pcurve_list]
            ymax_list = [pc.bounding_box.ymax for pc in self.pcurve_list]
            self.bounding_box = BoundingBox(
                xmin = min(xmin_list),
                xmax = max(xmax_list),
                ymin = min(ymin_list),
                ymax = max(ymax_list)
                )

class ParametricCurve:
    """A parametric curve in the xy-plane.
    
    Initalization arguments:
    - function:    callable (function R-> R2 in Python)
    - pcurve_name: string
    - start:       float
    - end:         float
    - closed:      boolean
    - custom:      CustomPoints or None
    - custom_gaps: float or None          CHANGE FROM 1.08
    
    Attributes:
        function:        vector valued parametric function in Python
                         syntax (the default 'None' causes
                         the program to abort immediately);
        pcurve_name:     a string: the name of the curve
                         (default="parametric curve");
        lo:              the lowest value of the parameter
                         (default=0.);
        hi:              the highest value of the parameter
                         (default=1.);
        closed:          Boolean (default=False);
        custom:          CustomPoint (default=None);
        custom_gaps:     float or None                    CHANGE FROM 1.08
                         the upper limit, taken relative to the length of
                         interval, of gaps in the list of the parameter
                         values of custom points;
                         (default=1e-6, forced minimum=1e-14).
        reversed:        Boolean: set to True if in __init__() the
                         arguments 'start' and 'end' satisfy
                         start > end, False otherwise;
        bounding_box:    BoundingBox or None (must be
                         initialized by the method
                         'compute_bounding_box' before use);
        bounding_params: list of parameter values at which the curve
                         meets the bounding box, or None
                         (must be initialized before use by the method
                         'compute_bounding_box').
    
    Methods:
        __init__
        compute_bounding_box
        prepare_custom
    """
    # __init__: Instead of 'lo' and 'hi', initialization takes
    #           arguments 'start' and 'end' where it is allowed that
    #           start > end. The attributes 'lo' and 'hi' are then set
    #           so that lo < hi, and the attribute 'reversed' is set
    #           to indicate if start > end.
    #           If start = end, an error is raised.
    def __init__(self,
            function    = None,
            pcurve_name = "parametric curve",
            start       = 0.,
            end         = 1.,
            closed      = False,
            custom      = None,
            # Setting custom_gaps to None value custom_gaps=0. is used (risky!)
            custom_gaps = 1e-6,  # undocumented CHANGE FROM 1.08
            ):
        if function is None:
            mess = "No parametric function given."
            raise ParametricCurveError(mess)
        else:
            self.function = function
        self.pcurve_name  = pcurve_name
        start, end = float(start), float(end)
        if start < end:
            self.lo, self.hi, self.reversed = start, end, False
        elif start > end:
            self.lo, self.hi, self.reversed = end, start, True
        else: # start = end (Note: would better to use tolerance, but how?)
            mess = "Zero parameter interval on curve "+pcurve_name+" not allowed!"
            error_message_function(mess)      # Give a message to Gimp
            raise ParametricCurveError(mess)
        self.closed   = closed
        if custom is None:
            self.custom = CustomPoints()
        else:
            self.custom = custom
        if custom_gaps is None:        # CHANGE FROM 1.08
            self.custom_gaps = 1e-14                  # Forced minimum value
        else:
            self.custom_gaps = max(1e-14,custom_gaps) # Forced minimum value
        self.bounding_box = None
        self.bounding_params = None
    
    # --------------------
    # compute_bounding_box
    # --------------------
    # Complete the fields 'bounding_box' and 'bounding_params' of self.
    # Here 'bounding_params' is a list of parameters, not perhaps quite
    # exact and not quite exchaustive, such that at those parameter
    # values the curve meets the boundary of the bounding box.
    def compute_bounding_box(self):
        bb, bpar = bounding_box_of_curve(self.function,self.lo,self.hi)
        self.bounding_box = bb
        self.bounding_params = bpar
    # --------------
    # prepare_custom
    # --------------
    # Prepare the 'custom' field of 'self' by combining
    # its fields 'params', 'points', and 'screen_points' into the
    # field 'params' by computing the parameter values for the custom
    # points; the other two will be set to [].
    # If original 'screen_params' is not [], then an object of class of
    # ScreenCoordinateSettings is needed as input 'scoordsettings';
    # however, if original 'screen_params' is [], then the argument
    # 'scoordsettings' has no effect and can be anything or missing.
    # For more explanations, see the notes above the routine
    # 'combine_custom' called here.
    def prepare_custom(self, scoordsettings=None):
        customparameters = combine_custom(self, scoordsettings)
        self.custom = CustomPoints(params = customparameters)

class CustomPoints:
    """User-defined points on the parametric curve which the user
    wants to be forced as anchor points of the approximate Bezier curve,
    in addition to the anchor points which the program itself makes.
    
    The custom points can be given in any of the following three forms,
    or even in two or three forms simultaneously. Custom points is the
    means for the user to provide, for example, exact cusp points or
    inflection points - rather than just relying on what the program
    itself finds, which is always a little inexact.
    
    Here "raw coordinates" means the coordinate system in which the
    input parametric curve is defined, and "screen coordinates" means
    the coordinate system of the screen of the drawing application.
    
    Attributes (the same as initialization arguments):
        params:        a list [...,t,...] of custom parameter values t
                       such that each f(t) will be a custom point
                       (default=[]);
        points:        a list [...,[x,y],...] of custom points [x,y] in
                       raw coordinates
                       (default=[]);
        screen_points: a similar list [...,[x,y],...] of custom points
                       [x,y] but in screen coordinates
                       (default=[]).
    """
    def __init__(self,
            params = [],
            points = [],
            screen_points = []
            ):
        self.params        = params
        self.points        = points
        self.screen_points = screen_points

class Curve2BezierSettings:
    """Numerical parameters to manage how a parametric curve is
    resolved approximately as a Bezier curve.
    
    The program works by subdividing the input curve into a succession of
    arcs; the endpoints of the arcs will then be anchor points of the
    resulting Bezier curve. (The program may create also additional
    anchor points.) The rules for how the subdivision is built, is
    governed by the following attributes:
    
    Attributes (the same as initializatin arguments except for the last two):
        angle:         the maximum allowed turn in each subdivision arc
                       (forced to between 30 and 180
                       degrees with tolerance of 1e-8)
                       (default=90 degrees);
        refinement:    higher refinement forces refinement of the
                       subdivision along the whole curve, increasing
                       the number of anchor points otherwise used
                       (default=0);
        max_error:     maximum allowed relative error
                       ('max_error' is forced to between 0.0005 and .5
                       but these are arbitrary limits)
                       (default=0.01);
        max_recursion: if the error (in initial computation), for some
                       arc of the subdivision, appears to be larger
                       than the allowed maximum ('max_error'), then the
                       program goes to recursion by splitting the
                       troublesome arc in two, so increasing the number
                       of anchor points; the value 'max_recursion' is
                       the maximum allowed recursion depth; setting it
                       to 0 suppresses recursion
                       (default=3);
        steps:         the number of steps to be used when searching
                       for special points
                       (default=500);
        short_arc:     the upper limit, taken relative to the bounding
                       box of the curve, of length of the arcs in the
                       subdivision (some shorter arcs may be allowed)
                       (default=0.001).
    """
    
    def __init__(self, 
            angle          = pi/2,
            refinement     = 0,
            max_error      = 0.01,
            max_recursion  = 3,
            steps     = 500,  # undocumented CHANGE FROM 1.08
            short_arc = 0.001 # undocumented CHANGE FROM 1.08
            ):
        #DEFAULT_steps         = 500    # Moved to arguments CHANGE FROM 1.08
        #DEFAULT_short_arc     = 0.001  # Moved to arguments CHANGE FROM 1.08
        MIN_angle,     MAX_angle     = pi/6-1e-8, pi+1e-8
        MIN_max_error, MAX_max_error = 0.0001, .1 # arbitrary
        self.angle         = max(MIN_angle, min(angle, MAX_angle))
        self.refinement    = max(0, refinement)
        self.max_error     = max(MIN_max_error, min(MAX_max_error, max_error))
        self.max_recursion = max(0, max_recursion)
        self.steps         = steps     #  CHANGE FROM 1.08: taken from arguments
        self.short_arc     = short_arc #  CHANGE FROM 1.08: taken from arguments
        if not MIN_angle <= angle <= MAX_angle:
            mess1 = "Only angles in [pi/6, pi] are allowed."
            mess2 = "\n    Instead of {:1.6f}, the angle {:1.6f} radians is used".format(angle,self.angle)
            mess = mess1+mess2
            error_message_function(mess) # Tell the forced change of angle.
        if not MIN_max_error <= max_error <= MAX_max_error:
            mess1 = "Only max_errors in [{:1.6f}, {:1.6f}] are allowed.".format(MIN_max_error, MAX_max_error)
            mess2 = "\n    Instead of {:1.6f}, the value {:1.6f} is used".format(max_error,self.max_error)
            mess = mess1+mess2
            error_message_function(mess) # Tell the forced change of max_error.

class Curve2BezierOptions:
    """Boolean parameters to manage which special points are searched for
    and included among subdivision points (in addition to the
    user-defined custom points if any).
    
    The default values should be safe. Other choices may cause the
    algorithm to fail but often may give interesting results.
    
    Attributes (all Boolean, the same as initialization arguments):
        cusp_points:           include cusp points
                               (default=True);
        high_curvature_points: include points of locally highest
                               curvature
                               (default=True);
        low_curvature_points:  include points of locally lowest
                               curvature
                               (default=True);
        straight_end_points:   include both start and end points
                               of straight line segments
                               (default=True);
        inflection_points:     include inflection points
                               (the program forces this to be
                               - False if 'low_curvature_points'=True,
                               - True if 'low_curvature_points'=False
                                        and 'angle_turn_points'=True)
                               (default=False);
        angle_turn_points:     include subdivision points to ensure
                               that each arc turns at most by the
                               amount of 'angle' (set in the attribute
                               'angle' in an input argument of class
                               Curve2BezierSettings)
                               (default=True).
    """
    
    def __init__(self, 
            cusp_points           = True,
            high_curvature_points = True,
            low_curvature_points  = True,
            straight_end_points   = True,
            inflection_points     = False,
            angle_turn_points     = True
            ):
        self.cusp_points           = cusp_points
        self.high_curvature_points = high_curvature_points
        self.low_curvature_points  = low_curvature_points
        self.straight_end_points   = straight_end_points
        self.angle_turn_points     = angle_turn_points
        if low_curvature_points:
            self.inflection_points = False
        elif angle_turn_points:
            self.inflection_points = True
        else:
            self.inflection_points = inflection_points

class CoordinateSettings:
    """Manage how the final plot is positioned on the screen.
    
    Attributes (the same as initalization arguments): 
        fit_window:   Boolean: determines if the plot is to be fit in
                      the window minus the paddings
                      (when 'fit_window' is True, the settings for
                      'origo', and 'scale' are ignored)
                      (default=True);
        padding:      float: the padding to be left empty on the screen
                      as a fraction of the window size when 'fit_window'
                      is True (it is forced that 0 <= padding <= 0.45)
                      (default=0.);
        origo:        a pair of floats: where to set the origo on the
                      screen in screen coordinates
                      (ignored if 'fit_window' is True)
                      (default=[500,500]);
        scale:        scaling factor from raw coordinates to screen
                      coordinates
                      (ignored if 'fit_window' is True)
                      (default=100.).
    """
    
    def __init__(self, 
            fit_window = True,
            padding    = 0.,
            origo      = [500.,500.],
            scale      = 100.
            ):
        self.fit_window = fit_window
        self.padding = float(min(0.45, max(padding, 0.)))
        self.origo  = [float(origo[0]), float(origo[1])]
        self.scale = float(scale)

class DrawingOptions:
    """Mainly Boolean parameters to manage which objects are to be
    drawn.
    
    Here "drawing" means creating a separate Gimp vectors object,
    inserting it in Gimp, and making visible.
    
    The 'pcurves' mentioned below are the elements of
    'pcurve_input.pcurve_collage.pcurve_list' where 'pcurve_input'
    is an object of class ParametricCurveInput. Thus, 'pcurve_list' is
    the list of parametric curves to be drawn simultaneously in the
    same picture using the same settings and options.
    
    Attributes (the same as initialization arguments):
        bezier_curves:  Boolean: if True, then the resulting Bezier
                        curves (approximations of the elements 
                        'pcurves') are not only computed but also
                        drawn, each Bezier curve individually
                        (default=True);
        bezier_collage: Boolean: if True, then from the Bezier curves
                        mentioned above, a combined object is created
                        and drawn (in Gimp this means one vectors
                        object where each one of the 'pcurves' appears
                        as one stroke)
                        (default=False);
        axes:           Boolean: if True, then the coordinate axes are
                        marked as guides on the screen;
                        (default=False);
        x_axis:         Boolean: if True, then the x axis is
                        marked as a guide on the screen;
                        (default=False);
        y_axis:         Boolean: if True, then the y axis is
                        marked as a guide on the screen;
                        (default=False);
        exact_pcurves:  Boolean: if True, a sparse set of points on
                        each exact input parametric curve is drawn
                        (default=False);
        handles:        Boolean: if True, the anchors and handles of
                        the computed Bezier curve are drawn
                        (default=False);
    """
    def __init__(self,
            bezier_curves  = True,
            bezier_collage = False,
            axes           = False,
            x_axis         = False,
            y_axis         = False,
            exact_pcurves  = False,
            handles        = False
            ):
        self.bezier_curves  = bezier_curves
        self.bezier_collage = bezier_collage
        self.axes           = axes
        self.x_axis         = x_axis
        self.y_axis         = y_axis
        self.exact_pcurves  = exact_pcurves
        self.handles        = handles


#==============================================================
#==============================================================
#                Classes etc for internal use
#==============================================================
#==============================================================

def gimp_message(txt, h=2):
    # 0 = message box; 1 = console; 2 = error console
    pdb.gimp_message_set_handler(h)
    pdb.gimp_message(txt)

def str_float(x): return '{:1.5f}'.format(x)

class ExitError(Exception): # Only for debugging purposes.
    def __init__(self,message):
        self.message = message
    def __str__(self):
        return 'EXIT: '+self.message

class ParametricCurveError(Exception):
    def __init__(self,message):
        self.message = message
    def __str__(self):
        return self.message


class ScreenCoordinateSettings:
    """The settings needed for conversion from raw coordinates to
    screen coordinates.
    
    Here "raw coordinates" means the coordinate system in which
    the input parametric curve is defined, and "screen coordinates"
    means the coordinate system of the screen of the drawing
    application.
    
    Attributes:
        origo:          [x,y] = the position in screen coordinates
                        where the origo of the plot should be placed;
        scalex, scaley: floats, the scaling factors from raw
                        coordinates to screen coordinates;
        window_SW, window_NE: the SW and NE corners of the window
                              in screen coordinates
                              (not used anywhere!);
        padded_SW, padded_NE: the SW and NE corners of the window
                              minus paddings, in screen
                              coordinates.
    """
    def __init__(self,
            origo,
            scalex,
            scaley,
            window_SW,
            window_NE,
            padded_SW,
            padded_NE,
            ):
        self.origo     = origo
        self.scalex    = scalex
        self.scaley    = scaley
        self.window_SW = window_SW # In screen coordinates
        self.window_NE = window_NE # In screen coordinates
        self.padded_SW = padded_SW # In screen coordinates
        self.padded_NE = padded_NE # In screen coordinates

class SubDivPoint:
    """Contains data about a parametric curve at one subdivision point.
    
    Attributes:
        type:             the type of the subdivision point (string);
        parameter:        the value of the parameter at the point;
        functionvalue:    the value of the patametric function
                          (i, e., a point on the curve);
        unittangentleft:  the tangent direction on the left side of
                          the point, of length one but may point at
                          either of the two directions;
        unittangentright: the tangent direction on the right side of
                          the point, of length one but may point at
                          either of the two directions.
    """
    subdivtypes = ['custom',
                   'low end',
                   'high end',
                   'cusp',
                   'high curvature',
                   'low curvature',
                   'straight end',
                   'inflection',
                   'angle turn'
                   ]
    def __init__(self,
            type,
            parameter,
            functionvalue,
            unittangentleft  = None,
            unittangentright = None
            ):
        if type not in SubDivPoint.subdivtypes:
            mess = "Invalid type for SubDivPoint: "+type
            #print(mess)
            #print("allowed types:",subdivtypes)
            raise ParametricCurveError(mess)
        self.type          = type
        self.parameter     = parameter
        self.functionvalue = functionvalue
        self.unittangentleft  = unittangentleft
        self.unittangentright = unittangentright

class BoundingBox:
    """Bounding box.
    
    Attributes: xmin, xmax, ymin, ymax.
    
    Methods:
        __init__
        make_copy
        add_origo
    """
    def __init__(self, xmin, xmax, ymin, ymax):
        self.xmin = xmin
        self.xmax = xmax
        self.ymin = ymin
        self.ymax = ymax
    def make_copy(self):
        new = BoundingBox(
            xmin = self.xmin,
            xmax = self.xmax,
            ymin = self.ymin,
            ymax = self.ymax
            )
        return new
    def add_origo(self):
        self.xmin = min(self.xmin, 0.)
        self.xmax = max(self.xmax, 0.)
        self.ymin = min(self.ymin, 0.)
        self.ymax = max(self.ymax, 0.)

class ArcData:
    """Data about a vector valued parametric arc: function, lo, hi,
    a list of values of the function at several points (default=1000),
    and unit direction vectors of tangents at both ends.
    
    Attributes:
        function:        vector valued parametric function;
        lo:              float: the lowest value of the parameter;
        hi:              float: the highest value of the parameter;
        function_values: list [ ..., [t,f(t)], ... ] where t runs a
                         set of parameter values in increasing order,
                         and for each t, f(t) is the function value,
                         f(t) = [x,y].
        unittangent_lo:  the tangent direction at the lo end of the
                         arc, of length one but may point at either
                         of the two directions;
        unittangent_hi:  the tangent direction at the hi end of the
                         arc, of length one but may point at either
                         of the two directions;
        steps:           the number of steps for the parameter in
                         initialization of function_values
                         (default=1000).
    """
    def __init__(self,
            f,
            start,
            end,
            tangent_start = None,
            tangent_end   = None,
            steps = 1000
            ):
        lo, hi = min(start,end), max(start,end)
        self.function = f
        self.lo = lo
        self.hi = hi
        self.function_values = curve_step_by_step(f,lo,hi, steps)
        try:
            x,y = tangent_start
            l = sqrt(x**2+y**2)
            tstart = [x/l, y/l]
            x,y = tangent_end
            l = sqrt(x**2+y**2)
            tend = [x/l, y/l]
            if lo <= hi:
                self.unittangent_lo = tstart
                self.unittangent_hi = tend
            else:
                self.unittangent_lo = tend
                self.unittangent_hi = tstart
        except:
            self.unittangent_lo = None
            self.unittangent_hi = None


#######################################################
##                                                   ##
##           General mathematical routines           ##
##                                                   ##
#######################################################

def cosine_between_vectors(v,w):
    vw = v[0]*w[0] + v[1]*w[1]
    vv = v[0]*v[0] + v[1]*v[1]
    ww = w[0]*w[0] + w[1]*w[1]
    try:
        return vw / sqrt(vv*ww)
    except ZeroDivisionError as e:
        mess = "_between_vectors: "+str(e)
        raise ParametricCurveError(mess)

def angle_between_vectors(v,w):
    vw = v[0]*w[0] + v[1]*w[1]
    vv = v[0]*v[0] + v[1]*v[1]
    ww = w[0]*w[0] + w[1]*w[1]
    try:
        return acos( vw / sqrt(vv*ww) )
    except ZeroDivisionError as e:
        mess = "angle_between_vectors: "+str(e)
        raise ParametricCurveError(mess)
    except ValueError as e:
        arg = vw / sqrt(vv*ww)
        if arg >= 1.:
            return 0.
        elif arg <= -1.:
            return pi
        else:
            mess = "angle_between_vectors: "+str(e)
            raise ParametricCurveError(mess)

# -----------
# extrapolate
# -----------
# Given a vector-valued function g and two points (t1,g(t1)) and
#(t2,g(t2)), and a third value for the argument t,
# compute (t, g(t)) by linear extrapolation.
def extrapolate(t, t1, t2, gt1, gt2):
    v0 = (t2-t)*gt1[0] + (t-t1)*gt2[0]
    v1 = (t2-t)*gt1[1] + (t-t1)*gt2[1]
    return [ v0/(t2-t1), v1/(t2-t1) ]

# ------------
# find_minimum
# ------------
# Assume that the function g(x) has a single minimum in [lo,hi].
# Returns:
# - the minimum point as (x,g(x)),
# - a short interval containing x for the minimum.
def find_minimum(g, lo, hi, accuracy=1e-5, max_accuracy=1e-10, tolerance=1e-10):
    class xgx:
        def __init__(self, g, x):
            self.x = x
            self.gx = g(x)
    def find_smallest_i(a): # a is an array of elements of type xgx
        smallest_i, smallest_gx = [0], a[0].gx
        for i in range(1,len(a)):
            if a[i].gx < smallest_gx:
                smallest_i, smallest_gx = [i], a[i].gx
            elif a[i].gx == smallest_gx:
                smallest_i.append(i)
        return smallest_i
    def middle_point(g,interval):
        x1,x2 = interval
        x = (x1+x2)/2
        return x,g(x)
    accuracy = min(accuracy, max_accuracy)
    a = [xgx(g, lo), 
         xgx(g, 0.75*lo + 0.25*hi),
         xgx(g, 0.5*lo + 0.5*hi),
         xgx(g, 0.25*lo + 0.75*hi),
         xgx(g, hi)]
    z = 4 # highest index in a
    while abs(a[0].x - a[z].x) > accuracy:
        smallest_i = find_smallest_i(a)
        if len(smallest_i) == 1:
            si = smallest_i[0]
            if si == 0:
                lx, mx, hx = a[0].x, a[1].x, a[2].x
                a = [a[0],
                     xgx(g, 0.5*lx + 0.5*mx),
                     a[1],
                     xgx(g, 0.5*mx + 0.5*hx),
                     a[2]]
            elif si == z:
                lx, mx, hx = a[z-2].x, a[z-1].x, a[z].x
                a = [a[z-2],
                     xgx(g, 0.5*lx + 0.5*mx),
                     a[z-1],
                     xgx(g, 0.5*mx + 0.5*hx),
                     a[z]]
            else:
                lx, mx, hx = a[si-1].x, a[si].x, a[si+1].x
                a = [a[si-1],
                     xgx(g, 0.5*lx + 0.5*mx),
                     a[si],
                     xgx(g, 0.5*mx + 0.5*hx),
                     a[si+1]]
        elif len(smallest_i) == 2:
            s1, s2 = smallest_i
            if s2-s1 != 1: # Should not happen but happens (computing accuracy?)
                s1,s2 = min(s1,s2), max(s1,s2)
                gs2 = a[s2].gx
                for ai in a[s1:s2]:
                    if abs(ai.gx - gs2) > tolerance:
                        mess = "find_minimum: impossible: not in tolerance"
                        raise ParametricCurveError(mess)  # SLOPPY?
                interval = [a[s1].x, a[s2].x]
                return middle_point(g, interval), interval
            lx, hx = a[s1].x, a[s2].x
            a = [a[s1],
                 xgx(g, 0.75*lx + 0.25*hx),
                 xgx(g, 0.5*lx + 0.5*hx),
                 xgx(g, 0.25*lx + 0.75*hx),
                 a[s2]]
        else:
            list_smallest = [[a[sm_i].x, g(a[sm_i].x)] for sm_i in smallest_i]
            interval = [list_smallest[0][0], list_smallest[-1][0]]
            return middle_point(g, interval), interval
    smallest_i = find_smallest_i(a)
    if len(smallest_i) == 1:
        si = smallest_i[0]
        interval = [a[si].x, a[si].x]
        return middle_point(g, interval), interval
    elif len(smallest_i) == 2:
        s1, s2 = smallest_i
        interval = [a[s1].x, a[s2].x]
        return middle_point(g, interval), interval
    else:
        list_smallest = [[a[sm_i].x, g(a[sm_i].x)] for sm_i in smallest_i]
        interval = [list_smallest[0][0], list_smallest[-1][0]]
        return middle_point(g, interval), interval

# --------------
# circle_radius2
# --------------
# Return the squared radius of the circle through the three points.
def circle_radius2(a,b,c):
    ab = [a[0]-b[0], a[1]-b[1]]
    ac = [a[0]-c[0], a[1]-c[1]]
    bc = [b[0]-c[0], b[1]-c[1]]
    acbc  = ac[0]*bc[0] + ac[1]*bc[1]
    abxbc = -ab[1]*bc[0] + ab[0]*bc[1]
    try:
        z = acbc/abxbc
    except:
        return None
    ab2 = ab[0]**2 + ab[1]**2
    R2 = .25 * ab2 * ( 1 + z**2 )
    return R2

# ------------------
# quadratic_extremum
# ------------------
# For a function f with f(x0-c)=left, f(x0)=middle, f(x0+c)=right, 
# return the minimum or maximum point by approximating with a parabola.
def quadratic_extremum(x0, c, left, middle, right):
    nomin = float(left - right)
    denom = float(left - 2*middle + right)
    try:
        x = nomin/(2*denom)
    except ZeroDivisionError: # straight line
        return None
    y = ( denom*x*x - nomin*x )/2 + middle
    return [x0+c*x, y]

# --------------------
# average_close_floats
# --------------------
# Remove from a sorted list of float numbers any too close ones by
# replacing them with their average.
#
# Args:
#   - flist:           sorted list of float numbers (ascending order);
#   - closeness_limit: float number > 0.
#
# Returns: new list otherwise similar to the input list except that any
#          succession of numbers with differences between any two
#          successive items is less than or equal to 'closeness_limit'
#          is replaced with the average.
#          In the new list the difference of any successive items is
#          guaranteed to be greater than 'closeness_limit'.
def average_close_floats(flist, closeness_limit):
    old_list = flist
    new_list = []
    i = 0
    while i < len(old_list):
        cum = old_list[i]
        ncum = 1
        j = i+1
        while j < len(old_list):
            if old_list[j] - old_list[j-1] > closeness_limit:
                break
            else:
                cum += old_list[j]
                ncum += 1
                j += 1
        new_list.append(cum/ncum) # replace close points with average
        i = j
    return new_list

#==============================================================
#==============================================================
#               Some routines for parametric curves
#==============================================================
#==============================================================

# ----------------
# is_straight_line
# ----------------
# Decide if the input arc is a straight line segment.
#
# Args:
# - f: a parametric arc (vector valued function);
# - start, end: the starting and ending values of the parameter;
# - rel_accuracy: determines how dense subdivision the algorithm uses
#                 (default=0,00001);
# - tolerance: determines how large deviations from a straight line is
#              tolerated (default=1e-8)).
# Returns: boolean
def is_straight_line(f, start, end, rel_accuracy=1e-5, tolerance = 1e-8):
    f0,f3 = f(start), f(end)
    f0f3 = [ f3[0]-f0[0], f3[1]-f0[1] ]
    lenf0f3 = sqrt(f0f3[0]**2 + f0f3[1]**2)
    if lenf0f3 > tolerance:
        u = [f0f3[0]/lenf0f3, f0f3[1]/lenf0f3]
    else:
        u = [f0[0]+1, f0[1]] # ???
    def is_on_line(point):
        point0 = [point[0]-f0[0], point[1]-f0[1]]
        return abs( -point0[0]*u[1] + point0[1]*u[0] ) < tolerance
    accuracy = float(end-start)*rel_accuracy
    product = 2
    nsteps = 3
    stept = (end-start)/nsteps
    while stept > accuracy:
        param = start + stept
        point = f(param)
        stop = end - stept/2.
        while param < stop:
            if not is_on_line(point):
                return False
            param += stept
        product *= nsteps
        nsteps = product+1
        stept = (end-start)/nsteps
    return True

# ---------------------
# bounding_box_of_curve
# ---------------------
# Return the bounding box of a parametric curve as [SW,NE] in raw
# coordinates.
#
# Args:
# - f:      a parametric curve (vector valued function);
# - lo, hi: the lowest and highest parameter values of the curve;
# - steps:  the number of subdivision intervals used in the
#           the algorithm (default=1000);
# - accuracy, max_accuracy: float numbers (defaults = 0.00001, 0.000001)).
#
# Returns:
# - BoundingBox;
# - a list of parameters for points where the curve meets the boundary
#   of the bounding box.
def bounding_box_of_curve(f,lot,hit, steps=1000, accuracy=1e-5, max_accuracy=1e-6):
    #print("bounding_box_of_curve called with lot, hit =",lot,hit)
    fvalues = curve_step_by_step(f,lot, hit, steps)
    #print("fvalues[:4] =",fvalues[:4])
    #print("fvalues[-5:] =",fvalues[-5:])
    first_interval = [fvalues[0][0], fvalues[2][0]]
    last_interval = [fvalues[-3][0], fvalues[-1][0]]
    xmax_intervals = [first_interval, last_interval]
    xmin_intervals = [first_interval, last_interval]
    ymax_intervals = [first_interval, last_interval]
    ymin_intervals = [first_interval, last_interval]
    for i in range(len(fvalues)-3):
        t0,f0 = fvalues[i]
        t1,f1 = fvalues[i+1]
        t2,f2 = fvalues[i+2]
        t3,f3 = fvalues[i+3]
        if (f0[0] <= f1[0]) and (f2[0] >= f3[0]):
            xmax_intervals.append([t0,t3])
        elif (f0[0] >= f1[0]) and (f2[0] <= f3[0]):
            xmin_intervals.append([t0,t3])
        if (f0[1] <= f1[1]) and (f2[1] >= f3[1]):
            ymax_intervals.append([t0,t3])
        elif (f0[1] >= f1[1]) and (f2[1] <= f3[1]):
            ymin_intervals.append([t0,t3])
    xmax_candidates = []
    xmin_candidates = []
    ymax_candidates = []
    ymin_candidates = []
    t_xmax_candidates = []
    t_xmin_candidates = []
    t_ymax_candidates = []
    t_ymin_candidates = []
    def fx(t): return f(t)[0]
    def minusfx(t): return -f(t)[0]
    def fy(t): return f(t)[1]
    def minusfy(t): return -f(t)[1]
    for t0,t3 in xmax_intervals:
        try:
            t, gt = find_minimum(minusfx, t0, t3, accuracy, max_accuracy)[0]
        except ParametricCurveError as e:
            mess = "bounding_box_of_curve/xmax\n    "+str(e)
            xmax_candidates.append(f((t0+t3)/2)[0])
            continue
            #raise ParametricCurveError(mess)
        xmax_candidates.append(-gt)
        t_xmax_candidates.append(t)
    for t0,t3 in xmin_intervals:
        try:
            t, gt = find_minimum(fx, t0, t3, accuracy, max_accuracy)[0]
        except ParametricCurveError as e:
            mess = "bounding_box_of_curve/xmin\n    "+str(e)
            xmin_candidates.append(f((t0+t3)/2)[0])
            continue
            #raise ParametricCurveError(mess)
        xmin_candidates.append(gt)
        t_xmin_candidates.append(t)
    for t0,t3 in ymax_intervals:
        try:
            t, gt = find_minimum(minusfy, t0, t3, accuracy, max_accuracy)[0]
        except ParametricCurveError as e:
            mess = "bounding_box_of_curve/ymax\n    "+str(e)
            ymax_candidates.append(f((t0+t3)/2)[1])
            continue
            #raise ParametricCurveError(mess)
        ymax_candidates.append(-gt)
        t_ymax_candidates.append(t)
    for t0,t3 in ymin_intervals:
        try:
            t, gt = find_minimum(fy, t0, t3, accuracy, max_accuracy)[0]
        except ParametricCurveError as e:
            mess = "bounding_box_of_curve/ymin\n    "+str(e)
            ymin_candidates.append(f((t0+t3)/2)[1])
            continue
            #raise ParametricCurveError(mess)
        ymin_candidates.append(gt)
        t_ymin_candidates.append(t)
    try:
        tfirst,ffirst = fvalues[0]
        tlast,flast = fvalues[-1]
        xmax = max(ffirst[0], flast[0], max(xmax_candidates))
        xmin = min(ffirst[0], flast[0], min(xmin_candidates))
        ymax = max(ffirst[1], flast[1], max(ymax_candidates))
        ymin = min(ffirst[1], flast[1], min(ymin_candidates))
        t_xmax_candidates += [tfirst, tlast]
        t_xmin_candidates += [tfirst, tlast]
        t_ymax_candidates += [tfirst, tlast]
        t_ymin_candidates += [tfirst, tlast]
    except ValueError as e:
        mess = "bounding_box_of_curve: "+str(e)
        raise ParametricCurveError(mess)
    bb = BoundingBox(xmax=xmax, xmin=xmin, ymax=ymax, ymin=ymin)
    bb_limit = (xmax+ymax-xmin-ymin)/1000 # 1000 arbitrary 
    bb_params = []
    for t in t_xmax_candidates:
        if f(t)[0] > xmax - bb_limit:
            bb_params.append(t)
    for t in t_xmin_candidates:
        if f(t)[0] < xmin + bb_limit:
            bb_params.append(t)
    for t in t_ymax_candidates:
        if f(t)[1] > ymax - bb_limit:
            bb_params.append(t)
    for t in t_ymin_candidates:
        if f(t)[1] < ymin + bb_limit:
            bb_params.append(t)
    bb_params.sort()
    duplicate_limit = (hit - lot)/1000 # arbitrary choice
    minus_duplicates = average_close_floats(bb_params, duplicate_limit)
    bb_params = minus_duplicates
    return bb, bb_params

# ------------------
# curve_step_by_step
# ------------------
# Return a subdivision of a parametric curve, positioning the
# points more densely where the curvature is high.
#
# Args:
# - f:             a parametric arc (vector valued function);
# - start, end:    the starting and ending values of the parameter;
# - steps:         the number of subdivision intervals to be used in the
#                  initial phase of the algorithm;
# - minrefinement: a float number which limits how short subdivision
#                  intervals are allowed (default=0.001);
# - maxbend:       a float number to control how sharp bend is allowed
#                  in each subdivision interval (default=10).
#
# Returns: a list [ ..., [t,f(t)], ... ] where each t is a parameter
#          value and f(t) is the point on the curve (f(t)=[x,y]).
def curve_step_by_step(f,start, end, steps, minrefinement=0.001, maxbend=10):
    mincos2 = cos(maxbend * pi/180)**2
    lo,hi = start, end
    reversed = (lo > hi) 
    if reversed:
        lo,hi = hi, lo
    stept = (hi-lo)/float(steps)
    minstep = stept*minrefinement
    fvalues = []
    t = lo
    while t <= hi-stept:
        fvalues.append([t,f(t)])
        t += stept
    fvalues.append([hi,f(hi)])
    first = 0 # first to study in refining the steps array
    while True:
        try:
            t1,f1 = fvalues[first]
            t2,f2 = fvalues[first+1]
            t3,f3 = fvalues[first+2]
        except IndexError:
            break
        if t2-t1 < minstep:
            first += 1
            continue
        f12 = [ f2[0]-f1[0], f2[1]-f1[1] ]
        f23 = [ f3[0]-f2[0], f3[1]-f2[1] ]
        dot = f12[0]*f23[0] + f12[1]*f23[1]
        f12sq = f12[0]**2 + f12[1]**2
        f23sq = f23[0]**2 + f23[1]**2
        if dot**2 > f12sq * f23sq * mincos2: # bend not too sharp
            first += 1
            continue
        t12, t23 = (t1+t2)/2, (t2+t3)/2
        added = [ [t12,f(t12)] , [t2,f2], [t23,f(t23)] ]
        fvalues = fvalues[:first+1] + added + fvalues[first+2:]
    if not reversed:
        return fvalues
    else:
        return fvalues[::-1]

# --------------------------
# directionvectors_for_curve
# --------------------------
# Return a list [..., [t,tangent(t)], ...], where tangent(t) is, roughly,
# a direction vector of the tangent of the input function f at t.
#
# Args:
# - f:       a parametric curve (vector valued function) having no cusp
#            points in the open interval (lo,hi);
# - lo, hi:  the lowest and highest parameter values of the curve;
# - fvalues: a list [t,f(t)] of points densely along the curve.
# Returns: a list [..., [t,tangent(t)], ...], where tangent(t) is,
#          roughly, a direction vector of the input function f at t.
#
# In fact, generally, tangent(t) is approximately the derivative f'(t).
# It is assumed that there are no cusp points except possibly at the
# end points. If there is a cusp at either end point, then the computed
# tangent(t) gives the correct tangent in that end point and points in
# the general direction of f'(t) close to that end point but has
# arbitrary length. (In a cusp point we usually have f'(t) = 0.)
def directionvectors_for_curve(f,lo,hi, fvalues):
    #fvalues = curve_step_by_step(f,lo,hi, steps) # list of [t,f(t)]
    if len(fvalues) < 30:
        return []
    ftangents = [] # list of [t,f'(t)]
    ## Find tangents at points not first or last.
    t0,f0 = fvalues[0]
    t1,f1 = fvalues[1]
    for t2,f2 in fvalues[2:]:
        diffleft  = [ f1[0]-f0[0], f1[1]-f0[1] ]
        diffright = [ f2[0]-f1[0], f2[1]-f1[1] ]
        tangleft  = [ diffleft[0]/(t1-t0), diffleft[1]/(t1-t0) ]
        tangright = [ diffright[0]/(t2-t1), diffright[1]/(t2-t1) ]
        tang = extrapolate(t1, t0,t2, tangleft, tangright )
        ftangents.append( [ t1, tang ] )
        t0,f0 = t1,f1
        t1,f1 = t2,f2
    ## Find tangent at first point:
    t0,f0 = fvalues[0]
    t1,f1 = fvalues[1]
    delta = t1-t0
    tangent_at_start = tangent_direction(f,t0, delta)
    # Turn the vector to point in the direction of f'(lo).
    f01 = [ f1[0]-f0[0], f1[1]-f0[1] ]
    if tangent_at_start[0]*f01[0] + tangent_at_start[1]*f01[1] < 0:
        tangent_at_start = [ -tangent_at_start[0], -tangent_at_start[1] ]
    len01 = sqrt( f01[0]**2 + f01[1]**2 )
    lentang = sqrt( tangent_at_start[0]**2 + tangent_at_start[1]**2 )
    #print("directionvectors_for_curve: t0,t1,lentang =",t0,t1,lentang)
    try:
        coeff = len01 / ( (t1-t0)*lentang )
        tangent_at_start = [ coeff*tangent_at_start[0], coeff*tangent_at_start[1] ]
    except ZeroDivisionError:
        #print("directionvectors_for_curve: ZeroDivisionError at first tangent")
        pass
    ftangents = [[ t0,tangent_at_start]] + ftangents
    ## Find tangent at last point:
    t0,f0 = fvalues[-2]
    t1,f1 = fvalues[-1]
    delta = t1-t0
    tangent_at_end = tangent_direction(f,t1, -delta)
    # Turn the vector to point in the direction of f'(hi).
    f01 = [ f1[0]-f0[0], f1[1]-f0[1] ]
    if tangent_at_end[0]*f01[0] + tangent_at_end[1]*f01[1] < 0:
        tangent_at_end = [ -tangent_at_end[0], -tangent_at_end[1] ]
    len01 = sqrt( f01[0]**2 + f01[1]**2 )
    lentang = sqrt( tangent_at_end[0]**2 + tangent_at_end[1]**2 )
    try:
        coeff = len01 / ( (t1-t0)*lentang )
        tangent_at_end = [ coeff*tangent_at_end[0], coeff*tangent_at_end[1] ]
    except ZeroDivisionError:
        #print("directionvectors_for_curve: ZeroDivisionError at last tangent")
        pass
    ftangents.append( [ t1,tangent_at_end] )
    return ftangents

# --------------
# param_of_point
# --------------
# Find the first parameter t in [lo,hi] such that the point f(t) on
# the curve is closest to the input point.
#
# Args:
# - f:        a parametric curve (vector valued function);
# - lo, hi:   the lowest and highest parameter values of the curve;
# - point:    [x,y]
# - accuracy: required accuracy, i.e., the accuracy in the returned
#             parameter value (the default 1e-8);
# - steps:    the number of steps in the initial phase of the algorithm.
#
# Returns: the parameter value which gives the closest point.
def param_of_point(f,lo,hi, point, accuracy=1e-8, steps=1000):
    fvalues = curve_step_by_step(f,lo,hi, steps) # list of [t,f(t)]
    first_t, first_f = fvalues[0]
    closest_t = first_t
    point = [float(point[0]),float(point[1])]
    closest_dist2 = (first_f[0]-point[0])**2 + (first_f[1]-point[1])**2
    for tt, ftt in fvalues[1:]:
        d2 = (ftt[0]-point[0])**2 + (ftt[1]-point[1])**2
        if d2 < closest_dist2:
            closest_t = tt
            closest_dist2 = d2
    if hi-lo < accuracy*steps:
        return closest_t 
    # Find the interval [lower,upper] where closest_t is.
    if len(fvalues) <= 2: # Should not happen.
        return closest_t
    lower,fv = fvalues[0]
    upper,fv = fvalues[2]
    if lower == closest_t: # closest_t is in interval [lower,next]
        upper,fv = fvalues[1]
    else:
        for i in range(len(fvalues)-2):
            lower,fv = fvalues[i]
            upper,fv = fvalues[i+2]
            if lower < closest_t < upper:
                break
            if closest_t == upper:
                lower,fv = fvalues[i+1]
                try:
                    upper,fv = fvalues[i+3]
                except IndexError:
                    pass
                break
    # Recursion
    return param_of_point(f,lower,upper,point, steps=100)

# ----------
# mean_point
# ----------
# Return the parameter value and tangent direction vector at a mean
# point.
#
# Args:
# - f:      a parametric curve (vector valued function);
# - lo, hi: the lowest and highest parameter values of the arc;
#
# Returns:
# - the parameter of a mean point;
# - direction vector of tangent at the mean point.
#
# Note: Assumes that there are no cusp points. Currently the mean point
#       is taken from the average value (lo + hi)/2 of the parameter.
def mean_point(f,lo,hi):
    def unit_vector(v):
        l = sqrt( v[0]**2 + v[1]**2 )
        return [v[0]/l, v[1]/l]
    def smooth_left_right_unit_tangents(left,right):
        if left[0]*right[0] + left[1]*right[1] < 0:
            c = [ (-left[0]+right[0])/2, (-left[1]+right[1])/2 ]
        else:
            c = [ (left[0]+right[0])/2, (left[1]+right[1])/2 ]
        return unit_vector(c)
    arclen = abs(hi-lo)
    deltaright = arclen/100.
    deltaleft = -arclen/100.
    t = (lo + hi)/2. # Current choice of the mean point: middle parameter value.
    cleft = unit_vector( tangent_direction(f,t,deltaleft) )
    cright = unit_vector( tangent_direction(f,t,deltaright) )
    c = smooth_left_right_unit_tangents(cleft,cright)
    return t, c

# -----------
# arc_size_ge
# -----------
# Tell if a certain rough measure of the
# size of the bounding block of the arc (f,lot,hit) is greater than
# 'test_element'.
# Args:
# - f:            a parametric curve (vector valued function);
# - lot, hit:     the lowest and highest parameter values of the curve;
# - test_element: float
# Returns True if the arc is, roughly of greater size than 'test_element'.
# Note: This routine is applied to check if an arc is very short.
def arc_size_ge(f, lot, hit, test_element):
    at = [1., 1./2, 1./4, 3./4, 1./8., 3./8., 5./8., 7./8.]
    f0 = f(lot)
    xmax = xmin = f0[0]
    ymax = ymin = f0[1]
    for t in at:
        x1, y1 = f((1-t)*lot + t*hit)
        xmax = max(xmax,x1)
        xmin = min(xmin,x1)
        ymax = max(ymax,y1)
        ymin = min(ymin,y1)
        if xmax-xmin+ymax-ymin > test_element:
            return True
    return False


#==============================================================
#==============================================================
#            Parametric curve: creating subdivision
#==============================================================
#==============================================================

# --------------
# combine_custom
# --------------
# Prepare the 'custom' field of the input object of class ParametricCurve
# by combining its fields 'params', 'points', and 'screen_points' into
# only one list 'params'.
#
# Args:
# - pcurve:         ParametricCurve;
# - scoordsettings: class ScreenCoordinateSettings or None;
# Returns: new "prepared" list of custom parameters, obtained by
#          combining the custom field of the object pcurve, i.e., its
#          fields 'params', 'points', and 'screen_points' into one
#          list.
#
# Notes: 1. This routine is used in the method 'prepare_custom' of
#           class ParametricCurve
#        2. The input custom points, if any, are in pcurve.custom, of
#           class CustomPoints, which contains three fields (possibly
#           empty lists):
#           - pcurve.custom.params = a list of parameter values t;
#           - pcurve.custom.points = a list of points [x,y] in raw
#                                    coordinates;
#           - pcurve.custom.screen_points = a list of points [dx,dy] in
#                                           the coordinate system of
#                                           the screen, i.e., 
#                                           in screen coordinates.
#           If any pcurve.custom.points or pcurve.custom.screen_points
#           are submitted, the program tries to find and use the
#           closest points on the exact curve (i.e., the parameters of
#           those points).
#        3. In any one submitted list, too close points are replaced by
#           their average. If points in two submitted lists are too
#           close to each other, one of them is omitted, according to
#           the following preferences: params > points > screen_points.
#           Also, points too close to the end points of the arc are
#           discarded.
#           The upper limit to the allowed distances between custom or
#           end points is set in the variable 'limit' in the code.
def combine_custom(pcurve, scoordsettings=None):
    custom_gaps = pcurve.custom_gaps    # CHANGE FROM 1.08
    if custom_gaps is None:
        limit = (pcurve.hi - pcurve.lo)/100. # Default: good or bad?
    else:
        limit = custom_gaps
    def param_of_dpoint(f,lo,hi,screen_point, scoordsettings):
        origo = scoordsettings.origo
        scalex = scoordsettings.scalex
        scaley = scoordsettings.scaley
        dx,dy = screen_point
        dx,dy = float(dx-origo[0]), float(dy-origo[1])
        p = [ dx/scalex, dy/scaley ]
        return param_of_point(f,lo,hi,p)
    # prune_list: Taking a sorted list, prune_list replaces successive
    # elements, which are closer than limit to each other, by their
    # average, returning new list.
    def prune_list(a,limit):
        return average_close_floats(a, limit)
    # prune_first_list: Taking two sorted lists, prune_first_list
    # removes from the first list those elements which are closer than
    # limit to an element in the second list, returning new list.
    def prune_first_list(a,b,limit):
        new = []
        for a1 in a:
            found = False
            for b1 in b:
                if abs(a1-b1) <= limit:
                    found = True
                    break
            if not found:
                new.append(a1)
        return new
    cpts = pcurve.custom
    if cpts is None:
        cpts = CustomPoints()
    f = pcurve.function
    lot,hit = pcurve.lo, pcurve.hi
    prms = [ float(t) for t in cpts.params if lot <= t <= hit ]
    pts  = [ param_of_point(f,lot,hit,p) for p in cpts.points ]
    dpts = []
    if len(cpts.screen_points) > 0:
        dpts = [param_of_dpoint(f,lot,hit,[float(dp[0]),float(dp[1])], scoordsettings)
                                for dp in cpts.screen_points ]
    prms.sort()
    pts.sort()
    dpts.sort()
    # Too close points in each list:
    prms = prune_list(prms,limit)
    pts  = prune_list(pts, limit)
    dpts = prune_list(dpts,limit)
    # Too close points in two lists:
    pts  = prune_first_list(pts, prms,limit)
    dpts = prune_first_list(dpts,prms,limit)
    dpts = prune_first_list(dpts,pts, limit)
    # Merge and sort
    ms = prms+pts+dpts
    ms.sort()
    # Remove those too close to lo and hi.
    customparameters = [x for x in ms if lot+limit < x < hit-limit ]
    return customparameters


# ------------
# basic_points
# ------------
# The first step in creating the subdivision on the parametric curve:
# Build a list consisting of the end points and custom points.
#
# Args:
# - pcurve: ParametricCurve;
#
# Returns: [SubDivPoint], sorted according to the field 'parameter'.
#          The fields for tangents are left 'None'.
#
# Note: Thus, the returned list contains the subdivision points of the
# following types:
#    - start point  (type = 'low end'),
#    - custom point (type = 'custom'),
#    - end point    (type = 'high end').
def basic_points(pcurve):
    f = pcurve.function
    lot, hit = pcurve.lo, pcurve.hi
    customparams = pcurve.custom.params
    bp = [SubDivPoint('low end', lot, f(lot))]      # start point
    if len(customparams) != 0:
        sortedcp = sorted(customparams)
        for t in sortedcp:
            if not lot < t < hit:
                mess = "basic_points: not lot < t < hit"
                raise ParametricCurveError(mess)
            bp.append(SubDivPoint('custom', t, f(t)))
    bp.append(SubDivPoint('high end', hit, f(hit))) # end point
    return bp

# -----------
# cusp_points
# -----------
# Create a sorted list of elements of class SubDivPoint for
# cusp points in interval lot < t < hit.
# 
# Args:
# - f:        parametric arc (vector valued function);
# - lot, hit: the end points of the interval;
# - steps:    the number of subdivision intervals used in the initial
#             phase of the algorithm;
# - rel_accuracy: a float number.
#
# Returns: a list of elements of class SubDivPoint for the cusp points
#          (type = 'cusp') in interval lot < t < hit.
#          The list is sorted according to the field 'parameter'.
#          The fields for tangents are left 'None'.
#
# Note: In the program, this routine is not applied to the whole
#       user-input parametric curve. Instead, it is applied to some
#       smaller arc limited by previously created subdivision points.
def cusp_points(f,lot,hit,steps, rel_accuracy=1e-10):
    def potential_cusp(f0,f1,f2): # A cusp point ?
        f01 = [ f0[0]-f1[0], f0[1]-f1[1] ]
        f12 = [ f1[0]-f2[0], f1[1]-f2[1] ]
        lf01 = f01[0]**2 + f01[1]**2
        lf12 = f12[0]**2 + f12[1]**2
        dotprod = f01[0]*f12[0] + f01[1]*f12[1]
        if dotprod < 0:
            return True
        if dotprod**2 < 0.9 * lf01 * lf12:
            return True
        return False
    def is_flat(f0,f1,f2,f3, tolerance=1e-4):
        f03 = [f3[0]-f0[0], f3[1]-f0[1]]
        f01 = [f1[0]-f0[0], f1[1]-f0[1]]
        f23 = [f3[0]-f2[0], f3[1]-f2[1]]
        f01xf03 = -f01[0]*f03[1] + f01[1]*f03[0]
        f23xf03 = -f23[0]*f03[1] + f23[1]*f03[0]
        f01len2 = f01[0]**2 + f01[1]**2
        f23len2 = f23[0]**2 + f23[1]**2
        f03len2 = f03[0]**2 + f03[1]**2
        c = (tolerance**2) * f03len2
        return (f01xf03**2 < c*f01len2) and (f23xf03**2 < c*f23len2)
    def measure_general(x, f0, f03):
        fx = f(x)
        f0x = [fx[0]-f0[0], fx[1]-f0[1]]
        cross = -f0x[0]*f03[1] + f0x[1]*f03[0]
        return -cross**2
    stept = (hit-lot)/float(steps)
    potential = []
    t0,t1,t2 = lot, lot+stept, lot+2*stept
    f0,f1,f2 = f(t0),f(t1),f(t2)
    prev = None
    while t2 < hit:
        if potential_cusp(f0,f1,f2):
            if prev is not None: # combine two intervals
                prev = [prev[0],t2]
                potential[-1] = prev
            else:
                prev = [t0,t2]
                potential.append(prev)
        else:
            prev = None
        t0,t1,t2 = t1,t2,t2+stept
        f0,f1 = f1,f2
        if t2<hit:
            f2 = f(t2)
    accuracy = rel_accuracy*(hit-lot)
    cusp_intervals = []
    for pot_lo, pot_hi in potential:
        t0, t3 = pot_lo, pot_hi
        flat = False
        round = 0
        #max_rounds = 3 # This puts also limit to the achieved accuracy.
        max_rounds = 6 # This puts also limit to the achieved accuracy.
        while (t3-t0 > accuracy) and (round < max_rounds):
            round += 1
            f0, f3 = f(t0), f(t3)
            f03 = [f3[0]-f0[0], f3[1]-f0[1]]
            measure = partial(measure_general, f0=f0, f03=f03)
            try:
                something, interval = find_minimum(measure, t0, t3, 1e-10)
            except ParametricCurveError as e:
                mess = "cusp_points\n    "+str(e)
                error_message_function(mess)      # Give message
                #raise ParametricCurveError(mess) # but do not raise error.
                break
            t1, t2 = interval
            f1, f2 = f(t1), f(t2)
            if is_flat(f0,f1,f2,f3):
                flat = True
                #print("FLAT")
                break
            else:
                t0,t3 = t1,t2
        if not flat:
            cusp_intervals.append([t0,t3])
    vertices = []
    for t0,t3 in cusp_intervals:
        ave = (t0+t3)/2
        vertices.append(SubDivPoint( 'cusp', ave, f(ave)))
    return vertices

# -------------------------
# high_low_curvature_points
# -------------------------
# Create a sorted list elements of class SubDivPoint in interval
# lot < t < hit for all or only some of the following:
# - points of local maxima of curvature;
# - points of local minima of curvature;
# - start and end points of straight line segments,
# according to the input Boolean arguments.
# 
# Args:
# - f:             a parametric arc (vector valued function);
# - lot, hit:      the end points of the interval;
# - find_high:     Boolean;
# - find_low:      Boolean;
# - find_straight: Boolean;
# - steps:         the number of subdivision intervals used in the algorithm.
#
# Returns: a list of elements of class SubDivPoint for the
#            - points of local maxima of curvature
#             (type = 'high curvature') if find_high = True;
#            - points of local minima of curvature if find_high=True;
#             (type = 'low curvature') if find_low = True;
#            - start and end points of straight line segments if
#             (type = 'straight end') if find_straight = True;
#          in interval lot < t < hit.
#          The list is sorted according to the field 'parameter'
#          The fields for tangents are left 'None'.
#
# Note: In the program, this routine is not applied to the whole
#       user-input parametric curve. Instead, it is applied to some
#       smaller arc limited by previously created subdivision points.
def high_low_curvature_points(f, lot, hit, steps, short_arc_limit,
                              find_high, find_low, find_straight):
    class PointRadiusData:
        def __init__(self, t, ft, radius2):
            self.t = t
            self.ft = ft
            self.radius2 = radius2
    limit = 1e-10 # because of computation accuracy
    high_limit = 1.+limit
    low_limit = 1.-limit
    circle_limit = 1e-6 # to test circle cases
    if is_straight_line(f,lot,hit):
        return []
    fvalues = curve_step_by_step(f,lot,hit, steps) # list of [t,f(t)]
    pr = [PointRadiusData(fvalues[0][0], fvalues[0][1], None)]
    for i in range(1,len(fvalues)-1):
        a = fvalues[i-1][1]
        b = fvalues[i][1]
        c = fvalues[i+1][1]
        r2 = circle_radius2(a,b,c)
        pr.append(PointRadiusData(fvalues[i][0], fvalues[i][1], r2))
    pr.append(PointRadiusData(fvalues[-1][0], fvalues[-1][1], None))
    hcps = []
    if len(pr)<=2:
        circle_case = True
    else:
        pr_not_None = [prd for prd in pr if not(prd.radius2 is None)]
        max_radius2 = max(pr_not_None, key=lambda x:x.radius2).radius2
        min_radius2 = min(pr_not_None, key=lambda x:x.radius2).radius2
        circle_case = (max_radius2 - min_radius2 < circle_limit)
    if circle_case:
        return hcps
    i = 1
    prev_added = lot # lot is not added but this is used as a reference point
    while 1 <= i < len(pr)-3:
        i += 1                  # Now 2 <= i < len(pr)-2
        r2 = pr[i].radius2
        if pr[i].radius2 is None: # radius2 = infinite
            j = i
            while (pr[j+1].radius2 is None) and (j < len(pr)-3):
                j += 1
            if j > i: # Found consecutive cases with radius2 = infinite
                if find_straight:
                    t1, ft1 = pr[i].t, pr[i].ft
                    t2, ft2 = pr[j].t, pr[j].ft
                    if (arc_size_ge(f, prev_added, t2, short_arc_limit)
                      and arc_size_ge(f, t2, hit, short_arc_limit)):
                        hcps.append(SubDivPoint('straight end', t1, ft1))
                        hcps.append(SubDivPoint('straight end', t2, ft2))
                        prev_added = t2
                i = j+1
                continue
        if pr[i].radius2 is None:
            if find_low:
                t = pr[i].t
                ft = pr[i].ft
                if (arc_size_ge(f, prev_added, t, short_arc_limit)
                  and arc_size_ge(f, t, hit, short_arc_limit)):
                    hcps.append( SubDivPoint('low curvature', t, ft))
                    prev_added = t
            continue
        t0, t1, t2 = pr[i-1].t, pr[i].t, pr[i+1].t
        delta = min(abs(t1-t0), abs(t2-t1))
        r20, r21, r22 = pr[i-1].radius2, pr[i].radius2, pr[i+1].radius2
        if (r20 is None) or (r22 is None):
            continue
        if r20 < r21 * low_limit > r22:
            t0, t2 = t1 - delta, t1 + delta
            f0, f2 = f(t0), f(t2)
            r20 = circle_radius2(pr[i-2].ft, f0, pr[i].ft)
            r22 = circle_radius2(pr[i].ft, f2, pr[i+2].ft)
            if (r20 is None) or (r22 is None):
                continue
            t_ext, r2_ext = quadratic_extremum(t1, delta, r20, r21, r22)
            if find_low:
                if (arc_size_ge(f, prev_added, t_ext, short_arc_limit)
                  and arc_size_ge(f, t_ext, hit, short_arc_limit)):
                    hcps.append(SubDivPoint('low curvature', t_ext, f(t_ext)))
                    prev_added = t_ext
        elif (r20 > r21 * high_limit < r22):
            t0, t2 = t1 - delta, t1 + delta
            f0, f2 = f(t0), f(t2)
            r20 = circle_radius2(pr[i-2].ft, f0, pr[i].ft)
            r22 = circle_radius2(pr[i].ft, f2, pr[i+2].ft)
            if (r20 is None) or (r22 is None):
                continue
            t_ext, r2_ext = quadratic_extremum(t1, delta, r20, r21, r22)
            if find_high:
                if (arc_size_ge(f, prev_added, t_ext, short_arc_limit)
                  and arc_size_ge(f, t_ext, hit, short_arc_limit)):
                    hcps.append(SubDivPoint('high curvature', t_ext, f(t_ext)))
                    prev_added = t_ext
        elif (-limit < r21-r22 < limit):
            t_ext = (t1+t2)/2
            if find_high and (r20 > r21*high_limit):
                if (arc_size_ge(f, prev_added, t_ext, short_arc_limit)
                  and arc_size_ge(f, t_ext, hit, short_arc_limit)):
                    hcps.append(SubDivPoint('high curvature', t_ext, f(t_ext)))
                    prev_added = t_ext
            elif find_low and (r20 < r21*low_limit):
                if (arc_size_ge(f, prev_added, t_ext, short_arc_limit)
                  and arc_size_ge(f, t_ext, hit, short_arc_limit)):
                    hcps.append(SubDivPoint('low curvature', t_ext, f(t_ext)))
                    prev_added = t_ext
        else:
            pass
    return hcps

# -----------------
# inflection_points
# -----------------
# Create a sorted list of elements of class SubDivPoint for inflection
# points in interval lot < t < hit.
# 
# Args:
# - f:        a parametric arc (vector valued function);
# - lot, hit: the end points of the interval;
# - steps:    the number of subdivision intervals used in the algorithm.
#
# Returns: A list of elements of class SubDivPoint for the inflection
#          points (type = 'inflection') in interval lot < t < hit. The list
#          is sorted according to the field 'parameter'.
#          The fields for tangents are left 'None'.
#
# Note: In the program, this routine is not applied to the whole
#       user-input parametric curve. Instead, it is applied to some
#       smaller arc limited by previously created subdivision points.
def inflection_points(f,lot,hit,steps, short_arc_limit):
    class PointCrossData:
        def __init__(self, t0, t1, t2, cross0, cross1, cross2):
            self.t012 = [t0, t1, t2]
            self.cross012 = [cross0, cross1, cross2]
    class IntervalCrossData:
        def __init__(self, t0 ,t1, t2, t3, cross0, cross1, cross2, cross3):
            self.t0123 = [t0 ,t1, t2, t3]
            self.cross0123 = [cross0, cross1, cross2, cross3]
    if is_straight_line(f,lot,hit):
        return []
    fvalues = curve_step_by_step(f,lot,hit, steps) # list of [t,f(t)]
    fvalues_turns = []
    f0, f1 = fvalues[0][1], fvalues[1][1]
    f01 = [f1[0]-f0[0], f1[1]-f0[1]]
    cross_array = [0.]
    for i in range(1, len(fvalues)-1):
        f2 = fvalues[i+1][1]
        f12 = [f2[0]-f1[0], f2[1]-f1[1]]
        cross = -f12[0]*f01[1] + f12[1]*f01[0]
        cross_array.append(cross)
        f1 = f2
        f01 = f12
    cross_array.append(0.)
    # Discard nodes with cross = 0 both from fvalues and cross_array:
    reduced_fvalues = []
    reduced_cross_array = []
    for i in range(len(fvalues)):
        if cross_array[i] != 0.:
            reduced_fvalues.append(fvalues[i])
            reduced_cross_array.append(cross_array[i])
    point_cross_data_array = []
    for i in range(len(reduced_fvalues)):
        if i == 0:
            t0 = reduced_fvalues[0][0]
            t1 = reduced_fvalues[0][0]
            t2 = reduced_fvalues[1][0]
            cross0 = reduced_cross_array[0]
            cross1 = reduced_cross_array[0]
            cross2 = reduced_cross_array[1]
        elif i == len(reduced_fvalues)-1:
            t0 = reduced_fvalues[-2][0]
            t1 = reduced_fvalues[-1][0]
            t2 = reduced_fvalues[-1][0]
            cross0 = reduced_cross_array[-2]
            cross1 = reduced_cross_array[-1]
            cross2 = reduced_cross_array[-1]
        else:
            t0 = reduced_fvalues[i-1][0]
            t1 = reduced_fvalues[i][0]
            t2 = reduced_fvalues[i+1][0]
            cross0 = reduced_cross_array[i-1]
            cross1 = reduced_cross_array[i]
            cross2 = reduced_cross_array[i+1]
        pc = PointCrossData(t0,t1,t2, cross0, cross1, cross2)
        point_cross_data_array.append(pc)
    interval_cross_data_array = []
    try:
        prev = point_cross_data_array[0]
    except IndexError: # straight line
        return []
    for curr in point_cross_data_array[1:]:
        if (curr.cross012[1] > 0.) == (prev.cross012[1] > 0.):
            prev = curr
            continue
        t0, t1, t2 = prev.t012
        cross0, cross1, cross2 = prev.cross012
        t3 = curr.t012[-1]
        cross3 = curr.cross012[-1]
        ic = IntervalCrossData(t0,t1,t2,t3, cross0, cross1, cross2, cross3)
        interval_cross_data_array.append(ic)
        prev = curr
    infl_points = []
    prev_added = lot # lot is not added but this is used as a reference point
    for ic in interval_cross_data_array:
        t0, t1, t2, t3 = ic.t0123
        cross0, cross1, cross2, cross3 = ic.cross0123
        # Interpolate to get the inflection point:
        try:
            t12 = (t2*cross1 - t1*cross2) / (cross1 - cross2)
            t03 = (t3*cross0 - t0*cross3) / (cross0 - cross3)
            t = 0.5*(3*t12 - t03) # Interpolated
        except ZeroDivisionError:
            try:
                t = (t2*cross1 - t1*cross2) / (cross1 - cross2)
            except ZeroDivisionError:
                t = (t1+t2)/2
        if (arc_size_ge(f, prev_added, t, short_arc_limit)
          and arc_size_ge(f, t, hit, short_arc_limit)):
            infl_points.append( SubDivPoint('inflection', t, f(t)))
            prev_added = t
    return infl_points

# --------------------------
# exists_an_inflection_point               CHANGE FROM 1.08
# --------------------------
# Check existence of at least one inflection point
# in interval lot < t < hit.
# 
# Args:
# - f:        a parametric arc (vector valued function);
# - lot, hit: the end points of the interval;
# - fvalues:  a list [t,f(t)] of points densely along the curve.
# Returns: Boolean.
#
# Notes:
# 1. Returns True if arc deemed to be straight line.
# 2. Code extracted from routine inflection_points.
def exists_an_inflection_point(f,lot,hit, fvalues):
    class PointCrossData:
        def __init__(self, t0, t1, t2, cross0, cross1, cross2):
            self.t012 = [t0, t1, t2]
            self.cross012 = [cross0, cross1, cross2]
    if is_straight_line(f,lot,hit):
        return True
    fvalues_turns = []
    f0, f1 = fvalues[0][1], fvalues[1][1]
    f01 = [f1[0]-f0[0], f1[1]-f0[1]]
    cross_array = [0.]
    for i in range(1, len(fvalues)-1):
        f2 = fvalues[i+1][1]
        f12 = [f2[0]-f1[0], f2[1]-f1[1]]
        cross = -f12[0]*f01[1] + f12[1]*f01[0]
        cross_array.append(cross)
        f1 = f2
        f01 = f12
    cross_array.append(0.)
    # Discard nodes with cross = 0 both from fvalues and cross_array:
    reduced_fvalues = []
    reduced_cross_array = []
    for i in range(len(fvalues)):
        if cross_array[i] != 0.:
            reduced_fvalues.append(fvalues[i])
            reduced_cross_array.append(cross_array[i])
    point_cross_data_array = []
    for i in range(len(reduced_fvalues)):
        if i == 0:
            t0 = reduced_fvalues[0][0]
            t1 = reduced_fvalues[0][0]
            t2 = reduced_fvalues[1][0]
            cross0 = reduced_cross_array[0]
            cross1 = reduced_cross_array[0]
            cross2 = reduced_cross_array[1]
        elif i == len(reduced_fvalues)-1:
            t0 = reduced_fvalues[-2][0]
            t1 = reduced_fvalues[-1][0]
            t2 = reduced_fvalues[-1][0]
            cross0 = reduced_cross_array[-2]
            cross1 = reduced_cross_array[-1]
            cross2 = reduced_cross_array[-1]
        else:
            t0 = reduced_fvalues[i-1][0]
            t1 = reduced_fvalues[i][0]
            t2 = reduced_fvalues[i+1][0]
            cross0 = reduced_cross_array[i-1]
            cross1 = reduced_cross_array[i]
            cross2 = reduced_cross_array[i+1]
        pc = PointCrossData(t0,t1,t2, cross0, cross1, cross2)
        point_cross_data_array.append(pc)
    interval_cross_data_array = []
    try:
        prev = point_cross_data_array[0]
    except IndexError: # straight line
        return True
    for curr in point_cross_data_array[1:]:
        if (curr.cross012[1] > 0.) != (prev.cross012[1] > 0.):
            #print("Found inflection point")
            return True
    return False

# --------------
# turns_by_angle
# --------------
# Create a sorted list of elements of class SubDivPoint for points such
# that they divide the arc into smaller arcs, each of which turns
# (roughly) the same amount <= input angle.
#
# Args:
# - f:      a parametric arc (vector valued function);
# - lo, hi: the end points of the interval;
# - steps:  the number of subdivision intervals used in the algorithm;
# - angle:  the target angle; must be in (0, 360 degrees) (default=pi/2).
#
# Returns: A list of elements of class SubDivPoint for the angle turn
#          points (type = 'angle turn') in interval lot < t < hit.
#          The list is sorted according to the field 'parameter'.
#          The fields for tangents are left 'None'.
#
# Note: Existence of inflection points is checked, and if any are present,
#       the work is abandoned and [] is returned.
#       Namely, the algorithm fails unless the arc turns all the way in the
#       same direction, i.e., constantly to the left or constantly to
#       the right.
#
# Note: In the program, this routine is not applied to the whole
#       user-input parametric curve. Instead, it is applied to some
#       smaller arc limited by previously created subdivision points.
def turns_by_angle(f,lo,hi, steps, angle=pi/2): 
    # Must have angle in (0,360 degrees). 
    if not (0 < angle < 2*pi):
        mess = "turns_by_angle: angle not valid."
        raise ParametricCurveError(mess)
    # first_turn_point_by_angle:
    # Find the first point where the curve has turned by angle.
    # - Always angle >= 0, in radians.
    # - turnright tells the direction in which the arc turns when moving
    #   from start to end, all the way.
    # - If forwards=False, the search is done backwards; in that case
    #   the caller must change turnright to the opposite value!
    # - ftangentsin is the output of directionvectors_for_curve,
    #   i.e., a list [..., [t,tangent(t)], ...], where tangent(t) is, roughly,
    #   a direction vector of f at t.
    def first_turn_point_by_angle(f,ftangentsin, angle, turnright, forwards=True):
        c = cos(angle)
        s = sin(angle)
        def rotate(v, turnright):
            if turnright:
                return [ v[0]*c + v[1]*s, -v[0]*s + v[1]*c ] # ccw
            else:
                return [ v[0]*c - v[1]*s,  v[0]*s + v[1]*c ] # cw
        def cross(v,w):
            return -v[0]*w[1] + v[1]*w[0]
        def check(comp,tang):
            if comp[0]*tang[0] + comp[1]*tang[1] < 0:
                return False
            if turnright:
                return cross(comp, tang) >= 0.
            else:
                return cross(comp, tang) <= 0.
        if forwards:
            ftangents = ftangentsin
        else:
            ftangents = ftangentsin[::-1] # reversed
        start, starttang = ftangents[0] # start of the next arc to be examined
        compare = rotate(starttang, turnright)
        prev,prevtang = start,starttang
        for cur, curtang in ftangents[1:]:
            if not check(compare,curtang):
                prev,prevtang = cur,curtang
                continue # not yet found a turn by angle
            # found!
            # Interpolation
            x1,tang1 = prev,prevtang
            x2,tang2 = cur, curtang
            cross1 = cross(compare, tang1)
            cross2 = cross(compare, tang2)
            midx = (cross2*x1-cross1*x2) / (cross2-cross1)
            midtang = extrapolate(midx, x1,x2, tang1, tang2 )
            return midx,midtang
        return None
    # find_turn_points:
    # Find the sequence of parameters to divide the arc lo<t<hi into
    # smaller arcs each turning the amount of angle (roughly).
    # Return the list and the remaining angle.
    def find_turn_points(f,ftangents, angle, turnright):
        result = []
        start, starttang = ftangents[0] # start of the next arc to be examined
        next_ftangents = ftangents
        while True:
            findnext = first_turn_point_by_angle(
                            f,
                            next_ftangents,
                            angle,
                            turnright,
                            forwards = True)
            if findnext is None:
                break
            next,nexttang = findnext
            next_ftangents = [ [next,nexttang] ]
            for mt,mftang in ftangents:
                if mt <= next:
                    continue
                next_ftangents.append([ mt,mftang ])
            result.append(next)
            start, starttang = next,nexttang
        remainder = angle_between_vectors( starttang, ftangents[-1][1] )
        return result, remainder
    tolerance = 0.015
    limit = 0.0000001
    fvalues = curve_step_by_step(f,lo,hi, steps=1000) # list of [t,f(t)]
    # The algorithm fails if there are inflection points.
    # Normally there should be none but it may happen:
    # Even if inflection points are searched for by inflection_points or
    # high_low_curvature_points, some may have been rejected because of
    # short_arc_limit.
    # Therefore check now.
    if exists_an_inflection_point(f,lo,hi, fvalues):
        return []
    ftangents = directionvectors_for_curve(f,lo,hi, fvalues)
    left_or_right = []
    t0,d0 = ftangents[0]
    for t1,d1 in ftangents[1:]:
        d0xd1 = -d0[0]*d1[1] + d0[1]*d1[0]
        if d0xd1 >= limit:
            left_or_right.append(1)
        elif d0xd1 <= -limit:
            left_or_right.append(-1)
        else:
            left_or_right.append(0)
        t0,d0 = t1,d1
    vote = 0
    for to_right in left_or_right:
        vote += to_right
    turn_right = ( vote > 0 )
    not_all_same = ( abs(vote) < len(left_or_right) )
    if not_all_same:
        if is_straight_line(f,lo,hi):
            return [] # Aiheuttanee virheen joskus.
    turn_points, remainder = find_turn_points(f,ftangents, angle, turn_right)
    n = len(turn_points)
    total_angle = n*angle + remainder
    if remainder/angle < tolerance:
        in_tolerance = True
        turn_points = turn_points[:-1] # Drop the last one
    elif abs(angle-remainder)/angle < tolerance:
        in_tolerance = True # Now don't drop the last one!
    else: # Make a new search using the more exact new_angle
        in_tolerance = False
        new_angle = total_angle / (n+1)
        turn_points, new_remainder = find_turn_points(f,
                                        ftangents, new_angle, turn_right)
        new_n = len(turn_points)
        new_total_angle = new_n*new_angle + new_remainder
        if new_remainder/new_angle < tolerance:
            in_tolerance = True
            turn_points = turn_points[:-1]  # Drop the last one
        elif abs(new_angle-new_remainder)/new_angle < tolerance:
            in_tolerance = True # Now don't drop the last one!
    if not in_tolerance:
        mess = "turns_by_angle: Not in tolerance"
        raise ParametricCurveError(mess)
    typed_turn_points = []
    for t in turn_points:
        typed_turn_points.append(SubDivPoint('angle turn', t, f(t)))
    return typed_turn_points

# ------------------
# subdivision_points
# ------------------
# CHANGE FROM 1.08
# See the code (NOT TESTED!)
#
# Create a sorted list of the various types of subdivision points on
# the whole input curve.
#
# Args:
# - pcurve:           ParametricCurve;
# - c2beziersettings: Curve2BezierSettings;
# - c2bezieroptions:  Curve2BezierOptions;
# - short_arc_limit:  a float number: suppress most subdivision points
#                     on shorter arcs;
# Returns:
# - 'rivps':       a list of elements of class SubDivPoint, sorted according to
#                  the field 'parameter'. The fields for tangents are left
#                  'None';
def subdivision_points(pcurve,
                        c2beziersettings,
                        c2bezieroptions,
                        short_arc_limit,
                        ):
    def row(type, t, ft): # For printing only
        str_ft = '[' + ', '.join('{: .3f}'.format(x) for x in ft) + ']'
        r ='{:<14}  {:>6.3f}  {}'.format(type,t, str_ft)
        r += '    t = '+repr(t)
        return r
    ## Initial phase: Collect basic points (start, end, and custom points)
    ## into a list.
    f = pcurve.function
    angle = c2beziersettings.angle
    steps = c2beziersettings.steps
    # cbps will be basic points (= end points + custom points)
    bps = basic_points(pcurve)
    ## Next phase: Find and add cusp points.
    if c2bezieroptions.cusp_points:
        # cbps will be basic points + cusp points
        cbps = [bps[0]]
        for i in range(len(bps)-1):
            arc_start, arc_end = bps[i].parameter, bps[i+1].parameter
            cps = cusp_points(f, arc_start, arc_end, steps)
            cbps = cbps + cps + [bps[i+1]]
    else:
        cbps = bps
    ## Next phase: Find and add inflection points.
    if c2bezieroptions.inflection_points:
        # icbps will be basic + cusp + inflection points
        icbps = [cbps[0]]
        for i in range(len(cbps)-1):
            arc_start, arc_end = cbps[i].parameter, cbps[i+1].parameter
            ips = inflection_points(f,
                        arc_start, arc_end,
                        steps, short_arc_limit
                        )
            # CHANGE FROM 1.08
            # Remove those too close to arc_start or arc_end
            ok_lo = arc_start + c2beziersettings.short_arc
            ok_hi = arc_end - c2beziersettings.short_arc
            ips = [ip for ip in ips if ok_lo < ip.parameter < ok_hi]
            ####################
            icbps = icbps + ips + [cbps[i+1]]
    else:
        icbps = cbps
    ## Next phase: Find and add high and/or low curvature points,
    ## together with end points of straight line segments.
    add_high = c2bezieroptions.high_curvature_points
    add_low = c2bezieroptions.low_curvature_points
    add_straight = c2bezieroptions.straight_end_points
    if add_high or add_low or add_straight:
        # hicbps will be
        # basic + cusp + inflection + high/low curvature + straight points
        hicbps = [icbps[0]]
        for i in range(len(icbps)-1):
            arc_start, arc_end = icbps[i].parameter, icbps[i+1].parameter
            hcps = high_low_curvature_points(f,
                        arc_start, arc_end,
                        steps, short_arc_limit,
                        add_high, add_low, add_straight
                        )
            # CHANGE FROM 1.08
            # Remove those too close to arc_start or arc_end
            ok_lo = arc_start + c2beziersettings.short_arc
            ok_hi = arc_end - c2beziersettings.short_arc
            hcps = [hp for hp in hcps if ok_lo < hp.parameter < ok_hi]
            ####################
            hicbps = hicbps + hcps + [icbps[i+1]]
    else:
        hicbps = icbps
    ## Next phase: Find and add turn points by angle.
    # (That is, find points splitting each arc into segments such that
    # in each segment the curve turns at most by the amount of angle.)
    if c2bezieroptions.angle_turn_points:
        # rivps will be
        # basic + cusp + inflection + high/low curvature + straight + angle turn points
        rivps = [hicbps[0]]
        for i in range(len(hicbps)-1):
            arc_start, arc_end = hicbps[i].parameter, hicbps[i+1].parameter
            # For angle turn points, check shortness of arc only about the whole arc
            # to be subdivided, and not about new subdivision arcs to be created.
            # (Otherwise the algorithm in 'turns_by_angle' might fail.)
            if arc_size_ge(f, arc_start, arc_end, short_arc_limit):
                rtps = turns_by_angle(f, arc_start, arc_end, steps, angle)
            else:
                rtps = []
            # CHANGE FROM 1.08
            # Remove those too close to arc_start or arc_end
            ok_lo = arc_start + c2beziersettings.short_arc
            ok_hi = arc_end - c2beziersettings.short_arc
            rtps = [rtp for rtp in rtps if ok_lo < rtp.parameter < ok_hi]
            ####################
            rivps = rivps + rtps + [hicbps[i+1]]
    else:
        rivps = hicbps
    return rivps

# -----------------
# subdivpoints_data
# -----------------
# Completes a list of elements of class SubDivPoint by filling
# the fields unittangentleft and unittangentright.
#
# Args:
# - f:            a vector valued parametric function;
# - subdivpoints: a list of elements of class SubDivPoint;
# - accuracy:     a float number determining the interval length used
#                 when computing tangents (default=0.001).
#
# Returns: The same input list with the fields unittangentleft and
#          unittangentright added.
def subdivpoints_data(f, subdivpoints, accuracy=.001):
    def unit_vector(v):
        l = sqrt( v[0]**2 + v[1]**2 )
        return [v[0]/l, v[1]/l]
    def smooth_left_right_unit_tangents(left,right):
        if left[0]*right[0] + left[1]*right[1] < 0:
            c = [ (-left[0]+right[0])/2, (-left[1]+right[1])/2 ]
        else:
            c = [ (left[0]+right[0])/2, (left[1]+right[1])/2 ]
        return unit_vector(c)
    i = 0
    while i < len(subdivpoints):
        type = subdivpoints[i].type
        t    = subdivpoints[i].parameter
        fvalue = subdivpoints[i].functionvalue
        # Compute suitable deltaleft and deltaright for tangent calculations.
        if type == 'low end':    # start point
            arclen = subdivpoints[1].parameter - subdivpoints[0].parameter
            deltaright = arclen*accuracy
        elif type == 'high end': # end point
            arclen = subdivpoints[-2].parameter - subdivpoints[-1].parameter
            deltaleft = arclen*accuracy
        else:                    # all others
            lenright  = subdivpoints[i+1].parameter - subdivpoints[i].parameter
            lenleft = subdivpoints[i-1].parameter - subdivpoints[i].parameter
            deltaright = lenright*accuracy
            deltaleft  = lenleft*accuracy
        # Calculate tangents and add to subdivpoints.
        if type == 'low end':    # start point
            c = unit_vector( tangent_direction(f,t,deltaright) )
            subdivpoints[i].unittangentleft  = None
            subdivpoints[i].unittangentright = c
        elif type == 'high end': # end point
            c = unit_vector( tangent_direction(f,t,deltaleft) )
            subdivpoints[i].unittangentleft  = c
            subdivpoints[i].unittangentright = None
        elif type == 'custom':   # custom point: possibly smooth or non-smooth
            cleft = unit_vector( tangent_direction(f,t,deltaleft) )
            cright = unit_vector( tangent_direction(f,t,deltaright) )
            if abs(-cleft[0]*cright[1] + cleft[1]*cright[0]) > 0.00001: # jokin accuracy?
                # Treat as a non-smooth point.
                subdivpoints[i].unittangentleft  = cleft
                subdivpoints[i].unittangentright = cright
            else: # Treat as a smooth point.
                c = smooth_left_right_unit_tangents(cleft,cright)
                subdivpoints[i].unittangentleft  = c
                subdivpoints[i].unittangentright = c
        elif type == 'cusp':     # cusp (not start nor end): non-smooth point
            cleft = unit_vector( tangent_direction(f,t,deltaleft) )
            cright = unit_vector( tangent_direction(f,t,deltaright) )
            subdivpoints[i].unittangentleft  = cleft
            subdivpoints[i].unittangentright = cright
        else:                    # All others are smooth points!
            # 'high curvature',
            # 'low curvature',
            # 'straight end',
            # 'inflection',
            # 'angle turn'
            cleft = unit_vector( tangent_direction(f,t,deltaleft) )
            cright = unit_vector( tangent_direction(f,t,deltaright) )
            c = smooth_left_right_unit_tangents(cleft,cright)
            subdivpoints[i].unittangentleft  = c
            subdivpoints[i].unittangentright = c
        i += 1
    return subdivpoints


#==============================================================
#==============================================================
#          Computing Bezier arcs from a parametric curve
#==============================================================
#==============================================================

# ------------------------
# bezier_arcs_from_one_arc
# ------------------------
# Take as input a smooth arc, relatively short and of such shape as can
# be expected to be well approximated with one cubic Bezier arc (for
# example, no cusp points, no loops, or anything wild).
# Find an approximation as a list (hopefully short) of butting Bezier
# arcs, ideally consisting of only one item.
#
# Args:
# - adata:            ArcData;
# - c2bezier:         Curve2BezierSettings;
# - short_arc_limit:  float number to suppress recursion on too short
#                     intervals;
# - err_comp_limit:   float number to suppress computation of inaccuracy
#                     of approximation for too short intervals;
# Returns:
# - [BCurve.BezierArc];
# - float (an error measure).
#
# Note: The routine first finds one cubic Bezier arc approximating the
# input arc, by calling the routine 'approximate_with_bezier_arc'.
# When such a Bezier arc is found, then if the inaccuracy of the
# approximation appears to be too large, the input arc is split
# in two, and the search of approximation is repeated recursively
# for the two halves, and so on.
# The largest allowed relative error and the largest allowed recursion
# depth are preset as
# - c2bezier.max_error
# - c2bezier.max_recursion.
# If the latter limit is reached while the relative error is still too
# large, the result is returned as such, despite the inaccuracy.
def bezier_arcs_from_one_arc(adata,
            c2bezier,
            short_arc_limit,
            err_comp_limit,
            messages=False,
            ):
    NSAMPLES = 2         # Effects: accuracy and running time
    ERROR_COMP_LEVEL = 2 # Effects: error computation and recursion depth
    #
    error_maximum = c2bezier.max_error
    recursion_depth = 0
    f = adata.function
    start,end = adata.lo,adata.hi
    fdotstart, fdotend = adata.unittangent_lo, adata.unittangent_hi
    p0, p3 = f(start), f(end)
    p0p3 = [ p3[0]-p0[0], p3[1]-p0[1] ]
    ## Check first if refinement > 0;
    ## in that case split the arc in two, and call
    ## bezier_arcs_from_one_arc recursively for each half
    ## with refinement=refinement-1.
    if c2bezier.refinement > 0:                         ## REFINEMENT
        # Make a new copy of c2bezier with refinement 1 less:
        new_c2bezier = Curve2BezierSettings(
                          angle         = c2bezier.angle,
                          refinement    = c2bezier.refinement - 1,
                          max_error     = c2bezier.max_error,
                          max_recursion = c2bezier.max_recursion,
                          )
        mean_parameter, mean_tangent = mean_point(f,start,end)
        adata_left = ArcData(
                             f,
                             start = start,
                             end = mean_parameter,
                             tangent_start = fdotstart,
                             tangent_end = mean_tangent
                             )
        # Recursion left
        bez_arcs_left, rel_error_left = bezier_arcs_from_one_arc(
                                       adata_left, 
                                       new_c2bezier,
                                       short_arc_limit,
                                       err_comp_limit,
                                       messages,
                                       )
        # Recursion right
        adata_right = ArcData(
                             f,
                             start = mean_parameter,
                             end = end,
                             tangent_start = mean_tangent,
                             tangent_end = fdotend
                             )
        bez_arcs_right, rel_error_right = bezier_arcs_from_one_arc(
                                       adata_right,
                                       new_c2bezier,
                                       short_arc_limit,
                                       err_comp_limit,
                                       messages,
                                       )
        bez_arcs = bez_arcs_left + bez_arcs_right
        rel_error = max(rel_error_left, rel_error_right) # ???
        return bez_arcs, rel_error
    chord_size = max(abs(p3[0]-p0[0]), abs(p3[1]-p0[1]))
    very_short_arc = (chord_size < short_arc_limit)
    p0p3len2 = max(abs(p0p3[0]), abs(p0p3[1]))
    # Main call
    cp4_list, rel_error, _ = paramcurve2cp4list(
                        f=adata.function,
                        fdot=None,
                        interval=[adata.lo,adata.hi],
                        accuracy=c2bezier.max_error,
                        max_rec_depth=c2bezier.max_recursion,
                        nsamples=NSAMPLES,
                        error_comp_level=ERROR_COMP_LEVEL,
                        rec_depth=0,
                        very_short_arc=False,  # CHANGE FROM 1.08
                        messages=messages,
                        message_handler=error_message_function
                        )
    bez_arcs = [BCurve.BezierArc(cp4) for cp4 in cp4_list]
    return bez_arcs, rel_error

# ------------------------------
# bezier_curve_from_whole_pcurve
# ------------------------------
# Compute for a parametric curve an approximate cubic Bezier curve, all in
# raw coordinates
#
# Args:
# - pcurve:           ParametricCurve;
# - c2beziersettings: Curve2BezierSettings;
# - c2bezieroptions:  Curve2BezierOptions;
# Returns:
# - BCurve.BezierCurve;
# - float (an error measure).
#
# Note: In contrast with 'bezier_arcs_from_one_arc', this routine is
# expected to be used for a whole parametric curve (i.e., not just for
# a short arc).
# It first calls suitable routines to subdivide the curve into shorter
# arcs, each of which is then, generally, handled by calling
# 'bezier_arcs_from_one_arc'.
def bezier_curve_from_whole_pcurve(
            pcurve,
            c2beziersettings,
            c2bezieroptions,
            messages=False,
            ):
    # xreversed: From input list of Bezier arcs, return a similar list
    # in reversed order with each Bezier arc reversed as well.
    def xreversed(b_list):
        r = list(reversed(b_list))
        for barc in r:
            barc.reverse()
        return r
    f = pcurve.function
    fname = pcurve.pcurve_name
    lot, hit = pcurve.lo, pcurve.hi
    bb = pcurve.bounding_box
    xmin = bb.xmin
    xmax = bb.xmax
    ymin = bb.ymin
    ymax = bb.ymax
    bb_size = max(xmax-xmin, ymax-ymin)
    # Do not compute errors for shorter intervals than the following limit.
    error_computing_limit = 0.015 * bb_size   # arbitrary choice
    # For arcs shorter than the following limit:
    # - do not compute all subdivision points, and
    # - use the simple approximating algorithm.
    short_arc_limit = c2beziersettings.short_arc * bb_size
    ## Create subdivision of the curve f. 
    ## This is the subdivision used in building the list of Bezier arcs,
    ## but in effect, in the algorithm further refinement may occur
    ## because of over-all refinement or recursion.
    subdivpoints = subdivision_points(
        pcurve,
        c2beziersettings,
        c2bezieroptions,
        short_arc_limit)
    subdivinfo = subdivpoints_data(
        f,
        subdivpoints)
    ## Build the bezier curve.
    bezier_arc_list = []
    all_relative_errors = [] # Relative errors
    i = 0
    for i in range(len(subdivinfo)-1): # Run through the arcs in the subdivision.
    #for i in range(0,1): # KOKEILUA VARTEN
    #for i in range(ALKU,LOPPU): # KOKEILUA VARTEN
        info_at_left_end = subdivinfo[i]
        type0 = info_at_left_end.type
        t0 = info_at_left_end.parameter
        f0 = info_at_left_end.functionvalue
        fdot0 = info_at_left_end.unittangentright
        info_at_right_end = subdivinfo[i+1]
        type3 = info_at_right_end.type
        t3 = info_at_right_end.parameter
        f3 = info_at_right_end.functionvalue
        fdot3 = info_at_right_end.unittangentleft
        adata = ArcData(
                       f,
                       start = t0,
                       end = t3,
                       tangent_start = fdot0,
                       tangent_end = fdot3
                       )
        if is_straight_line(f,t0,t3):
            p0 = f0
            p3 = f3
            p1 = [ ( 2*p0[0] + p3[0] )/3. , ( 2*p0[1] + p3[1] )/3. ]
            p2 = [ ( p0[0] + 2*p3[0] )/3. , ( p0[1] + 2*p3[1] )/3. ]
            cp4 = [BCurve.ControlPoint(cp[0],cp[1]) for cp in [p0,p1,p2,p3]]
            all_relative_errors.append(0.)
            bezier_arc_list.append(BCurve.BezierArc(cp4))
        else: # Normal case
            try:
                # Main call
                bezier_arcs, rel_error = bezier_arcs_from_one_arc(
                                adata,
                                c2beziersettings,
                                short_arc_limit,
                                error_computing_limit,
                                messages,
                                )
            except ParametricCurveError as e:
                mess ="bezier_curve_from_whole_pcurve\n    "+str(e)
                raise ParametricCurveError(mess)
            all_relative_errors.append(rel_error)
            bezier_arc_list += bezier_arcs
    max_relative_error = max(all_relative_errors)
    if pcurve.reversed:
        bezier_arc_list = xreversed(bezier_arc_list)
    return BCurve.BezierCurve(bezier_arcs=bezier_arc_list,
                       closed=pcurve.closed,
                       curve_name="Bezier:"+ fname
                       ), max_relative_error

# ------------------------
# bezier_curve_from_pcurve
# ------------------------
# First prepare custom points and then call the function building
# an approximate Bezier arcs from the input parametric curve.
#
# Args:
# - pcurve:           ParametricCurve;
# - c2beziersettings: Curve2BezierSettings;
# - c2bezieroptions:  Curve2BezierOptions;
# - scoordsettings:   ScreenCoordinateSettings;
#
# Returns:
# - ParametricCurve: same object as input pcurve but with new field 'custom';
# - BCurve.BezierCurve;
# - float (an error measure).
def bezier_curve_from_pcurve(
            pcurve,
            c2beziersettings,
            c2bezieroptions,
            scoordsettings,
            messages=False,
            ):
    # Handle custom points:
    # Input pcurve contains field custom which contains fields
    # params, points, screen_points.
    # Make from them a single list of parameters:
    pcurve.prepare_custom(scoordsettings) # Changes custom field only: no copy!
    ## Main call to build the Bezier curve:
    try:
        bezier_curve, rel_error = bezier_curve_from_whole_pcurve(
            pcurve,
            c2beziersettings,
            c2bezieroptions,
            messages,
            )
    except ParametricCurveError as e:
        mess = "bezier_curve_from_pcurve\n    "+str(e)
        raise ParametricCurveError(mess)
    return pcurve, bezier_curve, rel_error


#==============================================================
#==============================================================
#                Auxiliary routines for drawing
#==============================================================
#==============================================================

# ----------
# raw2screen
# ----------
# Conversion from a point=[px,py] in raw coordinates into screen
# coordinates.
#
# Args:
# - point: a point in raw coordinates;
# - scoordsettings: ScreenCoordinateSettings.
#
# Returns: a point [x,y] in screen coordinates.
def raw2screen(point, scoordsettings):
    origo = scoordsettings.origo
    scalex = scoordsettings.scalex
    scaley = scoordsettings.scaley
    x = origo[0] + scalex*point[0]
    y = origo[1] + scaley*point[1]
    return [x,y]

# ----------------------
# sparse_points_on_curve
# ----------------------
# Compute a sparse set of points on a parametric curve.
#
# Args:
# - pcurve:         ParametricCurve;
# - scoordsettings: ScreenCoordinateSettings;
# - npoints:        the initial number of the points to be calculated on the
#                   curve (default=100);
# - simple:         Boolean (default=False): if False, then
#                   - where the curve has a strong bend, an extra point is
#                     added, and if the strong bend happens to be a cusp
#                     point, an effort is made to find the point accurately;
#                   - points on the boundary of the bounding box area added.
#
# Returns: a list of points in screen coordinates.
def sparse_points_on_curve(pcurve, scoordsettings, npoints=100, simple=False):
    def find_vertex_param(f, t0, t3, repeat=6):
        n = 10
        ti = [ ((n-i)*t0 + i*t3)/n  for i in range(n)] + [t3]
        fti = [f(t) for t in ti]
        f0, f1, f2 = fti[0], fti[1], fti[2]
        f01 = [f1[0]-f0[0], f1[1]-f0[1]]
        f12 = [f2[0]-f1[0], f2[1]-f1[1]]
        i_min = 1
        c_min = cosine_between_vectors(f01,f12)
        for i in range(2,len(ti)-1):
            f0, f1, f2 = fti[i-1], fti[i], fti[i+1]
            f01 = [f1[0]-f0[0], f1[1]-f0[1]]
            f12 = [f2[0]-f1[0], f2[1]-f1[1]]
            c = cosine_between_vectors(f01,f12)
            if c < c_min:
                c_min = c
                i_min = i
        t0, t3 = ti[i_min-1], ti[i_min+1]
        if repeat <= 0:
            return (t0+t3)/2
        else:
            return find_vertex_param(f, t0, t3, repeat-1)
    f = pcurve.function
    lot, hit = pcurve.lo, pcurve.hi
    try:
        fstept = (hit-lot)/(npoints-1)
    except ZeroDivisionError:
        fstept = (hit-lot)/99
    rawp = [lot]
    t = lot + fstept
    while t < hit:
        rawp.append(t)
        t = t + fstept
    rawp.append(hit)
    if simple:
        p = []
        for rp in rawp:
            p.append(raw2screen(f(rp), scoordsettings))
        return p
    t0,t1,t2,t3 = rawp[:4]
    bend = []
    for i in range(len(rawp)-4):
        v0,v1,v2,v3 = f(t0),f(t1),f(t2),f(t3)
        v01 = [v1[0]-v0[0], v1[1]-v0[1]]
        v23 = [v3[0]-v2[0], v3[1]-v2[1]]
        dotprod = v01[0]*v23[0] + v01[1]*v23[1]
        crossprod = -v01[1]*v23[0] + v01[0]*v23[1]
        if dotprod < 0:
            found_bend = True
        else:
            #found_bend = False
            found_bend = (5*abs(dotprod) < abs(crossprod)) # 78.69 degrees
            #found_bend = (3*abs(dotprod) < abs(crossprod)) # 71.565 degrees
            #found_bend = (2*abs(dotprod) < abs(crossprod)) # 63.4 degrees
            #found_bend = (abs(dotprod) < abs(crossprod)) # 45 degrees
            #found_bend = (abs(dotprod) < 1.73*abs(crossprod)) # 30 degrees
        if found_bend:
            bend += [t1,t2]
        t0,t1,t2,t3 = t1,t2,t3,rawp[i+4]
    reduce_bend = []
    try:
        reduce_bend = [bend[0]]
        for i in range(1,len(bend)-1,2):
            a,b = bend[i], bend[i+1]
            if a < b:
                reduce_bend += [a,b]
            else: # Combines intervals with common end point.
                pass    
        reduce_bend.append(bend[-1])
        vertex_params = []
        for i in range(0,len(reduce_bend),2):
            t0, t3 = reduce_bend[i], reduce_bend[i+1]
            vparam = find_vertex_param(f, t0, t3)
            vertex_params.append(vparam)
        rawp_plus_bend = rawp + vertex_params
        rawp_plus_bend.sort()
    except IndexError:
        rawp_plus_bend = rawp
    plus_bb = rawp_plus_bend + pcurve.bounding_params
    plus_bb.sort()
    duplicate_limit = (hit - lot)/1000 # arbitrary choice
    minus_duplicates = average_close_floats(plus_bb, duplicate_limit)
    p = []
    for rp in minus_duplicates:
        p.append(raw2screen(f(rp), scoordsettings))
    return p


#==============================================================
#==============================================================
#                         Gimp draw
#==============================================================
#==============================================================

# -----------------------------
# gimp_bezier_to_vectors_object
# -----------------------------
# The Gimp version of the routine 'bezier_to_drawing_format':
# Convert an internally computed Bezier curve (in raw coordinates)
# into Gimp-specific format for drawing purpose,
# which means a Gimp vectors object in screen coordinates.
#
# Args:
# - image:          Gimp's image (obtained usually from GUI);
# - bezier_curve:   BCurve.BezierCurve;
# - scoordsettings: ScreenCoordinateSettings;
# - closed:         Boolean, tells if the curve should be closed
#                  (default=False);
# - pathname:       a string, the name of the curve to be used in the
#                   drawing application (default='bezier')
# Returns: gimp.Vectors
#
# Note: The object is only built here, and not yet inserted into Gimp.
def gimp_bezier_to_vectors_object(image, bezier_curve, scoordsettings,
                           closed = False, pathname = None):
    def cp_raw2cp_screen(cp, scoordsettings): # ControlPoint to screen coords
        origo = scoordsettings.origo
        scalex = scoordsettings.scalex
        scaley = scoordsettings.scaley
        x = origo[0] + scalex*cp[0]
        y = origo[1] + scaley*cp[1]
        return BCurve.ControlPoint(x,y)
    if pathname is None:
        pathname = bezier_curve.curve_name
    gs = bezier_curve.bc2gs() # GimpStroke
    cp_list = gs.cp_list      # [ControlPoint]
    screen_cp_list = [cp_raw2cp_screen(cp, scoordsettings) for cp in cp_list]
    screen_gs = BCurve.GimpStroke(stroke_name = gs.stroke_name,
                                  cp_list = screen_cp_list,
                                  closed = gs.closed
                                  )
    gimp_vectors = BCurve.GimpVectors(stroke_list = [screen_gs],
                                      name = 'GimpVectors'
                                      )              # BCurve.GimpVectors
    vectors_object = gimp_vectors.gv2vectors_object(image, # gimp.Vectors
                                                    pathname)
    return vectors_object

# --------------------------
# gimp_merge_vectors_objects
# --------------------------
# Combine into a single gimp.Vectors object the strokes of a list of
# gimp.Vectors objects.
#
# Args:
# - vectors_objects_list: [gimp.Vectors].
# Returns: gimp.Vectors.
#
# Note: The returned vectors object is only created and not yet inserted
# in Gimp.
def gimp_merge_vectors_objects(image, vectors_objects_list, new_name=None):
    if new_name is not None:
        merged_name = new_name
    else:
        names = [v.name.replace('Bezier: ','') for v in vectors_objects_list]
        joinnames = ', '.join(names)
        merged_name = 'merged Bezier: '+joinnames
    merged = pdb.gimp_vectors_new(image, merged_name)
    for vectors in vectors_objects_list:
        vs = len(vectors.strokes)
        for stroke in vectors.strokes:
            points = stroke.points
            stroke_id = pdb.gimp_vectors_stroke_new_from_points(
                       merged, 0, len(points[0]), points[0], points[1])
    return merged

# ------------------------
# gimp_draw_vectors_object
# ------------------------
# The Gimp version of the routine 'drawing_routine_for_bezier':
# Insert into Gimp the input vectors object, and make it visible if
# visible=True.
#
# Args:
# - image:          Gimp's image (obtained usually from GUI);
# - vectors_object: gimp.Vectors;
# - visible:        Boolean.
#
# Returns:
# - gimp.Vectors
def gimp_draw_vectors_object(image, vectors_object, visible=True):
    # Insert a Gimp vectors object in Gimp.
    # Adapted from http://registry.gimp.org/files/lqr_wpset-0.14.py.txt
    # by Mike Kazantsev.
    def gimp_insert_vectors(image, vectors_object, position=0):
        if gimp.version >= (2, 8, 0):
            # 2.7.0 has gimp-image-insert-vectors, but fails with
            # parent=None
            parent = None
            return pdb.gimp_image_insert_vectors(
                     image, vectors_object, parent, position)
        else:
            return pdb.gimp_image_add_vectors(
                     image, vectors_object, position)# Deprecated!
    gimp_insert_vectors(image, vectors_object)
    vectors_object.visible = visible
    return vectors_object

# -----------------------
# gimp_do_pcurve_drawings
# -----------------------
# The Gimp version of the routine 'do_pcurve_drawings':
# Do the actual drawings in the drawing application: Draw, or do not draw,
# according to the 'drawingoptions':
# - the approximating Bezier curve for the 'pcurve';
# - the exact pcurve;
# - the handles.
#
# Args:
# - image:           Gimp's image (obtained usually from GUI);
# - pcurve:          ParametricCurve;
# - bezier_curve:    BCurve.BezierCurve;
# - scoordsettings:  ScreenCoordinateSettings;
# - drawingoptions:  DrawingOptions;
#
# Returns: gimp.Vectors representing the approximating Bezier curve.
#
# Notes: 1. In the text of the function, the only explicitly
#           Gimp-specific parts is the argument 'image' appearing in
#           several points.
#        2. If c2bezieroptions.bezier_curves is False, the Bezier curve
#           is not drawn, i.e., not inserted to Gimp. However, the
#           vectors object is created and returned.
def gimp_do_pcurve_drawings(image,
                            pcurve,
                            bezier_curve,
                            scoordsettings,
                            drawingoptions,
                            ):
    try:
        bezier_vectors_object = bezier_to_drawing_format(
            image,
            bezier_curve,
            scoordsettings, 
            pcurve.closed,
            pathname = "Bezier: "+pcurve.pcurve_name
            )
        draw_pcurve   = drawingoptions.exact_pcurves
        draw_bezier   = drawingoptions.bezier_curves
        draw_handles  = drawingoptions.handles
        if draw_pcurve:
            drawing_routine_for_curve(image, pcurve, scoordsettings)
        if draw_handles:
            fname = pcurve.pcurve_name
            drawing_routine_for_handles(image, bezier_vectors_object,
                                        fname, scoordsettings)
        if draw_bezier:
            drawing_routine_for_bezier(image, bezier_vectors_object)
    except ParametricCurveError as e:
        mess = "gimp_bezier_from_function\n    "+str(e)
        raise ParametricCurveError(mess)
    return bezier_vectors_object

# ----------------------
# gimp_draw_curve_points
# ----------------------
# The Gimp version of the routine 'drawing_routine_for_curve':
# Draw in Gimp a sparse set of points on the curve as small stars.
# ("Draw" means: build the object, insert in Gimp, and make visible.)
#
# Args:
# - image:          Gimp's image (obtained usually from GUI);
# - pcurve:         ParametricCurve;
# - scoordsettings: ScreenCoordinateSettings.
#
# Returns: gimp.Vectors representing a sparse set of points on the
#          curve as small stars.
def gimp_draw_curve_points(image, pcurve, scoordsettings): 
    def draw_star(point, ray_length = 5.):
        r = ray_length
        r2 = r/sqrt(2.)
        line_W_E = [[point[0]-r, point[1]], [point[0]+r, point[1]]]
        line_S_N = [[point[0], point[1]-r], [point[0], point[1]+r]]
        line_SW_NE = [[point[0]-r2, point[1]-r2], [point[0]+r2, point[1]+r2]]
        line_NW_SE = [[point[0]+r2, point[1]-r2], [point[0]-r2, point[1]+r2]]
        return [line_W_E, line_S_N, line_SW_NE, line_NW_SE]
    points = sparse_points_on_curve(pcurve, scoordsettings, npoints=100)
    if len(points) == 0:
        return []
    stars = pdb.gimp_vectors_new(image, "exact points: "+pcurve.pcurve_name)
    for p in points:
        for start, end in draw_star(p):
            stroke_id = pdb.gimp_vectors_stroke_new_from_points(
                                stars, 0, 12, start*3 + end*3, False)
    gimp_draw_vectors_object(image, stars, visible=True)
    return stars

# --------------
# gimp_draw_axes
# --------------
# The Gimp version of the routine 'drawing_routine_for_axes'.
#
# Draw in Gimp the coordinate axes, both, and/or xaxis, and/or y axis,
# Currently just creates axes as guides (only inside window).
#
# Args:
# - image:          Gimp's image (obtained usually from GUI);
# - scoordsettings: ScreenCoordinateSettings;
# - draw_both, draw_x, draw_y: Boolean: which of the axes to draw.
#
# Returns currently nothing.
def gimp_draw_axes(image, scoordsettings, draw_both, draw_x, draw_y):
    origo = scoordsettings.origo
    # Currently creates guides:
    if draw_both or draw_x:
        try:
            x_axis = pdb.gimp_image_add_hguide(image, origo[1])
        except RuntimeError: # outside of window
            pass
    if draw_both or draw_y:
        try:
            y_axis = pdb.gimp_image_add_vguide(image, origo[0])
        except RuntimeError: # outside window
            pass
    return # Currently returns nothing

# -----------------
# gimp_draw_handles
# -----------------
# The Gimp version of the routine 'drawing_routine_for_handles'.
# Draw in Gimp the handles of the input gimp-Vectors object.
# ("Draw" means: build a separate gimp.Vectors object, insert in Gimp,
# and make visible.)
#
# Args:
# - image:          gimp.Image (obtained usually from GUI);
# - vectors_object: gimp.Vectors;
# - fname:          the name of the function (only to be attached to the
#                   name of the handles path);
# - scoordsettings: ScreenCoordinateSettings;
# - circle_diameter, handle_box_size: float numbers to manage the sizes
#          of the details of the handle drawing (default=None).
#
# Returns gimp.Vectors representing the anchors and handles.
def gimp_draw_handles(image, vectors_object, fname, scoordsettings,
                       circle_diameter=None, handle_box_size=None):
    def draw_box(center, size, direction):
        half = size/2.
        ll = sqrt(direction[0]**2+direction[1]**2)
        try:
            dir = [ direction[0]/ll, direction[1]/ll ]
        except ZeroDivisionError:
            dir = [1.,0.]
        start = [ center[0]-half*dir[0], center[1]-half*dir[1] ]
        boxNE = [ start[0]+half*dir[1], start[1]-half*dir[0] ]
        boxNW = [ boxNE[0]+size*dir[0], boxNE[1]+size*dir[1] ]
        boxSW = [ boxNW[0]-size*dir[1], boxNW[1]+size*dir[0] ]
        boxSE = [ boxSW[0]-size*dir[0], boxSW[1]-size*dir[1] ]
        box = boxNE*3 + boxNW*3 + boxSW*3 + boxSE*3
        return box
    def draw_loop(center, diam, direction):
        hch = 1.31 # Handle length for half circle
        radius = diam/2
        lendir = sqrt( direction[0]**2 + direction[1]**2 )
        try:
            radiusvector = [radius*direction[0]/lendir, radius*direction[1]/lendir]
        except ZeroDivisionError:
            radiusvector = [0., radius]
        handlevector = [-hch*radiusvector[1], hch*radiusvector[0]]
        anchor1 = [center[0] + radiusvector[0], center[1] + radiusvector[1]]
        anchor2 = [center[0] - radiusvector[0], center[1] - radiusvector[1]]
        handle_first = [anchor1[0] - handlevector[0], anchor1[1] - handlevector[1]]
        handle1 = [anchor1[0] + handlevector[0], anchor1[1] + handlevector[1]]
        handle2 = [anchor2[0] + handlevector[0], anchor2[1] + handlevector[1]]
        handle_last = [anchor2[0] - handlevector[0], anchor2[1] - handlevector[1]]
        loop = handle_first + anchor1 + handle1 + handle2 + anchor2 + handle_last
        return loop
    gxmin,gymin = scoordsettings.padded_SW
    gxmax,gymax = scoordsettings.padded_NE
    width,height = abs(gxmax-gxmin),abs(gymin-gymax)
    w = width/200.
    h = height/200.
    if circle_diameter is None:
        #circle_diameter = 10 # might be good
        circle_diameter = max(w,h)
    if handle_box_size is None:
        #handle_box_size = 10 # might be good
        handle_box_size = max(w,h)
    #
    handles = pdb.gimp_vectors_new(image, "handles: "+fname)
    for str in vectors_object.strokes:
        p = str.points[0]
        j = 0
        for i in range(0,len(p),6):
            j += 1
            ppp = p[i:i+2]*3 + p[i+2:i+4]*3 + p[i+4:i+6]*3
            pdb.gimp_vectors_stroke_new_from_points(
                        handles, 0, len(ppp), ppp, False)
    for str in handles.strokes:
        hp = str.points[0]
        leftend = hp[2:4]
        middle = hp[8:10]
        rightend = hp[14:16]
        dirleft  = [ leftend[0]-middle[0] , leftend[1]-middle[1] ]
        dirright = [ rightend[0]-middle[0] , rightend[1]-middle[1] ]
        leftlen2  = (leftend[0]-middle[0])**2 + (leftend[1]-middle[1])**2
        rightlen2 = (rightend[0]-middle[0])**2 + (rightend[1]-middle[1])**2
        leftlen  = max(abs(leftend[0]-middle[0]), abs(leftend[1]-middle[1]))
        rightlen = max(abs(rightend[0]-middle[0]), abs(rightend[1]-middle[1]))
        box_size = handle_box_size
        while box_size > leftlen: # Smaller boxe when handle very short.
            box_size /= 2
        lbox = draw_box(leftend, box_size, dirleft)
        pdb.gimp_vectors_stroke_new_from_points(
                        handles, 0, len(lbox), lbox, True)
        box_size = handle_box_size
        while box_size > rightlen: # Smaller boxe when handle very short.
            box_size /= 2
        rbox = draw_box(rightend, box_size, dirright)
        pdb.gimp_vectors_stroke_new_from_points(
                        handles, 0, len(rbox), rbox, True)
        dire = [ rightend[0]-leftend[0], rightend[1]-leftend[1] ]
        circle_size = circle_diameter
        while circle_size > max(leftlen,rightlen): # Smaller anchor when handle very short.
            circle_size /= 2
        loop = draw_loop(middle, circle_size, dire)
        pdb.gimp_vectors_stroke_new_from_points(
                        handles, 0, len(loop), loop, True)
    gimp_draw_vectors_object(image, handles, visible=True)
    return handles

# -------------------
# gimp_draw_polylines
# -------------------
# Draw list of polylines in one Gimp vectors object, one polyline in each
# stroke.
# Args:
# - image: gimp.Image;
# - scoordsettings: ScreenCoordinateSettings;
# - polyline_list = a list of items of either of the following two forms:
#   - [[x0,y0],[x1,y1],...,[xn,yn]]
#   - [[x0,y0],[x1,y1],...,[xn,yn], closed]
#   where xi,yi:float and closed = False or True;
# - name:  string;
# Returns:
# - gimp.Vectors
# Example: Given points p0,p1,p2,p3,p4,p5, each of type [float,float], then
# commands
#     poly0 = [p0]
#     poly1 = [p1, p2]
#     poly2 = [p3, p4, p5, True]
#     poly_list = [poly0, poly1, poly2]
#     gimp_draw_polylines(image, poly_list)
# will draw a vectors object with three strokes:
# - one point p0;
# - line segment from p1 to p2;
# - closed triangle with vertices p3,p4,p5.
def gimp_draw_polylines(image,
                        scoordsettings,
                        polyline_list,
                        name='polylines',
                        ):
    origo = scoordsettings.origo
    scalex = scoordsettings.scalex
    scaley = scoordsettings.scaley
    vectors = pdb.gimp_vectors_new(image, name)
    for polyline in polyline_list:
        try:
            if type(polyline[-1]) is bool:
                closed = polyline[-1]
                poly = polyline[:-1]
            else:
                closed = False
                poly = polyline
        except IndexError: # empty
            continue
        poly_scaled = [[origo[0] + scalex*p[0], origo[1] + scaley*p[1]]
                              for p in poly]
        points = []
        for xy in poly_scaled:
            points += 3*xy
        stroke_id = pdb.gimp_vectors_stroke_new_from_points(
                            vectors, 0, len(points), points, closed)
    pdb.gimp_image_undo_group_start(image) ### Gimp
    gimp_draw_vectors_object(image, vectors, visible=True)
    pdb.gimp_image_undo_group_end(image) ### Gimp
    return vectors


#==============================================================
#==============================================================
#                        Handling inputs
#==============================================================
#==============================================================

# -------------------------
# exec_get_inputs_from_file
# -------------------------
# Read inputs from a python file. Accept only those new names which appear
# in the tuple 'names_allowed' in the code.
# Args:
# - file;
# Returns:
# - directory with some of the keys in the 'names_allowed' in the code,
#   but perhaps not all of them.
def exec_get_inputs_from_file(file):
    glob_d = globals()
    exec_d = glob_d.copy()
    with open(file) as f:
        code = compile(f.read(), f.name, 'exec')
        exec(code, exec_d)
    # Remove from exec_d keys in glob_d => names defined in file
    file_d = dict()
    for key in exec_d:
        if key not in glob_d:
            file_d[key] = exec_d[key]
    # Remove keys not in names_allowed:
    stripped = dict()
    for key in file_d:
        if key in names_allowed:
            stripped[key] = file_d[key]
    return stripped

# Allowed names read from file or processed from GUI:
names_allowed = (
                 'pcurve_collage',
                 'c2bs',
                 'c2bo',
                 'coos',
                 'drawo',
                 'mark_polylines',    # not in GUI
                 )

# Default values for names
names_default_values = {
                        'pcurve_collage': ParametricCurveCollage(
                                             [ParametricCurve(
                                               function=lambda t:[cos(t),sin(t)],
                                               pcurve_name='Half circle',
                                               start=0.,
                                               end=pi,
                                               closed=False,
                                               custom=None
                                               )
                                             ]
                                            ),
                        'c2bs':           Curve2BezierSettings(),
                        'c2bo':           Curve2BezierOptions(),
                        'coos':           CoordinateSettings(),
                        'drawo':          DrawingOptions(),
                        'mark_polylines': None,
                        }

# ------------
# get_dict_GUI
# ------------
# Given inputs from the Gimp GUI for the plugin (received in a dict),
# convert them to the form applicable in the plugin.
# Return the results in a dict.
# Args:
# - dict_read_from_GUI: dict: input arguments from the Gimp GUI
# - messages:           boolean.
# Returns:
# - dict
def get_dict_GUI(dict_read_from_GUI, messages):
    # get_anchors: Read the anchor points as a list from a Gimp's
    # vector object.
    def get_anchors(vectors_object): ### Gimp
        y=[]
        for str in vectors_object.strokes:
            points = str.points[0]
            n = len(points)
            for i in range(0,n,6):
                y.append(points[i+2:i+4])
        return y
    dict_GUI = dict()
    xy_strings = [dict_read_from_GUI['x_function'],
                  dict_read_from_GUI['y_function'],
                 ]
    sx, sy = [w.strip() for w in xy_strings] # remove blanks
    if (sx == '') or (sy == ''):
        raise Exception("Empty x(t) or y(t) in the GUI")
    else: # From the strings sx, sy construct function f(t): R -> R2
        try:
            def f_GUI(t):  # f(t):R -> R2
                return [eval(sx), eval(sy)]
        except SyntaxError as e:
            m = "Syntax error in x(t) or y(t) in the GUI"
            if messages:
                gimp_message(m)
            raise
    s0 = dict_read_from_GUI['start_as_string'] # strings
    s1 = dict_read_from_GUI['end_as_string']
    start, end = float(eval(s0)), float(eval(s1))
    closed = dict_read_from_GUI['closed']
    custom_params = []
    param_string = dict_read_from_GUI['param_string']
    #print("param_string="+param_string)
    if param_string.strip() == '': # blank
        custom_params = []
    else:
        try:
            list_or_tuple_params = eval(param_string)
            if type(list_or_tuple_params) in (int,float):
                list_or_tuple_params = [float(list_or_tuple_params)]
        except:
            if messages:
                gimp_message(("Something wrong in custom parameter values."
                              +"\nPlease check syntax and so on."))
            raise
        custom_params = [float(p) for p in list_or_tuple_params]
    #print("custom_params="+str(custom_params))
    dict_GUI['custom_params'] = custom_params
    custom = CustomPoints(params=custom_params)
    pcurve_name = dict_read_from_GUI['curve_name']
    pcurve = ParametricCurve(f_GUI,
                             pcurve_name,
                             start,
                             end,
                             closed,
                             custom
                             )
    # When the inputs are taken form the GUI, the pcurve_collage will
    # contain only one pcurve:
    pcurve_collage = ParametricCurveCollage([pcurve], pcurve_name)
    dict_GUI['pcurve_collage'] = pcurve_collage
    #
    dict_GUI['c2bs'] = Curve2BezierSettings()
    dict_GUI['c2bo'] = Curve2BezierOptions()
    dict_GUI['coos'] = CoordinateSettings(
                           fit_window = dict_read_from_GUI['fit_window'],
                           padding    = 0.,
                           origo      = [dict_read_from_GUI['origo_x'],
                                         dict_read_from_GUI['origo_y']],
                           scale      = dict_read_from_GUI['scale']
                           )
    dict_GUI['drawo'] = DrawingOptions(axes = dict_read_from_GUI['draw_axes'])
    dict_GUI['mark_polylines'] = None
    return dict_GUI

# -------------
# get_dict_file
# -------------
# Read inputs from file and convert them to the form applicable in
# bezier_curve_from_dict_inputs (means resolfing 'function').
# Return the results in a dict.
# The names undefined in file are set to default values.
# Args:
# - input_file: file
# - messages:   boolean.
# Returns:
# - dict
def get_dict_file(input_file, display_messages):
    dict_file = exec_get_inputs_from_file(input_file)#, messages=messages)
    # Remove from dict_file keys not in names_allowed:
    stripped = dict()
    for key in dict_file:
        if key in names_allowed:
            stripped[key] = dict_file[key]
    dict_file = stripped
    if 'mark_polylines' not in dict_file:
        dict_file['mark_polylines'] = names_default_values['mark_polylines']
    names_missing_in_file = [name for name in names_allowed if name not in dict_file]
    for missing in names_missing_in_file:
        dict_file[missing] = names_default_values[missing]
    # Little test for validity of functions:
    try:
        for pcurve in dict_file['pcurve_collage'].pcurve_list:
            f = pcurve.function
            a,b = pcurve.lo, pcurve.hi
            try:
                fa = f(a)
                fhalf = f((a+b)/2)
                fb = f(b)
            except TypeError as e:
                if display_messages:
                    gimp_message(e)
                raise Exception(e)
            if type(fhalf) is not list:
                if display_messages:
                    gimp_message(e)
                raise Exception(e)
    except:
        raise
    return dict_file

# ------------------------------
# gimp_get_screen_coordsettings
# ------------------------------
# The Gimp version of the routine 'get_screen_coordsettings':
# From coordinate settings build screen coordinate settings.
#
# Args:
# - image:         gimp.Image (obtained usually from GUI);
# - pcurve:        ParametricCurve;
# - bounding_box:  BoundingBox: the required size of
#                  the plot (used if fit_window is True);
# - coordsettings: CoordinateSettings.
#
# Returns: ScreenCoordinateSettings.
def gimp_get_screen_coordsettings(
            image,
            bounding_box,
            coordsettings
            ):
    fit_window = coordsettings.fit_window
    padding = coordsettings.padding # Fraction of window sizes
    gw_height = pdb.gimp_image_height(image) # width and heigth of
    gw_width = pdb.gimp_image_width(image)   # Gimp window
    gw_SW = [0., gw_height] # The SW and NE corners
    gw_NE = [gw_width, 0.]  # of the Gimp window in screen coordinates.
    bgw_SW = [gw_width*padding, gw_height*(1.-padding)] # The SW and NE corners
    bgw_NE = [gw_width*(1.-padding), gw_height*padding] # of padded window.
    bgw_height = gw_height*(1.- 2*padding)
    bgw_width = gw_width*(1.- 2*padding)
    if not fit_window: # fit_window is False: use input origo and scale.
        origo = coordsettings.origo
        scale = coordsettings.scale
    else: # fit_window is True: ignore input origo and scale, compute new.
        fxmin = bounding_box.xmin
        fxmax = bounding_box.xmax
        fymin = bounding_box.ymin
        fymax = bounding_box.ymax
        f_width, f_height = fxmax-fxmin, fymax-fymin
        try:
            scale_x = bgw_width / f_width  # > 0
            scale_y = bgw_height / f_height  # > 0
            scale = min(scale_x, scale_y)
        except ZeroDivisionError: # One of f_width, f_height is 0
            #print("scale_x or scale_y failed")
            try:
                scale_x = bgw_width / f_width  
                scale_y = scale_x + 1 # Means infinity
                scale = scale_x    # width ok
            except ZeroDivisionError:
                try:
                    scale_y = bgw_height / f_height  # > 0
                    scale_x = scale_y + 1 # Means infinity
                    scale = scale_y    # height ok
                except ZeroDivisionError: # None is ok
                    mess = "Bounding box is of zero size: automatic scaling failed"
                    #print(mess)
                    error_message_function(mess)
                    raise ParametricCurveError(mess)
        # Get the offset of the origo compared with the position
        # where the plot is flush at SW corner of the whole window:
        if scale_x > scale_y: # Plot is tall
            x_offset = (bgw_width - scale*f_width) / 2.
            y_offset = 0.
        else:                 # Plot is wide
            x_offset = 0.
            y_offset = (bgw_height - scale*f_height) / 2.
        # Compute the origo of the plot for the position flush at the
        # SW corner of the padded window:
        origo_SW = [bgw_SW[0] - scale*fxmin, bgw_SW[1] + scale*fymin] # Gimp
        # Offsets:
        origo = [origo_SW[0] + x_offset, origo_SW[1] - y_offset]
    dgc = ScreenCoordinateSettings(
                origo,
                scale,
                -scale, # In Gimp's window y coordinate runs downwards.
                gw_SW,
                gw_NE,
                bgw_SW,
                bgw_NE,
                )
    return dgc


#==============================================================
#==============================================================
#                        Main functions
#==============================================================
#==============================================================

# --------------------------
# gimp_parametric_curve_main
# --------------------------
# Handle inputs from Gimp's GUI and call the the function
# 'gimp_bezier_from_collage' which is the "true" main function.
# Do all drawings and return:
# 1. If inputs are taken from GUI, return the Gimp vectors object
#    (gimp.Vectors) representing the approximate Bezier curve of the input
#    parametric curve f(t)=(x(t),y(t)).
# 2. If inputs are taken from a file, return the Gimp vectors object
#    (gimp.Vectors) which is the combined version of the results:
#    each stroke will be the approximate Bezier curve of one of of the
#    parametric curves in the input pcurve_collage:ParametricCurveCollage.
#
# Args (read from GUI):
# - image:      Gimp's image (obtained usually from GUI);
# - curve_name: string, the name of the curve to appear in the paths
#               tab (default="Half circle");
# - x_function: string, the x coordinate function x(t) of the
#               parametric curve f(t)=(x(t),y(t)), in Python syntax
#               (default="cos(t)");
# - y_function: string, the y coordinate function y(t) of the
#               parametric curve f(t)=(x(t),y(t)), in Python syntax
#               (default="sin(t)");
# - start_as_string: string, starting value for the parameter in Python
#                    syntax
#                    (default="0.");
# - end_as_string:   string, ending value for the parameter in Python
#                    syntax
#                    (default="pi");
# - closed:     Boolean, tells if the curve should be closed (default =
#               False);
# - fit_window: Boolean, tells if the plot should be fit in the window
#               (default=True);
# - origo_x:    float number, the x coordinate of the position in the
#               window where the origo of the plot should be
#               (default=500.) 
# - origo_y:    float number, the y coordinate of the position in the
#               window where the origo of the plot should be
#               (default=500.) 
# - scale:      float number, the scale factor from the coordinates
#               used in the input curve to the coordinates on the
#               window (default=100.);
# - draw_bezier:  Boolean, tells if the computed Bezier curve should
#                 also be drawn, i.e., created as gimp.Vectors object
#                 (default=True);
# - draw_axes:    Boolean, tells if the coordinate axes should
#                 also be drawn, i.e., created as a separate gimp.Vectors
#                 object (default=True);
# - read_custom_path: Boolean, tells if custom screen_points should be read
#                     from anchors of custom_path
#                     (default=False);
# - custom_path:      gimp-Vectors, containing the user input for
#                     custom points as anchors
#                     (default=None);
# - read_file:  Boolean, tells if advanced inputs shpuld be read from a file
#               (default=False);
# - input_file: the file containing advanced input if any
#               (default=None).;
# - display_messages: Boolean, tells if various messages are to be displayed
#                     in the error console.
#
# Returns:
# - gimp.Vectors
def gimp_parametric_curve_main(image,             # gimp.Image
                              curve_name,        # string
                              x_function,        # string, in Python syntax
                              y_function,        # string, in Python syntax
                              start_as_string,   # string, in Python syntax
                              end_as_string,     # string, in Python syntax
                              #################  KOE
                              param_string,      # string
                              closed,            # boolean
                              fit_window,        # boolean
                              origo_x,           # float
                              origo_y,           # float
                              scale,             # float
                              draw_axes,         # boolean
                              #read_custom_path,  # boolean  #KOE
                              #custom_path,       # path?    #KOE
                              read_file,         # boolean
                              input_file,        # file?
                              work_level,        # integer
                              display_messages   # boolean
                             ):
    dict_read_from_GUI = {'curve_name':        curve_name,
                          'x_function':        x_function,
                          'y_function':        y_function,
                          'start_as_string':   start_as_string,
                          'end_as_string':     end_as_string,
                          ##################  KOE
                          'param_string':    param_string,
                          'closed':            closed,
                          'fit_window':        fit_window,
                          'origo_x':           origo_x,
                          'origo_y':           origo_y,
                          'scale':             scale,
                          'draw_axes':         draw_axes,
                          #'read_custom_path':  read_custom_path,   #KOE
                          #'custom_path':       custom_path,       #KOE
                          'read_file':         read_file,
                          'input_file':        input_file,
                          'work_level':        work_level,
                          'display_messages':  display_messages
                         }
    if read_file:
        dict_inputs = get_dict_file(input_file, display_messages)
    else:
        dict_inputs = get_dict_GUI(dict_read_from_GUI, display_messages)
    work_level = min(100,max(1,work_level))
    dict_inputs['c2bs'].max_error /= work_level
    results = gimp_bezier_from_collage(image,
                                       dict_inputs,
                                       display_messages)
    vectors_objects, vectors_merged, polylines, rel_error = results
    return vectors_merged

# ------------------------
# gimp_bezier_from_collage
# ------------------------
# Called by the main function. The "true" main function.
# Create the approximating Bezier curves and do all drawings.
#
# Args:
# - image:       gimp.Image (obtained usually from GUI);
# - dict_inputs: inputs in the form of a dict (keys in the list names_allowed).
# Returns:
# - [gimp.Vectors]: list of Gimp vectors_objects, each representing the
#                   approximate Bezier curve of one of the input parametric
#                   curves;
# - gimp.Vectors:   the merged object of the list above,
#                   one Bezier curve as each stroke;
# - gimp.Vectors:   None or Gimp vectors_objects representing the polylines;
# - float:          an error measure.
#
# Notes:
# 1. This function 'gimp_bezier_from_collage' is the "true" main function,
#    called by the main function 'gimp_parametric_curve_main'.
# 2. If dict_inputs['c2bs']:Curve2BezierSettings has attribute 
#    'bezier_curves' set to False, the Bezier curves are not drawn, i.e.,
#    not inserted to Gimp, but they are created as vectors objects and
#    returned in the list.
# 3. If dict_inputs['mark_poylines'] is None, the returned polylines
#    will be None.
def gimp_bezier_from_collage(image,
                             dict_inputs,
                             messages=False,
                             ):
    pcurve_collage   = dict_inputs['pcurve_collage']
    c2beziersettings = dict_inputs['c2bs']
    c2bezieroptions  = dict_inputs['c2bo']
    coordsettings    = dict_inputs['coos']
    draw_options     = dict_inputs['drawo']
    mark_polylines   = dict_inputs['mark_polylines']
    #
    pdb.gimp_image_undo_group_start(image) ### Gimp
    max_rel_error = 0.
    try:
        bb = pcurve_collage.bounding_box
        if bb is None:
            pcurve_collage.compute_bounding_box()
            bb = pcurve_collage.bounding_box
        scoordsettings = get_screen_coordsettings(
            image,
            bb,
            coordsettings
            )
        vectors_objects = []
        pcurve_count = 0
        for pcurve in pcurve_collage.pcurve_list:
            # Main call
            results = bezier_curve_from_pcurve(
                pcurve,
                c2beziersettings,
                c2bezieroptions,
                scoordsettings,
                messages,
                )
            pcurve_prepared, bezier_curve, rel_error = results
            # Draw:
            # - the approximating Bezier curve for the 'pcurve'
            # - the exact pcurve
            # - the handles
            # according to 'drawing options':
            vectors_object = do_pcurve_drawings(
                image,
                pcurve_prepared,
                bezier_curve,
                scoordsettings,
                draw_options,
                )
            vectors_objects.append(vectors_object)
            max_rel_error = max(max_rel_error, rel_error)
            pcurve_count += 1
            if messages:
                s = ("Plugin finished successfully"
                     +"\n"
                     +pcurve.pcurve_name
                     +"\nwith accuracy "
                     + str_float(rel_error)+".")
                error_message_function(s)
        # Merge vectors objects:
        vectors_merged = merge_bezier_objects(
            image,
            vectors_objects,
            pcurve_collage.collage_name
            )
        # Draw the merged vectors object:
        if draw_options.bezier_collage:
            gimp_draw_vectors_object(image, vectors_merged, visible=True)
    except ParametricCurveError as e:
        mess = "gimp_bezier_from_collage\n    "+str(e)
        mess = "The plugin was not able to handle this case.\n"+mess
        mess = "Please re-consider your inputs.\n\n"+mess
        error_message_function(mess)
        pdb.gimp_image_undo_group_end(image) ### Gimp
        raise ParametricCurveError(mess)
    except Exception as e:
        #error_message_function(str(e.args[0]))
        error_message_function(str(e))
        pdb.gimp_image_undo_group_end(image) ### Gimp
        raise
    drawing_routine_for_axes(image,                    # Returns nothing
                             scoordsettings,
                             draw_options.axes,
                             draw_options.x_axis,
                             draw_options.y_axis,
                             )
    if mark_polylines is not None:
        name  = 'polylines'
        polylines = drawing_routine_for_polylines(image,
                                                  scoordsettings,
                                                  mark_polylines,
                                                  name=name,
                                                  )
    else:
        polylines = None
    pdb.gimp_image_undo_group_end(image) ### Gimp
    return vectors_objects, vectors_merged, polylines, max_rel_error


############################################
##                                        ##
##   Bindings of some function names      ##
##                                        ##
############################################


bezier_to_drawing_format       = gimp_bezier_to_vectors_object
get_screen_coordsettings       = gimp_get_screen_coordsettings
drawing_routine_for_bezier     = gimp_draw_vectors_object
drawing_routine_for_axes       = gimp_draw_axes
drawing_routine_for_polylines  = gimp_draw_polylines
drawing_routine_for_handles    = gimp_draw_handles
drawing_routine_for_curve      = gimp_draw_curve_points
do_pcurve_drawings             = gimp_do_pcurve_drawings
merge_bezier_objects           = gimp_merge_vectors_objects
error_message_function         = gimp_message


#==============================================================
#==============================================================
#=====================  Registration  =========================
#==============================================================
#==============================================================
versionnumber = "1.09"
#

procedure_name  = "parametric_curves"
procedure_blurb = ("Approximate a parametric curve by a Bezier curve"
                   " (Version "+versionnumber+")."
                   "\nExperimental -- please save your work first!"
                   )
procedure_help  = "Draw a parametric curve approximately as a Bezier curve."
procedure_author = "Markku Koppinen"
procedure_copyright = procedure_author
procedure_date = "2020"
procedure_label = "Parametric curves"
image_types = "*"
procedure_function = gimp_parametric_curve_main

menupath = "<Image>/Filters/Render/Parametric curves"

register(
    procedure_name,
    procedure_blurb,
    procedure_help,
    procedure_author,
    procedure_copyright,
    procedure_date,
    procedure_label,
    image_types,
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_STRING, "curve_name", "Name for the curve", "Half circle"),
        (PF_STRING, "x_function", "Function x(t) -- in Python syntax", "cos(t)"),
        (PF_STRING, "y_function", "Function y(t) -- in Python syntax", "sin(t)"),
        (PF_STRING, "start_as_string", "Starting value for t -- in Python syntax", "0."),
        (PF_STRING, "end_as_string", "Ending value for t -- in Python syntax", "pi"),
        (PF_STRING, "param_string", ("Custom values for t to force certain anchors on the curve"
                                     "\n -- list in Python syntax or values separated by commas"), ""),
        (PF_BOOL, "closed", "Is the curve closed?", False),
        (PF_BOOL, "fit_window", "Fit the plot to fill the window?", True),
        (PF_FLOAT, "origo_x", "The x coordinate of the origo (ignored if fit in window)", 500.),
        (PF_FLOAT, "origo_y", "The y coordinate of the origo (ignored if fit in window)", 500.),
        (PF_FLOAT, "scale", "The scaling factor for the plot (ignored if fit in window)", 100.),
        (PF_BOOL, "draw_axes", "Mark the coordinate axes as guides?", False),
        #(PF_BOOL, "read_custom_path", "Read custom points from a path (to force certain anchors on the curve)?", False),
        #(PF_VECTORS, "custom_path", ">>> Choose a path having your custom points as its anchors:", None),
        (PF_BOOL, "read_file", "Read inputs from a file?", False),
        (PF_FILE, "input_file", ">>> Choose a file containing your inputs:", None),
        (PF_INT, "work_level", "Level: 1..100; larger number may mean more control points", 1),
        (PF_BOOL, "display_messages", "Display messages in the error console?", False),
    ],
    [
        (PF_VECTORS, "paramcurve_bezier", "Parametric curve approximated as a Bezier curve"),
    ],
    procedure_function,
    menu=menupath)

main()

